#include <glib.h>
#include "main.hh"

GString *output = g_string_new("");


gboolean vertical_black(unsigned char* framebuffer,gint line,gint width,gint height) {
    /* Return TRUE if line is 100% black, else FALSE */
    gulong pixel;
    gulong base;
    guint8 R,G,B,Average;

    for (pixel=0;pixel<height;pixel++) {
        base = (line+pixel*width)*3;
        B = *(framebuffer+base);
        G = *(framebuffer+base+1);
        R = *(framebuffer+base+2);
        if ((B<60) && (R<60) && (G<60)) {
            /* Black pixel */
        } else {
            return FALSE;
        }
    }
    return TRUE;
}

gboolean horizontal_black(unsigned char* framebuffer,gint line,gint width) {
    /* Return TRUE if line is 100% black, else FALSE */
    gulong pixel;
    gulong base = line*width;
    guint8 R,G,B,Average;

    for (pixel=0;pixel<width;pixel++) {
        B = *(framebuffer+(base+pixel)*3);
        G = *(framebuffer+(base+pixel)*3+1);
        R = *(framebuffer+(base+pixel)*3+2);
        if ((B<60) && (R<60) && (G<60)) {
            /* Black pixel */
        } else {
            return FALSE;
        }
    }
    return TRUE;
}

/* Check amount of clipping for top, bottom, left and right. There're 2 samples
   taken, to reduce to possibility a whole black frame is selected */
gboolean autoclipper(gchar* filename) {
    mpeg3_t* file;
    gint vstrm,vistream,result,i;
    gulong frames,framenr;
    gint cliptop30,clipbottom30,clipleft30,clipright30;
    gint cliptop70,clipbottom70,clipleft70,clipright70;
    gboolean top,bottom,right,left;
    guint width,height;
    guchar *pixel;
    gfloat perc;

    /* Open input MPEG2 stream */
    g_string_sprintf(output,"Opening stream %s for autoclipping",filename);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);
    file = mpeg3_open(filename);
    if (file==0) {
        /* Error, not able to open MPEG2 stream properly */
        g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Unable to open %s, possibly not an MPEG2 or IFO file?",filename);
        return FALSE;
    }
    /* Open video stream */
    vstrm=mpeg3_total_vstreams(file);
    vistream=(vistream>vstrm - 1 ? vstrm - 1 : vistream);
    g_string_sprintf(output,"Set video stream %d of %d", vistream, vstrm);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);
    /* Get number of frames in input stream */
    frames = mpeg3_video_frames(file, vistream);
    /* Parse parameters of stream */
    width=mpeg3_video_width(file, vistream);
    height=mpeg3_video_height(file, vistream);

    /* Allocate buffers */
    unsigned char* framebuffer = new unsigned char[width*height*BYTES_PER_COLOUR_OUTPOUT];
    unsigned char* extrabuffer = new unsigned char[width*BYTES_PER_COLOUR_OUTPOUT+4];
    unsigned char** rowptr = new (unsigned char*)[height];
    for (i=0; i<height; i++) {
        if(i==height-1) {
            rowptr[i]=extrabuffer;
        } else {
            rowptr[i]=framebuffer+(width*(height-1)*BYTES_PER_COLOUR_OUTPOUT)-(width*BYTES_PER_COLOUR_OUTPOUT*i);
        }
    }

    /* DETERMINE TOP\BOTTOM CLIPPING FOR 30% FRAME */
    /* Pick a frame at 30% of stream */
    perc = random();
    perc = perc/RAND_MAX;
    framenr = mpeg3_seek_percentage(file, perc); /* Seek to 10% of stream */
    if(mpeg3_read_frame(file, rowptr, 0, 0, width, height, width, height, LIBMPEG_COLOUR_DEPT_OUPUT, vistream)) {
        log_handler(G_LOG_LEVEL_ERROR,"Autoclipper couldn't open frame");
        return FALSE;
    }
    /* Top\Bottom clipping */
    top = TRUE;bottom = TRUE;
    for (i=0;i<(height/2);i++) {
        /* Check lines */
        if (top==TRUE)    { top = horizontal_black(framebuffer,i,width);}
        if (bottom==TRUE) { bottom = horizontal_black(framebuffer,height-1-i,width);}
        /* Non black line found -> set clipping */
        if (top==FALSE)    { cliptop30 = i+1;top=2;}
        if (bottom==FALSE) { clipbottom30 = i;bottom=2;}
        /* Ready? */
        if ((top==FALSE) && (bottom==FALSE)) {
            /* Found both lines that are not 100% black */
            i = height;
        }
    }
    /* Left\Right clipping */
    left = TRUE;right = TRUE;
    for (i=0;i<(width/2);i++) {
        /* Check lines */
        if (left==TRUE)   { left = vertical_black(framebuffer,i,width,height);}
        if (right==TRUE)  { right = vertical_black(framebuffer,width-1-i,width,height);}
        /* Non black line found -> set clipping */
        if (left==FALSE)  { clipleft30 = i+1;left=2;}
        if (right==FALSE) { clipright30 = i;right=2;}
        /* Ready? */
        if ((left==FALSE) && (right==FALSE)) {
            /* Found both lines that are not 100% black */
            i = width;
        }
    }
    g_string_sprintf(output,"AUTOCLIPPER: clipping 30 perc set to top %i, bottom %i, left %i, right %i line(s)",cliptop30,clipbottom30,clipleft30,clipright30);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);



    /* DETERMINE TOP\BOTTOM CLIPPING FOR 70% FRAME */
    /* Pick a frame at 70% of stream */
    perc = 1-perc;
    framenr = mpeg3_seek_percentage(file, perc);
    if(mpeg3_read_frame(file, rowptr, 0, 0, width, height, width, height, LIBMPEG_COLOUR_DEPT_OUPUT, vistream)) {
        log_handler(G_LOG_LEVEL_ERROR,"Autoclipper couldn't open frame");
        return FALSE;
    }
    /* Top\Bottom clipping */
    top = TRUE;bottom = TRUE;
    for (i=0;i<(height/2);i++) {
        /* Check lines */
        if (top==TRUE)    { top = horizontal_black(framebuffer,i,width);}
        if (bottom==TRUE) { bottom = horizontal_black(framebuffer,height-1-i,width);}
        /* Non black line found -> set clipping */
        if (top==FALSE)    { cliptop70 = i+1;top=2;}
        if (bottom==FALSE) { clipbottom70 = i;bottom=2;}
        /* Ready? */
        if ((top==FALSE) && (bottom==FALSE)) {
            /* Found both lines that are not 100% black */
            i = height;
        }
    }
    /* Left\Right clipping */
    left = TRUE;right = TRUE;
    for (i=0;i<(width/2);i++) {
        /* Check lines */
        if (left==TRUE)   { left = vertical_black(framebuffer,i,width,height);}
        if (right==TRUE)  { right = vertical_black(framebuffer,width-1-i,width,height);}
        /* Non black line found -> set clipping */
        if (left==FALSE)  { clipleft70 = i+1;left=2;}
        if (right==FALSE) { clipright70 = i;right=2;}
        /* Ready? */
        if ((left==FALSE) && (right==FALSE)) {
            /* Found both lines that are not 100% black */
            i = width;
        }
    }
    g_string_sprintf(output,"AUTOCLIPPER: clipping 70 perc set to top %i, bottom %i, left %i, right %i line(s)",cliptop70,clipbottom70,clipleft70,clipright70);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);


    /* Fill in global clip settings */
    if (cliptop30>cliptop70) { cliptop = cliptop70;}
        else { cliptop = cliptop30;}
    if (clipbottom30>clipbottom70) { clipbottom = clipbottom70;}
        else { clipbottom = clipbottom30;}
    if (clipleft30>clipleft70) { clipleft = clipleft70;}
        else { clipleft = clipleft30;}
    if (clipright30>clipright70) { clipright = clipright70;}
        else { clipright = clipright30;}

    g_string_sprintf(output,"Auto clipping top %i,bottom %i,left %i,right %i lines",cliptop,clipbottom,clipleft,clipright);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);


    /* Clean & Exit */
    mpeg3_close(file);
    return TRUE;
}


