// Jarl van Katwijk - 2001
// licensed under the GNU Public License (GPL) v2

#include "main.hh"
#include "corba_impl.hh"
#include "encoder.hh"

CORBA::ORB_ptr ORB;
PortableServer::POA_ptr POA;
gboolean client_orb_running = FALSE;


static gboolean
orb_handle_connection(GIOChannel *source, GIOCondition cond,GIOPConnection *cnx) {
    /* The best way to know about an fd exception is if select()/poll()
     * tells you about it, so we just relay that information on to ORBit
     * if possible
     */
    if(cond & (G_IO_HUP|G_IO_NVAL|G_IO_ERR))
        giop_main_handle_connection_exception(cnx);
    else
        giop_main_handle_connection(cnx);

    return TRUE;
}


static void orb_add_connection(GIOPConnection *cnx) {
    int tag;
    GIOChannel *channel;

    channel = g_io_channel_unix_new(GIOP_CONNECTION_GET_FD(cnx));
    tag = g_io_add_watch_full   (channel, G_PRIORITY_DEFAULT,
                                (GIOCondition)(G_IO_IN|G_IO_ERR|G_IO_HUP|G_IO_NVAL),
                                (GIOFunc)orb_handle_connection,
                                cnx, NULL);
    g_io_channel_unref (channel);
    cnx->user_data = GUINT_TO_POINTER (tag);
}

static void orb_remove_connection(GIOPConnection *cnx) {
    gtk_input_remove((guint)cnx->user_data);
    cnx->user_data = (gpointer)-1;
}

/* Start the client for the GUI orb */
gpointer start_gui_client(gpointer data) {
    int argc = 1;
    char *argv[2];
    argv[0] = (gchar*)"dripencoder";
    argv[1] = 0;

    CORBA_Environment ev;

    /* Reset connection handlers */
    IIOPAddConnectionHandler = orb_add_connection;
    IIOPRemoveConnectionHandler = orb_remove_connection;

    CORBA_exception_init(&ev);
    CORBA::ORB_var client_orb = CORBA::ORB_init(argc, argv, "orbit-local-orb"); 
    /* Reset connection handlers */
    IIOPAddConnectionHandler = orb_add_connection;
    IIOPRemoveConnectionHandler = orb_remove_connection;

    /* convert object reference */
    ifstream in("dripgui.ior");
    string ior;
    in >> ior;
    in.close();
    CORBA::Object_var gui_ = client_orb->string_to_object(ior.c_str());
    gui_client = drip::gui::_narrow(gui_);

    /* initialize the root POA and activate the corresponding manager */
    CORBA::Object_var poa_ = client_orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poa_);
    PortableServer::POAManager_var pm = poa->the_POAManager();

    client_orb_running = TRUE;
    pm->activate();

    g_print("Started frontend corba client, pid %i",getpid());
    while (TRUE) {
        sleep(1000000);
    }

    log_handler(G_LOG_LEVEL_WARNING,"Client orb quitting..");
    return NULL;
}


/* objects */
struct encoder_impl : public POA_drip::encoder {

    encoder_impl() {
    }

    ~encoder_impl() {
        pthread_cancel(encoderthread); /* terminate encoding thread */
    }

    void ping()throw(CORBA::SystemException) {
       //CORBA::Boolean result = FALSE;
        /* GUI is operational: start local client for gui server orb */
        g_print("Got ping from GUI\n");
        if (guiclientthread_id == -1) {
            client_orb_running = FALSE;
            guiclientthread_id = pthread_create(&guiclientthread,NULL,start_gui_client,NULL);
            while (!client_orb_running) {
                /* wait for orb to be initialised */
                usleep(100000);
            }
            fflush(NULL);
            gui_client->ping(1234);
            fflush(NULL);
        }
        return;
    }

    void start(const char *parameters,CORBA::Long nice)throw(CORBA::SystemException) {
        /* start encoding */
        gint count = 0;
        gchar **param;
        arg_parmS arg_parm;
        gpointer temp;

        //g_print("started encoder\n");
        log_handler(G_LOG_LEVEL_DEBUG,"Got activated");
        /* already encoding? */
        if (encoding) {
            log_handler(G_LOG_LEVEL_WARNING,"Already encoding.");
            return;
        }

        /* split parameters */
        param = g_strsplit(parameters," ",0);
        while (param[count]!=NULL) {
            count++;
        }
        arg_parm.argc = count;
        arg_parm.argv = param;
        arg_parm.nice = nice;

        /* do the encoding */
        temp = &arg_parm; //keep compiler happy about next line :
        encoderthread_id = pthread_create(&encoderthread,NULL,encode,temp);

        /* wait for parsing to be done and real encoding to start */
        while (!encoding) {
            usleep(100000);
        }
        //log_handler(G_LOG_LEVEL_DEBUG,"Started encoding after poll");

        /* free & exit */
        g_strfreev(param);
        return;
    }

    void quit()throw(CORBA::SystemException) {
        /* TODO: cleanup before exit */
        if (encoding) {
            pthread_cancel(encoderthread);
            encoding = FALSE;
        }
        /* remove ior file */
        unlink("dripencoder.ior");
        log_handler(G_LOG_LEVEL_MESSAGE,"dripencoder exiting.");
        exit(0);
    }
    void stop()throw(CORBA::SystemException) {
        if (encoding) {
            stopped = TRUE;
            encoding = FALSE;
            log_handler(G_LOG_LEVEL_DEBUG,"Stopped encoding");
        } else {
            log_handler(G_LOG_LEVEL_WARNING,"Not encoding, couldn't stop");
        }
        return;
    }

    void pause()throw(CORBA::SystemException) {
        if (encoding) {
            encoding_paused = TRUE;
            log_handler(G_LOG_LEVEL_MESSAGE,"Paused encoding");
        } else {
            log_handler(G_LOG_LEVEL_WARNING,"Not encoding, couldn't pause");
        }
        return;
    }

    void resume()throw(CORBA::SystemException) {
        if (encoding && encoding_paused) {
            encoding_paused = FALSE;
            log_handler(G_LOG_LEVEL_MESSAGE,"Resumed encoding");
        } else {
            log_handler(G_LOG_LEVEL_WARNING,"Not paused, couldn't resume");
        }
        return;
    }
};

gpointer run_encoder_orb(gpointer data) {
    int argc = 1;
    char *argv[2];
    argv[0] = (gchar*)"dripencoder";
    argv[1] = 0;

    /* start encoder orb */
    try {
        CORBA_Environment ev;
        unlink("dripencoder.ior");

        /* Reset connection handlers */
        IIOPAddConnectionHandler = orb_add_connection;
        IIOPRemoveConnectionHandler = orb_remove_connection;

        CORBA_exception_init(&ev);
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv,"orbit-local-orb");

        /* Reset connection handlers */
        IIOPAddConnectionHandler = orb_add_connection;
        IIOPRemoveConnectionHandler = orb_remove_connection;

        CORBA::Object_var poa_ = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(poa_);

        POA = poa;
        ORB = orb;

        PortableServer::POAManager_var pm = poa->the_POAManager();
        pm->activate();

        encoder_impl ei;

        PortableServer::ObjectId_var oid = poa->activate_object(&ei);
        CORBA::Object_var mswco = poa->id_to_reference(oid);
        // write the ior out to a file
        ofstream out("dripencoder.ior");
        out << orb->object_to_string(mswco) << flush;
        g_print("Started backend corba server, pid %i\n",getpid());

        orb->run();
        log_handler(G_LOG_LEVEL_WARNING,"Quitting encoder ORB");
    } catch (CORBA::SystemException &ex) {
        cout << "Caught system exception" << endl;
    }
    unlink("dripencoder.ior");

    return NULL;
}

void corba_init() {
    /* start corba servers */
    
    /* start encoder orb */
    run_encoder_orb(NULL);

    /* Reset connection handlers */
    IIOPAddConnectionHandler = orb_add_connection;
    IIOPRemoveConnectionHandler = orb_remove_connection;

    /* Start glib */
    gtk_main();
    return;
};

