// Jarl van Katwijk - 2001
// original code: mpeg2divx 0.8
// 2000/10/23 Ulrich Hecht <uli@emulinks.de>
// licensed under the GNU Public License (GPL) v2

#include <videoencoder.h>
#include <avifile.h>
#include <aviplay.h>
#include <version.h>
#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
   #include <fourcc.h>
   #include <creators.h>
#else
   #include <aviutil.h>
#endif
#include <libmpeg3.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <fstream>
#include "encoder-cpp.hh"
#include "gui-cpp.hh"
#include <errno.h>
#include "../libmpeg3-1.2.3/spu.h"
#include "../libmpeg3-1.2.3/subpictures.h"
#include "../src/drip.h"
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "deinterlace.hh"
#include "autoclipper.hh"
#include "main.hh"

using namespace std;

gint clipleft = 0;
gint clipright = 0;
gint cliptop = 0;
gint clipbottom = 0;

#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
/* Wrapper for SetAttribute of avifile codec object */
void Codec_SetAttribute(const CodecInfo* codecInfo,const gchar* attribute, gint value) {
    gint result;
    GString *output = g_string_new("");

    /* Try to set attribute */
    result = Creators::SetCodecAttr(*codecInfo, attribute, value);

    /* Handle negative sets */
    if (result==-1) {
        g_string_sprintf(output,"Could not set attribute %s to value %i. Ignoring, wrong codec or outdated avifile?",attribute,value);
        log_handler(G_LOG_LEVEL_WARNING,output->str);
    }

    g_string_free(output,TRUE);
    return;
}
#endif

gpointer encode(gpointer data) {
    arg_parmS *arg_parm = (arg_parmS*)data;
    int argc = arg_parm->argc;
    gchar **argv = arg_parm->argv;
    mpeg3_t* file;
    gint result;
    GString *output = g_string_new("");

    result = nice(arg_parm->nice);
    if (result!=0) {
        log_handler(G_LOG_LEVEL_WARNING,"Renice failed, used an invalid value");
    }

    log_handler(G_LOG_LEVEL_DEBUG,"Starting encoding engine...");
    if(argc < 4) {
        printf("\nUsage: %s [OPTION]... [input MPEG]... [output AVI]\n",argv[0]);
        printf("\nOptions:\n");
        printf("  -A\tRun standalone, dont start and use Drip CORBA services\n");
        printf("  -F\tDo not fork to background, usefull for debugging and standalone mode\n");
        printf("  -b\tDivX ;-) bitrate (default 1200)\n");
        printf("  -f\tuse fast-motion codec (default low-motion)\n");
#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
        printf("  -d\tuse DIVX4 encoding codec\n");
#endif
        printf("  -a\tMP3 bitrate (kBit, default auto)\n");
        printf("  -e\tnumber of frames to encode (default all)\n");
        printf("  -ct\tnumber of lines to clip from top (default none)\n");
        printf("  -cb\tnumber of lines to clip from bottom (default none)\n");
//        printf("  -o\toffset of clipping window (default 0 = center)\n");
        printf("  -cl\tnumber of lines to clip from left (default none)\n");
        printf("  -cr\tnumber of lines to clip from right (default none)\n");
        printf("  -ac\tauto clip (default off, disables other clipping options)\n");
        printf("  -bf\tradius for blur filter (default off)\n");
        printf("  -w\toutput width (-1 = input width)\n");
        printf("  -h\toutput height (-1 = auto, when width=-1 too both are same as input)\n");
        printf("  -z\thyper zooming, looks better but very slow (default off)\n");
        printf("  -s\tadd subpictures to result, also needs a CLUT file\n");
        printf("  -n\tnum cpus for decoding (default 1)\n");
        printf("  -as\taudio stream number (default 0)\n");
        //always disabled for now, not enough speed gain to compensate quality loss: printf("  -m\tuse mmx when availeble. Know to be buggish\n");
        printf("  -vs\tvideo stream number (default 0)\n");
        printf("  -l\tlimit output to size in kb, open new output when limit is reached\n");
        printf("  -xy\taspect ratio after clipping, 2 numbers (like: 16 9, default no aspect correction)\n");
        printf("\n" APPNAME " " VERSION " by Jarl van Katwijk <jarl@xs4all.nl>, based on work by Ulrich Hecht <uli@emulinks.de>\n");
        exit(0);
    }

    g_print("Started encoding engine, pid %i\n",getpid());

    gint argbase=0;
    gint divxbitrate=1200;
    gfloat mp3bitrate=-1;
    FOURCC codec=fccDIV3;
    gint endframe=-1;
    gboolean autoclip = FALSE;
    gboolean hyper_zoom = FALSE;
    gboolean blur=FALSE;
    gboolean deinterlace=TRUE;
    gint owidth=-1;
    gint oheight=-1;
    gint numcpus=1;
    gint austream=0;
    gint vistream=0;
    gint subpicture = -1;
    gchar* clutfile;
    static struct spu_img_struct spu;
    gint mmx=0; //default disabled: very little speed gain, and mxx decoding is said to be lossy.
    gint limit=0;
    gfloat aspect = 0;
    gboolean scale = FALSE;

    gboolean moreopts=true;
    gint max_size = 731906048;
    while(moreopts) {
        moreopts=FALSE;

        if(strcmp(argv[argbase+1],"-cl")==0) {
            clipleft=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=TRUE;
        }
        if(strcmp(argv[argbase+1],"-cr")==0) {
            clipright=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=TRUE;
        }
        if(strcmp(argv[argbase+1],"-b")==0) {
            divxbitrate=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=TRUE;
        }
        if(strcmp(argv[argbase+1],"-f")==0) {
            codec=fccDIV4;
            argbase++;
            moreopts=TRUE;
        }
#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
        if(strcmp(argv[argbase+1],"-d")==0) {
            codec=fccDIVX;//mmioFOURCC('d', 'i', 'v', '3');//1482049860;
            argbase++;
            moreopts=TRUE;
        }
#endif
        if(strcmp(argv[argbase+1],"-a")==0) {
            mp3bitrate=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=TRUE;
        }
        if(strcmp(argv[argbase+1],"-xy")==0) {
            aspect=atoi(argv[argbase+2]);
            aspect=aspect/atoi(argv[argbase+3]);
            argbase+=3;
            moreopts=TRUE;
        }
        if(strcmp(argv[argbase+1],"-e")==0) {
            endframe=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=TRUE;
        }
        if(strcmp(argv[argbase+1],"-ct")==0) {
            cliptop=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-cb")==0) {
            clipbottom=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-z")==0) {
            hyper_zoom = TRUE;
            argbase+=1;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-ac")==0) {
            autoclip = TRUE;
            argbase+=1;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-bf")==0) {
            blur=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-w")==0) {
            owidth=atoi(argv[argbase+2]);
            scale = TRUE;
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-h")==0) {
            oheight=atoi(argv[argbase+2]);
            scale = TRUE;
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-as")==0) {
            austream=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-vs")==0) {
            vistream=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-s")==0) {
            subpicture=atoi(argv[argbase+2]);
            clutfile= argv[argbase+3];
            argbase+=3;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-n")==0) {
            numcpus=atoi(argv[argbase+2]);
            argbase+=2;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-m")==0) {
            mmx=1;
            argbase+=1;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-l")==0) {
            limit=1;
            max_size=atoi(argv[argbase+2]);
            argbase+=2;
          moreopts=true;
        }

        if(strcmp(argv[argbase+1],"-A")==0) {
            argbase+=1;
            moreopts=true;
        }
        if(strcmp(argv[argbase+1],"-F")==0) {
            argbase+=1;
            moreopts=true;
        }

    }

    /* Chunk size */
    if (max_size<50000) {
        /* Stop polling loops */
        encoding = TRUE;
        usleep(15000);
        if (standalone==FALSE) {
            gui_client->ready();
            fflush(NULL);
        }
        encoding = FALSE;
        log_handler(G_LOG_LEVEL_WARNING,"Max size too small, use at least 50000kb");
        return 0;
    } else {
        /* Adjust max size so max_size_new + header =~ max_size */
        max_size=max_size-50000;
    }

    /* Input files */
    int numfiles=argc-argbase-2;
    GString* inputfiles[numfiles];
    for(int i=argbase+1; i<argc-1; i++) {
        inputfiles[i-argbase-1] = g_string_new(argv[i]);
        g_string_sprintf(output,"Input %d out %s",i-argbase-1,inputfiles[i-argbase-1]->str);
        log_handler(G_LOG_LEVEL_DEBUG,output->str);
    }
    char* outputfile=argv[argc-1];

    /* Open video */
    g_string_sprintf(output,"Opening %s",inputfiles[0]->str);
    log_handler(G_LOG_LEVEL_MESSAGE,output->str);
    file = mpeg3_open(inputfiles[0]->str);
    if (file==0) {
        /* Error, not able to open stream properly */
        g_string_sprintf(output,"Unable to open %s, possibly not a plain MPEG2 or valid IFO file",inputfiles[0]->str);
        log_handler(G_LOG_LEVEL_WARNING,output->str);
        encoding = TRUE;
        if (!standalone) {
            gui_client->ready();
            fflush(NULL);
        }
        encoding = FALSE;
        return 0;
    }

    /* Open audio stream */
    int astrm=mpeg3_total_astreams(file);
    austream=(austream>astrm - 1 ? astrm - 1 : austream);
    g_string_sprintf(output,"Set audio stream %d of %d", austream, astrm);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);
    if (austream < 0) log_handler(G_LOG_LEVEL_WARNING,"There will be NO audio encoded");
   
    /* Open video stream */
    int vstrm=mpeg3_total_vstreams(file);
    vistream=(vistream>vstrm - 1 ? vstrm - 1 : vistream);
    g_string_sprintf(output,"Set video stream %d of %d", vistream, vstrm);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);

    /* Audio channels */
    //int achan=mpeg3_audio_channels(file, austream);
    //int rate=mpeg3_sample_rate(file, austream);
    //long samples=mpeg3_audio_samples(file, austream);
    int achan;
    int rate;
    long samples;
    if (austream >= 0) {
        achan=mpeg3_audio_channels(file, austream);
        rate=mpeg3_sample_rate(file, austream);
        samples=mpeg3_audio_samples(file, austream);
    } else {
        achan = 0;
        rate = 0;
        samples = 0;
    }
    g_string_sprintf(output,"Channels %d, rate %d, samples %li",achan,rate,samples);
    log_handler(G_LOG_LEVEL_DEBUG,output->str);

    /* Video input is a IFO file? -> precalculate total frames */
    ulong vobtotalframes = 0;
    GString *extension = g_string_new((gchar*)(inputfiles[0]->str + inputfiles[0]->len - 3));
    if (strcasecmp("IFO",extension->str)==0) {
         /* Got an IFO :
          *  - open 1st VOB,
          *  - get estimated frames
          *  - calculate total frames
          */
         GString *sequence = g_string_new((gchar*)(inputfiles[0]->str + inputfiles[0]->len - 8));
         sequence = g_string_truncate(sequence,2);
         GString *dir = g_string_new(inputfiles[0]->str);
         dir = g_string_truncate(dir,dir->len - 12);
         GString *vob = g_string_new("");
         if (strcmp(extension->str,"IFO")==0) {
             g_string_sprintf(vob,"%sVTS_%s_1.VOB",dir->str,sequence->str);
         } else {
             g_string_sprintf(vob,"%svts_%s_1.vob",dir->str,sequence->str);
         }
         /* Open VOB */
         mpeg3_t* vobfile = mpeg3_open(vob->str);
         long vobframes   = mpeg3_video_frames(vobfile, vistream);
         struct stat vobstat; stat(vob->str, &vobstat);
         ulong vobtotalsize = vobstat.st_size / 1024;
         float bytes_frame = (float)vobframes / (float)vobtotalsize;

         /* Loop through all VOB files and calculate total frames */
         int i = 2;
         int result;
         do {
             if (strcmp(extension->str,"IFO")==0) {
                 g_string_sprintf(vob,"%sVTS_%s_%i.VOB",dir->str,sequence->str,i);
             } else {
                 g_string_sprintf(vob,"%svts_%s_%i.vob",dir->str,sequence->str,i);
             }
             result = stat(vob->str, &vobstat);
             if (result == 0) {
                 /* Add size to total size */
                 vobtotalsize += vobstat.st_size/1024;
             }
             i++;
         } while (result == 0);

         vobtotalframes = (ulong)(bytes_frame * vobtotalsize);

         /* Clean */
         g_string_free(sequence,TRUE);
         g_string_free(vob,TRUE);
         g_string_free(dir,TRUE);
    }
    g_string_free(extension,TRUE);


    /* SMP */
    if (numcpus>1) {
         mpeg3_set_cpus(file, numcpus);
    } 
   
    /* MMX */
    if (mmx==1) {
         mpeg3_set_mmx(file,TRUE);
    } else {
         mpeg3_set_mmx(file,FALSE);
    }

    struct stat VOB_file;
    ulong total_bytes;
    float frames_per_bytes;
    uint width=mpeg3_video_width(file, vistream);
    uint height=mpeg3_video_height(file, vistream);
    float framerate = mpeg3_frame_rate(file, vistream);
    long frames=mpeg3_video_frames(file, vistream);
    g_string_sprintf(output,"Input video reads %dx%d %.2ffps",width,height,framerate);
    log_handler(G_LOG_LEVEL_INFO,output->str);

    /* Setup frames/bytes for more correct total frames estimation */
    stat(inputfiles[0]->str, &VOB_file);
    total_bytes = VOB_file.st_size;
    frames_per_bytes = (float)frames / (float)total_bytes;
    glong todo_frames = frames;

    /* Allocate buffers */
    unsigned char* framebuffer = new unsigned char[width*height*BYTES_PER_COLOUR_OUTPOUT];
    unsigned char* extrabuffer = new unsigned char[width*BYTES_PER_COLOUR_OUTPOUT+4];
    unsigned char** rowptr = new (unsigned char*)[height];
    for (guint i=0; i<height; i++) {
        if(i==height-1) {
            rowptr[i]=extrabuffer;
        } else {
            rowptr[i]=framebuffer+(width*(height-1)*BYTES_PER_COLOUR_OUTPOUT)-(width*BYTES_PER_COLOUR_OUTPOUT*i);
        }
    }
#ifdef YCRCB
    char* Ybuffer = new (char)[width*height];
    char* CRbuffer = new (char)[width*height/4];
    char* CBbuffer = new (char)[width*height/4];
#endif

    gint sampsperframe = (gint)(rate/framerate);
    gshort* audiobufferint = new gshort[sampsperframe*2];
    gshort* read_buffer = new gshort[sampsperframe];

    /* Clipping */
    if (autoclip==TRUE) {
        log_handler(G_LOG_LEVEL_INFO,"Autocropper determining borders...");
	gint _cliptop = 10000;
	gint _clipbot = 10000;
	gint _cliprig = 10000;
	gint _cliplef = 10000;
        gint i;
	for (i=0;i<10;i++) { /* 10 too high\low?? */
	    autoclipper(inputfiles[0]->str);
	    if (_cliptop>cliptop) { _cliptop = cliptop;}
            if (_clipbot>clipbottom) { _clipbot = clipbottom;}
            if (_cliplef>clipleft) { _cliplef = clipleft;}
            if (_cliprig>clipright) { _cliprig = clipright;}
        }
        cliptop = _cliptop;
        clipbottom = _clipbot;
        clipleft = _cliplef;
        clipright = _cliprig;
    }
    if ((cliptop!=0) || (clipbottom!=0) || (clipleft!=0) || (clipright!=0)) {
        scale = TRUE;
    };
    g_string_sprintf(output,"Clipping top %i,bottom %i,left %i,right %i",cliptop,clipbottom,clipleft,clipright);
    log_handler(G_LOG_LEVEL_INFO,output->str);

    /* --- Zooming related sizes */
    if(oheight==-1 && owidth==-1) {
        oheight=height-(cliptop+clipbottom);
    }
    if(owidth==-1) {
        owidth=width-(clipleft+clipright);
    }
    if(oheight==-1) {
        gfloat ow,wclip,oh;
        ow = width;
        wclip = owidth;
        oh = height-(cliptop+clipbottom);
        oheight=(gint)((wclip/ow)*oh); /* calc Y and make it even */
    }
    g_string_sprintf(output,"Output video clipped %ix%i %.2ffps",owidth,oheight,framerate);
    log_handler(G_LOG_LEVEL_INFO,output->str);
    if (aspect!=0) {
        /* Fixed aspect scaling */
        g_string_sprintf(output,"Using DVD aspect %.2f:1",aspect);
        log_handler(G_LOG_LEVEL_INFO,output->str);
        gfloat ow = owidth;
        oheight=(gint)(ow/aspect);
    }
    /* Adjust Width & Height to a multipe of 16 for better encoding */
    owidth = ((gint)owidth/16)*16;
    oheight= ((gint)oheight/16)*16;

    g_string_sprintf(output,"Output video corrected %ix%i at %.2fFPS",owidth,oheight,framerate);
    log_handler(G_LOG_LEVEL_INFO,output->str);


    /* --- Output Video header */
    VideoEncoderInfo info;
    memset(&(info.header),0,sizeof(BITMAPINFOHEADER));
    info.header.biSize=sizeof(BITMAPINFOHEADER);
    info.header.biPlanes=1;
    info.header.biBitCount=AVIFILE_COLOUR_DEPT_INPUT;
#ifdef YCRCB
    info.header.biCompression=fccYUV;
#endif
    info.header.biWidth = owidth;
    info.header.biHeight = oheight;
    info.header.biSizeImage=info.header.biWidth*info.header.biHeight*BYTES_PER_COLOUR_OUTPOUT;
    info.compressor = codec;
    info.quality = 10000;
    info.keyfreq = 200;

    /* Output codec */
    switch (codec) {
#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
        case fccDIVX:    log_handler(G_LOG_LEVEL_INFO,"Encoding to OpenDIVX"); //1482049860
                         break;
#endif
        case fccDIV3:    log_handler(G_LOG_LEVEL_INFO,"Encoding to MPEG4 low motion");
                         break;
        case fccDIV4:    log_handler(G_LOG_LEVEL_INFO,"Encoding to MPEG4 high motion");
                         break;
        default:         log_handler(G_LOG_LEVEL_ERROR,"No codec, not encoding.");
    }

    /* Output codec attributes */
    log_handler(G_LOG_LEVEL_DEBUG,"About to make avifile call, when Drip stops (crashes) here get yourself a working avifile...");
#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
    const CodecInfo* codecInfo=CodecInfo::match(codec);
    if (codec==fccDIV3 || codec==fccDIV4) { /* MSMPEG4 */
       Codec_SetAttribute(codecInfo, "Crispness", 100);
       Codec_SetAttribute(codecInfo, "KeyFrames", info.keyfreq);
       Codec_SetAttribute(codecInfo, "BitRate", divxbitrate);
    }
    if (codec==fccDIVX) { /* OpenDIVX, DIVX4 */
       Codec_SetAttribute(codecInfo, "BitRate",divxbitrate*1500);
       Codec_SetAttribute(codecInfo, "rc_period",info.keyfreq*10);
       Codec_SetAttribute(codecInfo, "rc_reaction_period",(gint)(info.keyfreq/20));
       Codec_SetAttribute(codecInfo, "rc_reaction_ratio",(gint)(info.keyfreq/10));
       Codec_SetAttribute(codecInfo, "max_key_interval",(gint)((info.keyfreq*3)/2));
       Codec_SetAttribute(codecInfo, "obmc",0); //TODO
       Codec_SetAttribute(codecInfo, "quality",5); // DIVX:5=best,1=lowest, avifile:1=enable VBR... ????
       Codec_SetAttribute(codecInfo, "max_quantizer",12);
       Codec_SetAttribute(codecInfo, "min_quantizer",2);
       Codec_SetAttribute(codecInfo, "bidirect",0); //TODO:2pass encoding,0 = disabled
       Codec_SetAttribute(codecInfo, "deinterlace",1); //TODO
    }
#else
       IVideoEncoder::SetExtendedAttr(codec, "BitRate", divxbitrate);
       IVideoEncoder::SetExtendedAttr(codec, "KeyFrames", info.keyfreq);
       IVideoEncoder::SetExtendedAttr(codec, "Crispness", 100);
#endif

    /* --- Create & open avi output stream */
    IAviSegWriteFile* avifile = CreateSegmentedFile(outputfile,max_size,0,00777);
    IAviVideoWriteStream* stream=avifile->AddVideoStream(info.compressor, &(info.header), (unsigned int)1000000./framerate);
    stream->SetQuality(info.quality);
    stream->SetKeyFrame(info.keyfreq);
    stream->Start();
    log_handler(G_LOG_LEVEL_DEBUG,"Avifile initialisation calls passed");
    g_string_sprintf(output,"Encoding bitrate %i",divxbitrate);
    log_handler(G_LOG_LEVEL_INFO,output->str);
    g_string_sprintf(output,"Chunk size %i bytes",max_size);
    log_handler(G_LOG_LEVEL_INFO,output->str);
    g_string_sprintf(output,"Output %s",outputfile);
    log_handler(G_LOG_LEVEL_MESSAGE,output->str);

    /* --- Buffers */
    unsigned char* scaledbuffer = new unsigned char[info.header.biHeight*info.header.biWidth*BYTES_PER_COLOUR_OUTPOUT];
    CImage* im;
    if (scale == TRUE) {
        im = new CImage((BitmapInfo*)&(info.header),scaledbuffer, false);
    } else {
        im = new CImage((BitmapInfo*)&(info.header),framebuffer, false);
    }
    guchar* _RGBbuffer = (guchar*)malloc(info.header.biHeight*info.header.biWidth*3);
    
    /* Scaling buffers */
    GdkPixbuf*  gframe = gdk_pixbuf_new_from_data( framebuffer+clipbottom*width*BYTES_PER_COLOUR_OUTPOUT, GDK_COLORSPACE_RGB, FALSE, 8, width, height-(cliptop+clipbottom), width*BYTES_PER_COLOUR_OUTPOUT, NULL, NULL);
    GdkPixbuf*  gscaled = gdk_pixbuf_new_from_data( scaledbuffer, GDK_COLORSPACE_RGB, FALSE, 8, info.header.biWidth, info.header.biHeight, info.header.biWidth*BYTES_PER_COLOUR_OUTPOUT, NULL, NULL);

    /* --- Output Audio header */
    WAVEFORMATEX fmt;
    memset(&fmt,0,sizeof(WAVEFORMATEX));
    fmt.wFormatTag=1;
    fmt.nChannels=(achan>2 ? 2 : achan);
    fmt.nSamplesPerSec=rate;
    fmt.nAvgBytesPerSec=(achan>1 ? rate : rate*2);
    fmt.nBlockAlign=(achan>1 ? 2 : 1);
    fmt.wBitsPerSample=16;
    fmt.cbSize=sizeof(WAVEFORMATEX);

    if(achan>0 && mp3bitrate == -1) switch(rate) {
        case 48000: mp3bitrate=(achan>1?160:80*achan); break;
        case 44100: mp3bitrate=(achan>1?112:56*achan); break;
        case 22050: mp3bitrate=(achan>1?64:32*achan); break;
        default: g_string_sprintf(output,"unknown sample rate %d Hz, please specify MP3 bitrate",rate);
                log_handler(G_LOG_LEVEL_ERROR,output->str);
                exit(1);
    }

    IAviAudioWriteStream* astream = NULL;
    if(achan>0) {
        astream=avifile->AddAudioStream(0x55, &fmt, (int)(mp3bitrate*1000)/8);
        astream->Start();
    }

    glong totalframes=0;
    struct timeval tv;
    struct timezone tz={0,0};
    gettimeofday(&tv,&tz);
    glong startsec=tv.tv_sec;
    glong startusec=tv.tv_usec;
    double fps=0;
    gint total_channels;
    gint assetChannels;
    int16_t *input_ptr;
    int16_t *output_ptr;
    gchar *buffer;
    gint downmix51;
    int16_t *input_end;
    gint len;
    gint cached_frames = 0;
    gint skip_frame = 0;
    glong totalframes_countdown = 0;
    gint framefile_fd;
    guint gio_result;
    GIOChannel *framefile_channel;
    GString *framefile = g_string_new("/tmp/dripframe.raw");
    glong writeout_frames_countdown = 1;

    /* Open subpicture */
    if (subpicture > -1) {
        subpicture_init(subpicture,width,height,(u_int*)framebuffer,framerate,clutfile);
        g_string_sprintf(output,"Initialized subpicture %i",subpicture);
        log_handler(G_LOG_LEVEL_DEBUG,output->str);
    }

    /* 
     * Encoding starts
     */
    encoding = TRUE;
    for (gint j=0; j< numfiles; j++) {
        if(j) {
            /* Close previous input stream */
            mpeg3_close(file);
            /* Open new input stream */
            file   = mpeg3_open(inputfiles[j]->str);
            frames = mpeg3_video_frames(file, vistream);
            g_string_sprintf(output,"Opening input AV stream from %s with %li frames",inputfiles[j]->str,frames);
            log_handler(G_LOG_LEVEL_MESSAGE,output->str);
            if (file==0) {
                /* Error, not able to open input stream properly */
                g_string_sprintf(output,"Unable to open %s, possibly not a plain MPEG2 or valid IFO file",inputfiles[0]->str);
                log_handler(G_LOG_LEVEL_WARNING,output->str);
                encoding = TRUE;
                if (!standalone) {
                    gui_client->ready();
                    fflush(NULL);
                }
                /* Close streams */
                stream->Stop();
                delete avifile;
                mpeg3_close(file);
                /* Encoding done */
                encoding = FALSE;
            }
            /* mark skip_frame so the first 9 frames of this new VOB are not encoded */
            skip_frame = 6;
            /* Reset total frames counter */
            totalframes_countdown = 24;
            /* calculate new estimated total frames */
            stat(inputfiles[j]->str, &VOB_file);
            total_bytes += VOB_file.st_size;
            todo_frames = (long)((float)total_bytes * (float)frames_per_bytes);
        }

        gint quit=FALSE;
        for (gint i=0;quit==FALSE; i++, totalframes++) {

            /* Reached endframe? */
            quit = (totalframes == (endframe-1));

            /* Paused? */
            while (encoding_paused) {
                usleep(1000000);
            }

            /* Stop? */
            if (stopped) {
                stopped = FALSE;
                printf("\tstopped.\n");
                todo_frames = totalframes;
                quit = TRUE;
                j = numfiles;
                break;
            }

            /* Notify GUI with progress every ?100? frames */
            if ( (totalframes_countdown--) == 0) {
                ulong todo = 0;
                totalframes_countdown = 24;
                if (todo_frames == 0) {
                    todo = vobtotalframes;
                } else {
                    todo = todo_frames;
                }
                if (standalone==FALSE) {
                    gui_client->progress(totalframes,todo,j,numfiles);
                    fflush(NULL);
                } else {
                    g_print("\rEncoding frame %i of %i",totalframes,todo);
                }
            }

            /* Decode Next Frame */
#ifndef YCRCB
            if(mpeg3_read_frame(file, rowptr, 0, 0, width, height, width, height, LIBMPEG_COLOUR_DEPT_OUPUT, vistream)) {
#else
            if(mpeg3_read_yuvframe(file, Ybuffer,CRbuffer,CBbuffer, 0, 0, width, height, vistream)) {
#endif
                /* Reached end of input stream */
                todo_frames = totalframes;
                quit = TRUE;
                /* Update caching */
                cached_frames = 0;
                break;
            }

            /* Audio handling */
            if(achan>0) {
                downmix51 = 0;
                result = 0;
                len = sampsperframe;
                buffer = (gchar *)audiobufferint;
                output_ptr = NULL;
                input_ptr = (int16_t*)read_buffer;
                input_end = input_ptr + len;
                assetChannels = achan;
                if (assetChannels > 2) {
                    assetChannels = 2;
                }
                if(achan > assetChannels) {
                    downmix51 = 1;
                    for(gint channel = 0; channel < assetChannels; channel++) {
                        output_ptr = (int16_t*)buffer + channel;
                        int16_t *output_end = (int16_t*)output_ptr + len * assetChannels;
                        while(output_ptr < output_end) {
                            *output_ptr++ = 0;
                        }
                    }
                }
                total_channels = (downmix51 ? achan : assetChannels);
                for(gint channel = 0; 
                    channel < total_channels && !result; 
                    channel++) {
                    if(channel == 0)
                        result = mpeg3_read_audio(file, 0,read_buffer,channel,len,austream);
                    else
                        result = mpeg3_reread_audio(file,0,read_buffer,channel, len,austream);
                    /* Interlace the audio for the sound driver */
                    if(downmix51) {
                        input_ptr = (int16_t*)read_buffer;
                        input_end = input_ptr + len;
                        switch(channel) {
                            case 0: output_ptr = (int16_t*)buffer;       break; // Left
                            case 1: output_ptr = (int16_t*)buffer;       break; // Center
                            case 2: output_ptr = (int16_t*)buffer + 1;   break; // Right
                            case 3: output_ptr = (int16_t*)buffer;       break; // Left2
                            case 4: output_ptr = (int16_t*)buffer + 1;   break; // Right2
                        }
                        if(channel == 1) {
                            while(input_ptr < input_end) {
                                *output_ptr++ += *input_ptr;
                                *output_ptr++ += *input_ptr++;
                            }
                        } else {
                            while(input_ptr < input_end) {
                                *output_ptr += *input_ptr++;
                                output_ptr += assetChannels;
                            }
                        }
                    } else {
                        output_ptr = (int16_t*)buffer + channel;
                        input_ptr = (int16_t*)read_buffer;
                        input_end = input_ptr + len;
                        while(input_ptr < input_end) {
                            *output_ptr = *input_ptr++;
                            output_ptr += assetChannels;
                        }
                    }
                }
                if(achan>1) {
                    astream->AddData(audiobufferint,sampsperframe*2);
                } else {
                    astream->AddData(audiobufferint,sampsperframe);
                }
            }

            /* Deinterlace */
            //if (deinterlace==TRUE) {
            //    deinterlace_apply(framebuffer,width,height,totalframes);
            //}
                
            /* Blurring */
            if (blur==TRUE) {
                im->Blur(blur);
            }

            /* Subpicture */
            if (subpicture > -1) {
                /* Overlay spubpicture to current frame */
                subpicture_overlay((u_int*)framebuffer,width,height,clipbottom);
            }

            /* Scaling */
            if (scale==TRUE) {
                if(hyper_zoom==TRUE) {
                    gdk_pixbuf_scale(gframe, gscaled, 0, 0, info.header.biWidth, info.header.biHeight,-clipleft*info.header.biWidth / double(width), 0, info.header.biWidth / double(width-(clipleft+clipright)), info.header.biHeight / double(height-(cliptop+clipbottom)),GDK_INTERP_HYPER);
                } else {
                    gdk_pixbuf_scale(gframe, gscaled, 0, 0, info.header.biWidth, info.header.biHeight,-clipleft*info.header.biWidth / double(width), 0, info.header.biWidth / double(width-(clipleft+clipright)), info.header.biHeight / double(height-(cliptop+clipbottom)),GDK_INTERP_BILINEAR);
                }
            }

            /* Writeout current frame every 250 frames */
            //TODO: once working ok speed up 
            if ( (standalone==FALSE) && ((writeout_frames_countdown--) == 0) ) {
                writeout_frames_countdown = 249;
                framefile_fd = open(framefile->str,O_WRONLY|O_CREAT|O_TRUNC,S_IREAD|S_IWRITE);
                guchar* BGRbuffer; 
                if (scale == TRUE) {
                    BGRbuffer = scaledbuffer;
                } else {
                    BGRbuffer = framebuffer;
                }
                guchar* RGBbuffer;
                /* Yaxis invert and BGR->RGB convert */
                for (guint y=0;y<info.header.biHeight;y++) {
                    RGBbuffer = _RGBbuffer+(info.header.biHeight-1-y)*(info.header.biWidth*3);
                    for (guint x=0;x<info.header.biWidth;x++) {
                        *RGBbuffer = *(BGRbuffer+2);RGBbuffer++;
                        *RGBbuffer = *(BGRbuffer+1);RGBbuffer++;
                        *RGBbuffer = *BGRbuffer;RGBbuffer++;
                        BGRbuffer+=3; 
                    }
                }
                /* Writeout frame */
                write(framefile_fd,_RGBbuffer,info.header.biHeight*info.header.biWidth*3);
                close(framefile_fd);
                gui_client->frame(totalframes,info.header.biWidth,info.header.biHeight);
                fflush(NULL);
            }

            /* Encode next frame (if not skipping) */
            if (skip_frame == 0) {
                stream->AddFrame(im);
            } else {
                skip_frame--;
            }
        }
    }

    /* Stop & close encoding */
    stream->Stop();
    delete avifile;
    mpeg3_close(file);
    /* Display encoding info */
    gettimeofday(&tv,&tz);
    fps=totalframes/((tv.tv_sec+tv.tv_usec/1000000.0)-(startsec+startusec/1000000.0));
    g_string_sprintf(output,"Finished, done %g frames per sec",fps);
    log_handler(G_LOG_LEVEL_INFO,output->str);
    g_string_free(output,TRUE);
    /* Encoding done */
    encoding = FALSE;
    /* Notify GUI encoding has finished */
    if (!standalone) {
        gui_client->ready();
        fflush(NULL);
    }
    /* Clean & Exit */
    free(_RGBbuffer);
    delete(framebuffer);
    gdk_pixbuf_finalize(gframe);
    delete(scaledbuffer);
    gdk_pixbuf_finalize(gscaled);
    delete(extrabuffer);
    delete(rowptr);
    if (!standalone) delete(gui_client);
    return(0);
}

