// Jarl van Katwijk - 2001
// licensed under the GNU Public License (GPL) v2

#include "main.hh"
#include "corba_impl.hh"
#include "encoder.hh"
#include <signal.h>
#include <setjmp.h>



using namespace std;

gboolean encoding = FALSE;
gboolean stopped = FALSE;
gboolean encoding_paused = FALSE;
pthread_t serverthread;
gint serverthread_id;
pthread_t encoderthread;
gint encoderthread_id;
pthread_t guiclientthread;
gint guiclientthread_id = -1;
drip::gui_var gui_client = NULL;
gint linenr = 0;
gint log_handler_encoder_id;
gboolean forking = TRUE;
gboolean standalone = FALSE;
sigjmp_buf environment; /* used for SIGSEGV handling */



/* Functions */
void log_handler(glong level,const gchar *message) {
    /* output to GUI Corba orb or to stdout */
    if (standalone==FALSE && gui_client!=NULL) {
        gui_client->log(level,message);
        fflush(NULL);
    } else {
        g_print("%s\n",message);
    }
    return;
}

/* Handles signals */
void signal_handler(int sig) {

    if (standalone==FALSE && gui_client!=NULL) {
        gui_client->errorsignal(sig);
        fflush(NULL);
    } else {
        g_print("Got an error (%i), please report details of this bug",sig);
    }

    if (sig == SIGSEGV) {
        /* Restore situation as it was before the segvault signal */
        siglongjmp(environment,1);
        printf("After SIGSEGV catching longjmp? Something is very wrong here...\n");
        fflush(0);
        signal(SIGSEGV,SIG_DFL); /* Reset segv signal handler, something has gone very wrong */
    }

    fflush(0);
    exit(0);
    return;
}

/* Hooks signals to internal handler */
void signal_hook(void) {

    /* Save environment */
    sigsetjmp(environment,1);

    if (sigsetjmp(environment,1) == 0) {
        /* Hook signals */
        signal(SIGTERM,signal_handler);
        signal(SIGHUP,signal_handler);
        signal(SIGABRT,signal_handler);
        signal(SIGTERM,signal_handler);
        //signal(,signal_handler);
        //signal(,signal_handler);
        //signal(,signal_handler);
        //signal(,signal_handler);
        //signal(SIGSEGV,signal_handler);
        //signal(SIGINT,signal_handler);
    } else {
        /* Got a SIGSEGV inside handler? */
        signal(SIGSEGV,SIG_DFL);
    }

    return;
}

int main(int argc, char** argv) {
    gint i;
    arg_parmS params;

    printf(APPNAME " " VERSION " by Jarl van Katwijk <jarl@xs4all.nl>, based on work by Ulrich Hecht <uli@emulinks.de>\n");

    /* Hook signals */
    //signal_hook();

    /* Check libavi version */
    if (GetAvifileVersion()!=AVIFILE_VERSION) {
        cout<<"This binary was compiled for Avifile ver. "<<AVIFILE_VERSION<<", but the library is ver. "<<GetAvifileVersion()<<". Aborting."<<endl;
        return 0;
    }
    g_print("Started dripencoder, pid %i\n",getpid());

    /* Init threading */
    g_thread_init(NULL);

    /* Seed random generator */
    srandom(1); //todo time feed

    /* STANDALONE? */
    for (i=0;i<argc;i++) {
        if(strcmp(argv[i],"-A")==0) {
            standalone = TRUE;
            forking = FALSE;
        }
        if(strcmp(argv[i],"-F")==0) {
            forking = FALSE;
        }

    }
    if (standalone) {
        params.argc = argc;
        params.argv = argv;
        g_print("dripencoder is started\n");
        encode((gpointer)&params);
        exit(0);
    }

    /* Nope, runnning as backend, fork() */
    if (forking) {
        switch(fork()) {
            case 0:
                g_print("dripencoder is forked in the background, use -F for forground execution or -A for standalone mode\n");
                /* start corba */
                corba_init();
                break;
            case -1:
                g_error("Drip could not be forked into the background (Error Num: %d)\n",errno);
                exit(-1);
                break;
        }
    } else {
        /* start corba */
        corba_init();
    }

    /* Clean & exit */
    exit(0);
    return(0);
}

