/*****
*
* This file is part of the OMS program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#ifndef __SPU_H__
#define __SPU_H__

typedef struct {        // CLUT == Color LookUp Table
    guint8:8;
    guint8 y:8;
    guint8 cr:8;
    guint8 cb:8;
} __attribute__ ((packed)) clut_t_;

struct spu_img_struct {
	u_char *data;		// 7-4: mixer key, 3-0: color index
	u_int x;		// x start of subpicture area
	u_int y;		// y start of subpicture area
	int width;		// width of subpicture area
	int height;		// height of subpicture area
	u_char clut_index[4];		// color lookup table
	u_char trans[4];	// mixer key table

// DENT: also need to have active areas here (for menu stuff - set by nav-pkts)
	guint time_execute;	// time in ms

	clut_t_ *clut;           // YUV-CLUT

// private stuff
	u_int	_x;		// current destination x, y
	u_int	_y;
	u_int	offset[2];	// address in PXD to fetch next rle-code from, one per field
};


typedef struct spu_clut_struct {
#ifdef WORDS_BIGENDIAN
guint8	entry0	: 4;
guint8	entry1	: 4;
guint8	entry2	: 4;
guint8	entry3	: 4;
#else
guint8  entry2  : 4;
guint8  entry3  : 4;
guint8  entry0  : 4;
guint8  entry1  : 4;
#endif
} spu_clut_t;

extern u_int spu_clut[16];/* = 
{
    0xffffff,
    0xE0E0E0, //0xff0000,
    0xffffff,
    0x000000,
    0x005555,
    0x00ff00,
    0xcc2255,
    0xcc0055,
    0x404040,
    0x202020,
    0xb0b0b0,
    0xd0d0d0,
    0x606000,
    0x707000,
    0x808000,
    0x606000,
};
*/

void spuInit (void);
void decode_spu (u_char *data_start, u_char *data_end);
u_int buffer_spupack (u_int *length, u_char **start, u_char *end);
int spuParseHdr (struct spu_img_struct *spu, u_char *pkt_data, u_int pkt_len);
void spuParseData (struct spu_img_struct *spu);

#endif
