#include "../config.h"
#include <gnome.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "drip.h"
#include "corbashared.h"
#include <pthread.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dvddb.h>
#include "../ifo/libifo/ifo.h"
#include <sys/ioctl.h>
#include "decss.h"
#include <inttypes.h>
#include "cdinfo_ext.h"
#include <version.h> /* avifile */

/* GLOBALS */
GtkWidget *propertybox1 = NULL;
GtkWidget *select_dialog;
gboolean converting = FALSE;
gboolean paused = FALSE;
gboolean cacheing = FALSE;
gint selected_files = -1;
GString *files[1000];
GString *cached_files;
GtkWidget *spinbutton1;
GtkWidget *spinbutton2;
GtkWidget *spinbutton3;
GtkWidget *entry3;
GtkWidget *stop_dialog = NULL;
GtkWidget *audio_subpicture_dialog;
u_int spu_clut[16];

/* converting thread */
pthread_t ripthread;
gint ripthread_id;
#ifdef LM_SENSORS
/* heat monitor thread */
pthread_t heatthread;
gint heatthread_id;
#endif /* LM_SENSORS */

gint largest_vob_cached = 0;
GString *video_location;



/* FUNTIONS */

gboolean parse_clut(ifo_t *ifo) {
    /* Fetch CLUT */
    gulong R,G,B;
    gfloat Y,CR,CB,Rf,Gf,Bf;
    const pgc_t *pgc;
    const clut_t *clut;
    ifo_hdr_t *hdr;
    gint title,num,result,fd;
    GIOChannel *clutfile_channel;

    hdr = (ifo_hdr_t *) ifo->tbl[ID_TITLE_PGCI];
    //for (title=0;title<hdr->num; title++) {
        title = 0;
        if ((pgc = ifoGetPGCI (ifo, IFO_TITLE, title))) { //TODO: use right title number instead of '0'
            clut = pgc->clut;
            for (num=0;num<16;num++) {
                Y = clut->y; CB = clut->cb; CR = clut->cr;
                // YUV = YCbCr
                Bf = 1.164*(Y-16)+2.018*(CB-128);
                Gf = 1.164*(Y-16)-0.813*(CR-128)-0.391*(CB-128);
                Rf = 1.164*(Y-16)+1.596*(CR-128);

                B = (uint)(Bf) & 255;
                G = (uint)(Gf) & 255; 
                R = (uint)(Rf) & 255;
                spu_clut[num] = (R) + (G*256) + (B*256*256); // BGR 24bits
                printf("Colour %i : BGR %x\n",num,spu_clut[num]);
                clut++;
            }
        }
    //}

    /* Create cache dir */
    g_string_sprintf(Config.clutfile,"mkdir -p %s",Config.CACHElocation->str);
    drip_system(Config.clutfile->str,&result,TRUE);

    /* Write out clut data */
    g_string_sprintf(Config.clutfile,"%s/%s",Config.CACHElocation->str,"CLUT.tbl");
    fd = open(Config.clutfile->str,O_WRONLY|O_CREAT,S_IREAD|S_IWRITE);
    clutfile_channel = g_io_channel_unix_new(fd);

    for (num=0;num<16;num++) {
        g_io_channel_write(clutfile_channel, (gchar*)&spu_clut[num],sizeof(u_int),&result);
    }

    /* Clean & Exit */
    fflush(NULL);
    g_io_channel_close(clutfile_channel);
    close(fd);
    return TRUE;
}
gboolean parse_ifo(gchar *ifo_file) {
    /* Parse IFO file, for now it handles:
     * - audio channels + language code
     * - subpicture + language code
     *
     * TODO: angles\others..
     */
    ifo_t *ifo;
    gint fd,i;
    GString *multiple = NULL;

    fd = open (ifo_file, O_RDONLY);
    if (fd < 0) {
        g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"Error opening ifo file |%s|", ifo_file);
        return -1;
    }

    ifo = ifoOpen (fd, 0);
    if (ifo == NULL) {
        g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"Error initializing ifo file |%s|",ifo_file);
        return -1;
    }

    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Opening ifo file |%s|",ifo_file);
    if (!ifoIsVTS (ifo)) {
        uint8_t *ptr;
        gint num;
        multiple = g_string_new("");

        /* Dump video to log file */
        ifoPrintVideo (ifo->tbl[ID_MAT] + IFO_OFFSET_VIDEO,&Config);

        /* -- Number of audio channels -- */
        if ((num = ifoGetAudio (ifo->tbl[ID_MAT] + IFO_OFFSET_AUDIO, &ptr)) > 0)
            ifoPrintAudio (ptr, num,&Config);
        /* check if audio channels are valid */
        for (i=num-1;i>-1;i--) {
            if (Config.audio_channel_names[i]->len == 0) {
                num--;
            }
        }
        /* Display result and set Config */
        if (num>1 || num==0) {
            g_string_sprintf(multiple,"%s","s");
        } else {
            g_string_sprintf(multiple,"%s","");
        }
        g_log(DRIP_LD,G_LOG_LEVEL_INFO,"Stream contains %i audio channel%s",num,multiple->str);
        Config.audio_channels_in_dvd = num;

        /* -- Number of subpicture streams -- */
        if ((num = ifoGetSPU (ifo->tbl[ID_MAT] + IFO_OFFSET_SUBPIC, &ptr)) > 0)
            ifoPrintSPU ((ifo_spu_t *) ptr, num,&Config);
        /* check if subpictures are valid */
        for (i=num-1;i>-1;i--) {
            if (Config.subpicture_names[i]->len == 0) {
                num--;
            }
        }
        /* Display result and set Config */
        if (num>1 || num==0) {
            g_string_sprintf(multiple,"%s","s");
        } else {
            g_string_sprintf(multiple,"%s","");
        }
        g_log(DRIP_LD,G_LOG_LEVEL_INFO,"Stream contains %i subpicture%s",num,multiple->str);
        Config.subpictures_in_dvd = num;

        /* Parse CLUT */
        parse_clut(ifo);
    }

    /* Clean & Exit */
    ifoClose (ifo);
    fflush(NULL);
    close(fd);
    if (multiple) { g_string_free(multiple,TRUE);}
    return TRUE;
}


gboolean save_config() {
    /* Save configuration */
    GString *configfile;
    GIOChannel *configfile_channel;
    guint bytesloaded;
    gint saveerror;
    gint fd;
    gboolean true_udf = FALSE;

    configfile = g_string_new("");

    /* delete old config */
    g_string_sprintf(configfile,"rm %s/.drip.cfg",getenv("HOME"));
    unlink(configfile->str);
    /* create new one */
    g_string_sprintf(configfile,"%s/.drip.cfg",getenv("HOME"));

    fd = open(configfile->str,O_WRONLY|O_CREAT,S_IREAD|S_IWRITE);
    configfile_channel = g_io_channel_unix_new(fd);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.DVDdevice->len,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,Config.DVDdevice->str,Config.DVDdevice->len,&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.DIVXfile->len,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,Config.DIVXfile->str,Config.DIVXfile->len,&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.width,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.height,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.video_bitrate,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.video_stream,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.audio_bitrate,sizeof(gint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.audio_stream,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.codec,sizeof(gint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.cliptop,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.clipbottom,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.blurradius,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.cache,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.eject_dvdrom,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.use_dvddb,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.nice,sizeof(gint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.full_movie_rip_mode,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.maxsize,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&true_udf,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.encode,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.DVDsource->len,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,Config.DVDsource->str,Config.DVDsource->len,&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.useDVDdevice,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.cpus,sizeof(gint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.done_warning,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.CACHElocation->len,sizeof(glong),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,Config.CACHElocation->str,Config.CACHElocation->len,&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.cache_delete_on_exit,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.create_iso,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.hyper_zoom,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.autoclip,sizeof(gboolean),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.clipright,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.clipleft,sizeof(guint),&bytesloaded);
    saveerror = g_io_channel_write(configfile_channel,(gchar*)&Config.honor_aspect,sizeof(gboolean),&bytesloaded);


    /* Clean & exit */
    fflush(NULL);
    g_io_channel_close(configfile_channel);
    close(fd);
    g_string_free(configfile,TRUE);
    //g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Saved configuration");
    return TRUE;
}

void
drip_error(gchar *text) {
    GtkWidget *warning_window;
    //gdk_threads_enter();
    warning_window = create_infobox(text);
    gtk_widget_show(warning_window);
    //gdk_threads_leave();
}

void
drip_info(gchar *text) {
    GtkWidget *info_window = create_infobox(text);
    gtk_widget_show(info_window);
    return;
}

/* system() wrapper */
GString* drip_system(gchar *command,gint *exec_result,gboolean freebuffer) {
    GString *format = g_string_new("");
    GString *result = g_string_new("");
    GString *wrapperfile = g_string_new("");
    GIOChannel *pipefile;
    glong pipefile_size;
    guint bytessaved;
    gint loaderror;
    gchar *buf;
    gint fd;
    struct stat file_stats;

    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"executing |%s|",command);

    /* build wrapper filename */
    g_string_sprintf(wrapperfile,"/tmp/drip.system.wrapper.%d.tmp",getpid());

    /* execute system command */
    g_string_sprintf(format,"%s 1>%s 2>&1",command,wrapperfile->str);
    *exec_result = system(format->str);

    /* read system command output */
    fd = open(wrapperfile->str,O_RDONLY);
    pipefile = g_io_channel_unix_new(fd);

    stat(wrapperfile->str, &file_stats);
    pipefile_size = file_stats.st_size - 1; 
    if (pipefile_size>0 && pipefile_size<10000000) {
        buf = g_malloc0(pipefile_size+1);
        loaderror = g_io_channel_read(pipefile,buf,pipefile_size,&bytessaved);
        g_string_sprintf(result,"%s",buf);
        g_free(buf);
    } else {
        g_string_sprintf(result,"{empty output}");
    }

    g_io_channel_close(pipefile);
    fflush(NULL);
    close(fd);
    unlink(wrapperfile->str);
    g_string_free(wrapperfile,TRUE);
    g_string_free(format,TRUE);
    if (freebuffer == TRUE) {
        g_string_free(result,TRUE);
        return NULL;
    } else {
        return result;
    }
}

/* Threaded wrapper for drip_system */
gpointer drip_system_threaded(gpointer data) {
    drip_system_t* ds_data = (drip_system_t*)data;
    drip_system(ds_data->command,ds_data->exec_result,ds_data->freebuffer);
    pthread_exit(0);
    return NULL;
}

/* Terminates drip, should do clean shutdown  */
void terminate(gboolean hard) {
    GString *str = g_string_new("");
    if (converting && hard==FALSE) {
        GtkWidget* buzy;
        buzy = create_messagebox_buzy();
        gtk_widget_show(buzy);
    } else {
        save_config();
        unlink("dripgui.ior");
        unlink("dripencoder.ior");
        unlink("title-key");
        unlink("disc-key");

        /* Remove cache files */
        if (Config.cache_delete_on_exit) {
            g_string_sprintf(str,"rm %s*.VOB",Config.CACHElocation->str);
            system(str->str);
            g_string_sprintf(str,"rm %s*.vob",Config.CACHElocation->str);
            system(str->str);
        }
        corba = FALSE;
        g_log_remove_handler(DRIP_LD,log_handler_drip_id);
        close(log_file);
        if (encoder_client) { 
            drip_encoder_quit(encoder_client,&ev);
            CORBA_Object_release(encoder_client,&ev);
        }
        exit(0);
    }

    /* Clean & Exit */
    g_string_free(str,TRUE);
    return;
}

void
on_save_preferences_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
    save_config();
    gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Done saving.");
    return;
}

void
on_ripdvd_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
    on_button_main_rip_clicked((GtkButton*)menuitem,user_data);
    return;
}

void
on_button_calculate_bitrate_clicked    (GtkButton       *button,
                                        gpointer         user_data) {
    GtkWidget *calc_window;

    calc_window = create_calculate_bitrate();
    gtk_widget_show(calc_window);
    return;
}

void
on_rescan_for_dvd_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
    on_button_main_rescan_clicked(NULL,NULL);
    return;
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    terminate(FALSE);
    return;
}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    /* show properties interface */
    if (propertybox1 == NULL) {
        propertybox1 = create_propertybox1 ();
        gtk_widget_show (propertybox1);
    } 
    return;
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* aboutbox;
    aboutbox = create_about_drip();
    gtk_widget_show(aboutbox);
    return;
}

gchar*
mpeg2divx_parameters()
{
    GString *param = g_string_new("");
    gchar *temp = g_malloc(65530);
    
    sprintf(temp,"-b %i ",Config.video_bitrate);
    param = g_string_append(param,temp);

    if (Config.video_stream != 0) {
        sprintf(temp,"-vs %i ",Config.video_stream);
        param = g_string_append(param,temp);
    }

    if (Config.audio_bitrate != -1) {
        sprintf(temp,"-a %i ",Config.audio_bitrate);
        param = g_string_append(param,temp);
    }

    if (Config.audio_stream != 0) {
        sprintf(temp,"-as %i ",Config.audio_stream);
        param = g_string_append(param,temp);
    }

    sprintf(temp,"-w %li ",Config.width);
    param = g_string_append(param,temp);

    sprintf(temp,"-h %li ",Config.height);
    param = g_string_append(param,temp);

    if (Config.codec == 0) { /* fast_motion MSMPEG4 */
        param = g_string_append(param,"-f ");
    }

    if (Config.codec == 2) { /* OpenDIVX */
        param = g_string_append(param,"-d ");
    }

    if (Config.cliptop != 0) {
        sprintf(temp,"-ct %i ",Config.cliptop);
        param = g_string_append(param,temp);
    }

    if (Config.clipleft != 0) {
        sprintf(temp,"-cl %i ",Config.clipleft);
        param = g_string_append(param,temp);
    }

    if (Config.clipright != 0) {
        sprintf(temp,"-cr %i ",Config.clipright);
        param = g_string_append(param,temp);
    }

    if (Config.clipbottom != 0) {
        sprintf(temp,"-cb %i ",Config.clipbottom);
        param = g_string_append(param,temp);
    }

    if (Config.blurradius > 0) {
        sprintf(temp,"-bf %i ",Config.blurradius);
        param = g_string_append(param,temp);
    }

    if (Config.cpus > 0) {
        sprintf(temp,"-n %i ",Config.cpus);
        param = g_string_append(param,temp);
    }

    if (Config.subpicture_stream>-1) {
        sprintf(temp,"-s %i %s ",Config.subpicture_stream,Config.clutfile->str);
        param = g_string_append(param,temp);
    }

    if (Config.hyper_zoom == TRUE) {
        sprintf(temp,"-z ");
        param = g_string_append(param,temp);
    }

    if (Config.autoclip == TRUE) {
        sprintf(temp,"-ac ");
        param = g_string_append(param,temp);
    }

    if (Config.honor_aspect == TRUE) {
        sprintf(temp,"-xy %g 1000 ",(1000*Config.aspect));
        param = g_string_append(param,temp);
    }

    sprintf(temp,"-l %lu",Config.maxsize*1024*1024);
    param = g_string_append(param,temp);

    g_free(temp);
    return param->str;
}

gint dirfilter_all_vob(const struct dirent *entry) {
    /* checks if a vob file is a rip candidate */
    gchar *result1;
    gchar *result2;
    gchar *result3;
    result1 = strstr(entry->d_name,".vob");
    result2 = strstr(entry->d_name,".VOB");
    result3 = strstr(entry->d_name,".subpicture");
    if (result3!=NULL)
        return 0;
    if ((result1!=NULL) || (result2!=NULL))
        return 1;
    else
        return 0;
}

gint dirfilter_all(const struct dirent *entry) {
    /* true on any file on disk */
    if (strstr(entry->d_name,".")==entry->d_name)
        return 0;
    else
        return 1;
}

gint detect_largest_vob_sequence(gchar *video_location) {
    /* check all VTS_*_[1-N].vob, group and calculate the largest */
    struct dirent **vobsdir;
    GString *sequence = g_string_new("");
    GString *sequence_previous = g_string_new("");
    GString *fullname = g_string_new("");
    struct stat file_stats;
    gchar *sequence_string;
    guint64 sequence_bytes = 0;
    gint sequence_integer = 0;
    guint64 sequence_largest = 0;
    gint sequence_largest_nr = -1;
    gint i,n;

    /* already done this? */
    if (largest_vob_cached>0) {
        return largest_vob_cached;
    }

    /* Get all */
    i = scandir (video_location,&vobsdir,dirfilter_all_vob,alphasort);
    for (n=0;n<i;n++) {
        sequence_string = g_strndup((gchar*)(vobsdir[n]->d_name+4),2);
        g_string_sprintf(sequence,"%s",sequence_string);
        g_free(sequence_string);
        sequence_integer = (gint)g_strtod(sequence->str,NULL);
        if (sequence_integer>0 && strstr(vobsdir[n]->d_name,"_0.")==NULL) {
            /* Got a valid sequence file */
            g_string_sprintf(fullname,"%s%s",video_location,vobsdir[n]->d_name);
            stat(fullname->str, &file_stats);
            if (strcmp(sequence->str,sequence_previous->str)==0) {
                /* file belongs to same sequence */
                sequence_bytes += (guint64)file_stats.st_size;
            } else {
                /* new sequence */
                if (sequence_bytes>sequence_largest) {
                    /* store largest sequence */
                    sequence_largest = sequence_bytes;
                    sequence_largest_nr = sequence_integer-1;
                }
                sequence_bytes = file_stats.st_size;
                g_string_sprintf(sequence_previous,"%s",sequence->str);
            }
        }
    }

    /* check last sequence */
    if (sequence_bytes>sequence_largest) { 
        /* store largest sequence */
        sequence_largest = sequence_bytes;
        sequence_largest_nr = sequence_integer;
    }

    /* Clean & exit */
    largest_vob_cached = sequence_largest_nr;
    g_log(DRIP_LD,G_LOG_LEVEL_INFO,"Suggesting sequence %i (VTS_%i_*.VOB)",sequence_largest_nr, sequence_largest_nr);
    g_string_free(sequence,TRUE);
    g_string_free(sequence_previous,TRUE);
    g_string_free(fullname,TRUE);
    return largest_vob_cached;
}


gint dirfilter_movie_vob(const struct dirent *entry) {
    /* checks if a vob file is a rip candidate */
    gint N = 0;
    gchar *result1,*result1b,*result2,*result2b,*result3,*result3b,*result4;
    GString *sequence = g_string_new("");

    /* which sequence of vobs is largest */
    N = detect_largest_vob_sequence(video_location->str);

    /* movie determination */
    g_string_sprintf(sequence,"%i",N);
    if (sequence->len == 0) { g_string_sprintf(sequence,"01");} /* detetction failed? */
    if (sequence->len == 1) { g_string_prepend(sequence,"0");} /* add 0 to text name */
    sequence = g_string_prepend(sequence,"vts_");
    sequence = g_string_append(sequence,"_");

    /* check if name matches */
    result1 = strstr(entry->d_name,sequence->str);
    g_strup(sequence->str);result1b = strstr(entry->d_name,sequence->str);
    result2 = strstr(entry->d_name,".vob");
    result2b = strstr(entry->d_name,".VOB");

    /* dont use _0 of sequence */
    result3 = strstr(entry->d_name,"_0.vob");
    result3b = strstr(entry->d_name,"_0.VOB");

    result4 = strstr(entry->d_name,"subpicture"); /* TODO: can be deleted? */
    if (result4!=NULL)
        return 0;

    if ((result1==NULL && result1b==NULL)||(result2==NULL && result2b==NULL)||(result3!=NULL || result3b!=NULL))
        return 0;
    else
        return 1;
}

gint dirfilter_divx(const struct dirent *entry) {
    /* filters on .avi */
    gchar *result;
    result = strstr(entry->d_name,".avi");

    if (result==NULL)
        return 0;
    else
        return 1;
}

gint manual_stream_select(gchar *video_location) {
    /* allowes the user to manually select streams that are to be encoded */
    gint i;
    struct dirent **vobsdir;

    selected_files = -1; /* convertion thread polls this variable,
                           -1  = not ready selecting,
                           >-1 = selected number of files,
                           files are stored in namelist */
    i = scandir (video_location,&vobsdir,dirfilter_all,alphasort);

    gdk_threads_enter();
    select_dialog =  create_manual_selection_dialog(i,vobsdir,video_location);
    gtk_widget_show(select_dialog);
    gdk_threads_leave();
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Manual stream selection mode");
    return 0;
}


gchar* readlink_malloc (gchar *filename) {
    gint size = 100;

    while (1) {
        char *buffer = (char *) g_malloc0 (size);
        int nchars = readlink (filename, buffer, size);

        /* Is filename a symlink? */
        if (nchars == -1) {
            g_free(buffer);
            return NULL; /* No */
        }

        /* Do we have a good result so far? */
        if (nchars < size) {
            //buffer[size] = 0;
            return buffer; /* Yes */
        }

        g_free (buffer);
        size *= 2;
    }

    /* dummy */
    return NULL;
}


gchar* detect_source_track_dir() {
    GString *command = g_string_new("");
    GString *track_dir = g_string_new("");
    gint result = 0;
    gchar* eol;

    /* Are we about to use a DVDdevice or just source files */
    if (Config.useDVDdevice) {
        g_string_sprintf(command,"cat /etc/fstab | grep -w %s | awk '{print($2)}'",Config.DVDdevice->str);
        g_string_sprintf(track_dir,"%s",drip_system(command->str,&result,FALSE)->str);
        if (track_dir->len < 2 || strncmp(track_dir->str,"{empty",6)==0) {
            //gdk_threads_enter();
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Ready");
            //gdk_threads_leave();
            g_string_sprintf(command,"Your DVD device %s is not listed in /etc/fstab",Config.DVDdevice->str);
            drip_error(command->str);
            return NULL;
        }
        //track_dir = g_string_truncate(track_dir,track_dir->len-1);
        /* Fetch info of CD\DVD disk, fill various fields of Config */
        cdinfo(&Config);
        /* Display info */
        if (Config.DVDvolumeid->len > 0) {
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,"DVD volume id %s",Config.DVDvolumeid->str);
        }
        if (Config.DVDpublisher->len > 0) {
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,"DVD publisher %s",Config.DVDpublisher->str);
        }
        if (Config.DVDpreparer->len > 0) {
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,"DVD preparer %s",Config.DVDpreparer->str);
        }
        if (Config.DVDcreationdate->len > 0) {
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,"DVD creation date %s",Config.DVDcreationdate->str);
        }
        if (Config.DVDvolumesize_MB > 0) {
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,"DVD volume size %luMB",Config.DVDvolumesize_MB);
        }
        /* Check for free HD space if caching */
        if (Config.cache == TRUE) {
            //TODO
        }
    } else {
        /* Use user entered dir for source */
        g_string_sprintf(track_dir,"%s",Config.DVDsource->str);
    }

    /* Test track_dir for chr 10 at eol */
    eol = (gchar*)((gulong)track_dir->str + track_dir->len - 1);
    if (*eol == (gchar)10) {
        //track_dir = g_string_truncate(track_dir,track_dir->len-1);
    }
    /* clean up & exit */
    g_string_free(command,TRUE);
    return track_dir->str;
}

void
rip_track(gchar *dir,gchar *track,gfloat perc,guint nr,gint totalfiles) {
    /* Reads DVD stream, converts to plain mpeg2 it and writes it to HD */
    GString *command;
    GString *result;
    gint result_int = 0;
    struct timeval tv;
    struct timezone tz={0,0};
    glong startsec;
    glong startusec;
    struct stat input_file_stats;
    struct stat output_file_stats;
    gfloat mps = 0;

    gettimeofday(&tv,&tz);
    startsec=tv.tv_sec;
    startusec=tv.tv_usec;
    result = g_string_new("");

    /* Check if file is a VOB */
    if (strcasecmp("VOB",(gchar*)(track + strlen(track) - 3))!=0) {
        g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"File %s isn't a VOB file, skipping.",track);
        return;
    }

    /* get track key */
    command = g_string_new("");
    g_string_sprintf(command,"%s%s",dir,track);
    stat(command->str, &input_file_stats);
    tstdvd_file(command->str);

    /* convert track to plain mpeg2 */
    g_string_sprintf(command,"%s%s",dir,track);
    g_string_sprintf(result,"%s%s",Config.CACHElocation->str,track);
    g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Caching track %s to %s",command->str,result->str);
    result_int = css_cat(command->str,result->str,perc,totalfiles);
    stat(result->str, &output_file_stats);
    if (output_file_stats.st_size<1) {
        g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Can not cache %s to %s, DVD disk needs cleaning?",command->str,Config.CACHElocation->str);
    } else {
        /* display caching speed */
        gettimeofday(&tv,&tz);
        mps=input_file_stats.st_size/((tv.tv_sec+tv.tv_usec/1000000.0)-(startsec+startusec/1000000.0));
        g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Read %iMb at %.3fMb/sec and dropped %g%%",(int)(input_file_stats.st_size/(1024*1024)),mps/1048576,((gfloat)100./(gfloat)input_file_stats.st_size*((gfloat)input_file_stats.st_size-(gfloat)output_file_stats.st_size)));

        /* Check sanity of result:
         * - if result < 80% input then there might be a problem (we take all mpeg2 blocks, so result is ??100%?? )
         * - if result < 70% input then there is a problem
         * - if result > input then there is a problem
         */
        if (output_file_stats.st_size > input_file_stats.st_size) {
            g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Cached file %s is too large",result->str);
        } else {
            if (output_file_stats.st_size < (input_file_stats.st_size*0.7)) {
                g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Cached file %s is too small",result->str);
            } else {
                if (output_file_stats.st_size < (input_file_stats.st_size*0.8)) {
                    g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Cached file %s is smaller as exspected",result->str);
                }
            }
        }
    }

    /* list of cached files */
    cached_files = g_string_append(cached_files,Config.CACHElocation->str);
    cached_files = g_string_append(cached_files,track);
    cached_files = g_string_append(cached_files," ");

    /* Clean & exit */
    g_string_free(command,TRUE);
    g_string_free(result,TRUE);
    return;
}

gboolean
format_divxname(GString *DIVXfile) {
    GString *temp = g_string_new("");
    /* format filename to (name)-(videorate)-(audiorate).avi */

    /* ".avi" is end of name already? */
    if (strncasecmp(DIVXfile->str + (DIVXfile->len-4),".avi",4) == 0) {
        /* truncate .avi */
        DIVXfile = g_string_truncate(DIVXfile,DIVXfile->len-4);
    }

    /* Add video rate to name */
    g_string_sprintf(temp,"%i",Config.video_bitrate);
    if (strstr(DIVXfile->str,temp->str)!=NULL) {
        /* name already contains video bitrate */
    } else {
        g_string_sprintf(temp,"-%i",Config.video_bitrate);
        DIVXfile = g_string_append(DIVXfile,temp->str);
    }

    /* Add audio rate to name */
    g_string_sprintf(temp,"%i",Config.audio_bitrate);
    if (strstr(DIVXfile->str,temp->str)!=NULL) {
        /* name already contains audio bitrate */
    } else {
        g_string_sprintf(temp,"-%i",Config.audio_bitrate);
        DIVXfile = g_string_append(DIVXfile,temp->str);
    }

    /* Add ".avi" to name */
    DIVXfile = g_string_append(DIVXfile,".avi");
    g_string_sprintf(temp,"%s",DIVXfile->str);
    g_string_sprintf(Config.DIVXfile,"%s",temp->str);

    g_string_free(temp,TRUE);
    return TRUE;
}


gboolean
config_ok() {
    /* checks if the Config structure is filled with same data before 
       the dvd->divx conversion is initiated */

    /* 1: MPEG-1, 0: MPEG-2 LSF, 1995-07-11 shn */
    gint bitrate_table[2][16] = {
              {-1,0,8,16,24,32,40,48,56,64,80,96,112,128,144,160},
              {-1,0,32,40,48,56,64,80,96,112,128,160,192,224,256,320}};
    gint index = 0;
    gboolean found = FALSE;

    /* check audio rate */
    while(!found && index<16) {
        if(bitrate_table[1][index] == Config.audio_bitrate) /* dvd = mpeg2 */
            found = TRUE;
        else
            ++index;
    }
    return found;
}

gboolean
user_can_mount_dvd() {
    GString *command = g_string_new("");
    GString *mountoptions = g_string_new("");
    gint result = 0;
    gboolean allowed = FALSE;

    g_string_sprintf(command,"cat /etc/fstab | grep -w %s | awk '{print($4)}'",Config.DVDdevice->str);
    g_string_sprintf(mountoptions,"%s",drip_system(command->str,&result,FALSE)->str);

    /* check if mountoptions contains 'user' and not 'nouser' */
    if (strstr(mountoptions->str,"nouser")) {
        allowed = FALSE;
    } else {
        if (strstr(mountoptions->str,"user")) {
            allowed = TRUE;
        }
    }

    /* clean & exit */
    g_string_free(command,TRUE);
    g_string_free(mountoptions,TRUE);
    return allowed;
}


gint dirfilter_avi(const struct dirent *entry) {
/* returns *.avi */
    gchar *result1;
    gchar *result2;
    gchar **name_parts;
    GString *basename = g_string_new("");
    gint i;

    /* Split avi filename */
    name_parts = g_strsplit(Config.DIVXfile->str,"/",0);

    /* basename, name without '.avi' and basedir */
    i = 0; while (name_parts[i+1]!=NULL) { i++; }
    g_string_sprintf(basename,"%s",name_parts[i]);
    basename = g_string_truncate(basename,basename->len-4);

    /* check if name matches *.avi */
    result1 = strstr(entry->d_name,".avi");
    result2 = strstr(entry->d_name,basename->str);
    if (result1==NULL || result2==NULL) {
        return FALSE;
    } else {
        return TRUE;
    }
}

void
rename_divx_files() {
/* Renames divx files to proper numbering (like : 'I_of_II') */
    struct dirent **avi_files;
    gint avi_files_number,i,result;
    GString *basename = g_string_new("");
    GString *basedir = g_string_new("");
    GString *newname = g_string_new("");
    GString *command = g_string_new("");
    gchar **name_parts;

    /* Split avi filename */
    name_parts = g_strsplit(Config.DIVXfile->str,"/",0);

    /* basename, name without '.avi', basedir */
    i = 0; while (name_parts[i+1]!=NULL) {
        basedir = g_string_append(basedir,name_parts[i]);
        basedir = g_string_append(basedir,"/");
        i++;
    }
    if (basedir->len < 2) {
        g_string_sprintf(basedir,"./");
    }

    g_string_sprintf(basename,"%s",name_parts[i]);
    basename = g_string_truncate(basename,basename->len-4);

    /* Find out how many chunks the result is spread over */
    avi_files_number = scandir (basedir->str,&avi_files,dirfilter_avi,alphasort);
 
    /* rename iteration */
    g_string_sprintf(newname,"%s%s.%iof%i.avi",basedir->str,basename->str,1,avi_files_number);
    g_string_sprintf(command,"mv %s%s %s",basedir->str,avi_files[avi_files_number-1]->d_name,newname->str);
    drip_system(command->str,&result,TRUE);

    for (i=0;i<avi_files_number-1;i++) {
        g_string_sprintf(newname,"%s%s.%iof%i.avi",basedir->str,basename->str,i+2,avi_files_number);
        g_string_sprintf(command,"mv %s%s %s",basedir->str,avi_files[i]->d_name,newname->str);
        drip_system(command->str,&result,TRUE);
    }

    /* Clean & Exit */
    g_string_free(newname,TRUE);
    g_string_free(basename,TRUE);
    g_string_free(command,TRUE);
    return;
}

gboolean eject_dvdrom(gchar *track_dir,gchar *dvddevice) {
    /* Ejects DVDrom device and displays some info */
    GString *command = g_string_new("");
    gboolean closed;

    /* */ gdk_threads_leave(); /* */

    /* Close opens on DVD */
    closed = tstdvd_close();

    gdk_threads_enter();
    gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Ejecting DVDrom");
    gnome_appbar_set_progress(GNOME_APPBAR(appbar1),0);
    gdk_threads_leave();
    g_string_sprintf(command,"umount %s",track_dir);
    system(command->str);
    g_string_sprintf(command,"eject %s",dvddevice);
    system(command->str);

    g_string_free(command,TRUE);
    return TRUE;
}

gpointer make_iso_images(gpointer data) {
    /* Builds iso images out of the generated divx's */
    gint i;
    gint selected_files = 0;
    struct dirent **divxdir;
    GString *command = g_string_new("");
 
    selected_files = scandir(".",&divxdir,dirfilter_divx,alphasort);
    for (i=0;i<selected_files;i++) {
        g_string_sprintf(command,"Building ISO image %s.iso",divxdir[i]->d_name);
        g_log(DRIP_LD,G_LOG_LEVEL_INFO,command->str);
        g_string_sprintf(command,"mkisofs -J -l -T -R %s > %s.iso",divxdir[i]->d_name,divxdir[i]->d_name);
        system(command->str);
        unlink(divxdir[i]->d_name);
    }

    making_iso = FALSE;
    g_string_free(command,TRUE);
    pthread_exit(0);
    return NULL;
}

gboolean cache_vobs_for_ifo() {
/* Cache VOB file needed for selected ifo's */
    gint j,i,result;
    struct stat vobstat;
    GString *extension;
    GString *sequence;
    GString *vob;
    GString *vobnamestat;

    vobnamestat = g_string_new("");
    for (j=0;j<selected_files;j++) {
        /* Video input is a IFO file? -> precalculate total frames */
        extension = g_string_new((gchar*)(files[j]->str + files[j]->len - 3));
        if (strcasecmp("IFO",extension->str)==0) {
            sequence = g_string_new((gchar*)(files[j]->str + files[j]->len - 8));
            sequence = g_string_truncate(sequence,2);
            vob = g_string_new("");
            if (strcmp(extension->str,"IFO")==0) {
                g_string_sprintf(vob,"VTS_%s_1.VOB",sequence->str);
            } else {
                g_string_sprintf(vob,"vts_%s_1.vob",sequence->str);
            }

            /* Cache VOB */
            rip_track(video_location->str,vob->str,0,1,selected_files);

            /* Loop through all VOB files and calculate total frames */
            i = 2;
            do {
                if (strcmp(extension->str,"IFO")==0) {
                    g_string_sprintf(vob,"VTS_%s_%i.VOB",sequence->str,i);
                    
                } else {
                    g_string_sprintf(vob,"vts_%s_%i.vob",sequence->str,i);
                }

                g_string_sprintf(vobnamestat,"%s%s",video_location->str,vob->str);
                result = stat(vobnamestat->str, &vobstat);
                if (result == 0) {
                    /* Cache VOB */
                    rip_track(video_location->str,vob->str,0,1,selected_files);
                    i++;
                }
            } while (result == 0);

            /* Clean */
            g_string_free(sequence,TRUE);
            g_string_free(vob,TRUE);
        }
        g_string_free(extension,TRUE);
    }
    g_string_free(vobnamestat,TRUE);

    return TRUE;
}


/* Checks dvdname for default dvdnames etc */
gboolean dvdname_check(GString* dvdname) {
    /* Empty? */
    if (dvdname->len == 0) return FALSE;

    /* Default preparation names */
    if (strstr(dvdname->str,"DVD_VIDEO")!=NULL) return FALSE;
    if (strstr(dvdname->str,"DVDVOLUME")!=NULL) return FALSE;
    // TODO: add more?? 

    /* Checks passed, name OK */
    return TRUE;
}

gpointer
conversion_thread(gpointer data) {
    GString *command;
    GString *temp;
    GString *ifo;
    gint i,cnt;
    gint result = 0;
    struct dirent **vobsdir;
    //gchar *symlink;
    command = g_string_new("");
    temp = g_string_new("");

    
    /* conversion started: mark flag */
    converting = TRUE;
    cached_files = g_string_new("");

    /* Debug */
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"conversion_thread, pid = %i",getpid());
    
    /* check configuration */
    if (!config_ok()) {
        g_string_sprintf(temp,"Bitrate %ikbs not legal for output sampling.\n",Config.audio_bitrate);
        drip_error(temp->str);
        converting = FALSE;
        g_string_free(command,TRUE);
        g_string_free(temp,TRUE);
        pthread_exit(0);
    }

    /* check dvd device */
    g_string_sprintf(Config.track_dir,"%s",detect_source_track_dir());
    if (Config.track_dir == NULL) {
        converting = FALSE;
        g_string_free(command,TRUE);
        g_string_free(temp,TRUE);
        pthread_exit(0);
    }

    /*
     * mount DVDrom, 
     * or check if mounted when not running as root and no 'user' option in /etc/mtab 
     */
    if (Config.useDVDdevice) {
        if ((getuid()==0 || user_can_mount_dvd()==TRUE)) {
            /* we need to mount and are allowed to */
            gdk_threads_enter();
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Mounting DVDrom");
            gdk_threads_leave();
            /* unmount first */
            g_string_sprintf(command,"umount %s",Config.track_dir->str);
            temp = drip_system(command->str,&result,FALSE);
            /* mount */
            g_string_sprintf(command ,"mount %s",Config.track_dir->str);
            temp = drip_system(command->str,&result,FALSE);
        } else {
            /* we're not allowed to mount, just check if DVD is mounted */
            g_string_sprintf(command,"mount | grep -w %s | awk '{print($1)}'",Config.DVDdevice->str);
            temp = drip_system(command->str,&result,FALSE);
            if (strstr(temp->str,Config.DVDdevice->str)==NULL) {
                /* not mounted, warn user and terminate conversion process */
                g_string_sprintf(temp,"No DVD disc detected at %s in device %s.\n\nDrip is not able to mount your DVD disk because there's\nno 'user' option in /etc/fstab and Drip isn't running as root,\ndo this manually ('mount %s %s')\n\nBest would be to set the options of the DVD device in\n/etc/fstab to 'ro,nosuid,noauto,user'\n",Config.track_dir->str,Config.DVDdevice->str,Config.DVDdevice->str,Config.track_dir->str);
                drip_error(temp->str);
                converting = FALSE;
                g_string_free(command,TRUE);
                g_string_free(temp,TRUE);
                pthread_exit(0);
            }
        }
    }

    /* Build video stream location */
    video_location = g_string_new(Config.track_dir->str);
    video_location = g_string_append(video_location,"/");
    if (Config.useDVDdevice) {
        g_string_sprintf(command,"ls -lsa %s | grep -i VIDEO_TS | awk '{print($10)}'",Config.track_dir->str);
        video_location = g_string_append(video_location,drip_system(command->str,&result,FALSE)->str);
        //video_location = g_string_truncate(video_location,video_location->len-1);
        video_location = g_string_append(video_location,"/");
    }


    /* check if dvd\source is present at all */
    gdk_threads_enter();
    gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Checking DVDrom");
    gdk_threads_leave();
    g_string_sprintf(command,"ls -lsa %s[Vv][Tt][Ss]*.[Vv][Oo][Bb]",video_location->str);
    temp = drip_system(command->str,&result,FALSE);
    if (result!=0) {
        g_string_sprintf(temp,"No DVD disc detected %s (mount point %s)\nLoad a valid DVD disk or select an other drive",Config.DVDdevice->str,Config.track_dir->str);
        drip_error(temp->str);
        converting = FALSE;
        g_string_free(command,TRUE);
        g_string_free(temp,TRUE);
        pthread_exit(0);
    }

    /* Fetch ifo information */
    g_string_sprintf(command,"ls -lsa %s* | grep -i TS.IFO | awk '{print($10)}'",video_location->str);
    ifo = drip_system(command->str,&result,FALSE);
    parse_ifo(ifo->str);
    g_string_free(ifo,TRUE);

    /* get DVDDB name */
    if (Config.useDVDdevice && Config.encode && Config.use_dvddb) {
        int sock;
        char id[9];

        g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Querying DVDDB...");
        gdk_threads_enter();
        gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Querying DVDDB...");
        gdk_threads_leave();
        g_string_sprintf(temp,"%s","");

        dvddbGetID(Config.DVDdevice->str,id);
        if ((sock = dvddbNetConnect ()) < 0) {
            /* Use hardcoded name, no dvddb name available */
            g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"No DVDdb hit: using hardcoded name");
            if (dvdname_check(Config.DVDvolumeid)==TRUE) { 
                format_divxname(Config.DVDvolumeid);
            } else {
                format_divxname(Config.DIVXfile);
            }
        } else {
            g_string_sprintf(temp,"%s",dvddbNetGet(sock, id));
            if (strstr(temp->str,"(null)")!=NULL || temp->str == NULL || temp->len == 0) {
                /* Use hardcoded name, no dvddb name availeble */
                g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"No DVDdb hit: using hardcoded name");
                if (dvdname_check(Config.DVDvolumeid)==TRUE) {
                    format_divxname(Config.DVDvolumeid);
                } else {
                    format_divxname(Config.DIVXfile);
                }
            } else {
                g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Found DVD: using DVDdb name");
                g_string_sprintf(Config.DIVXfile,"%s-%i-%i.avi",temp->str,Config.video_bitrate,Config.audio_bitrate);
            }
        }
        dvddbNetDisconnect (sock);
    } else {
        /* No DVDDB */
        if (dvdname_check(Config.DVDvolumeid)) {
            format_divxname(Config.DVDvolumeid);
        } else {
            format_divxname(Config.DIVXfile);
        }
    }

    if (Config.encode) {
        g_log(DRIP_LD,G_LOG_LEVEL_INFO,"Generated DIVX filename: %s",Config.DIVXfile->str);
    }

    /* Get .vob files to be coneverted to avi */
    if (Config.full_movie_rip_mode) {
        /* Scan for biggest vts_*.vob collection on DVDrom */
        selected_files = scandir (video_location->str,&vobsdir,dirfilter_movie_vob,alphasort);
        for (i=0;i<selected_files;i++) {
            files[i] = g_string_new(vobsdir[i]->d_name);
        }
    } else {
        /* Manual select .vob files */
        gdk_threads_enter();
        gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Waiting for selection");
        gdk_threads_leave();
        i = manual_stream_select(video_location->str);
        while (selected_files == -1) {
            usleep(200000);
        }
        if (selected_files == 0) {
            converting = FALSE;
            gdk_threads_enter();
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Ready");
            gdk_threads_leave();
            g_string_free(command,TRUE);
            g_string_free(temp,TRUE);
            pthread_exit(0);
        }
    }

    if ((selected_files >= 0) && (Config.encode)) {
        /* Parse VTS*.IFO for audio channels and subpicture information */
        g_string_sprintf(temp,"%s",(gchar*)((glong)files[0]->str + 4));
        temp = g_string_truncate(temp,2);
        g_string_sprintf(command,"ls -lsa %s* | grep -i VTS_%s_0.IFO | awk '{print($10)}'",video_location->str,temp->str);
        ifo = drip_system(command->str,&result,FALSE);
        g_string_sprintf(command,"Using %s for subpicture and audio information",ifo->str);
        g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,command->str);
        parse_ifo(ifo->str);
        g_string_sprintf(command,"Stream aspect ration %g",Config.aspect);
        g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,command->str);

        /* -- Show user dialog for audio channel and subpictures -- */
        if (Config.audio_channels_in_dvd>1 || Config.subpictures_in_dvd>0) {
            gdk_threads_enter();
            audio_subpicture_dialog = create_rip_dialog();
            gtk_widget_show(audio_subpicture_dialog);
            gdk_threads_leave();
            while (audio_subpicture_dialog) { usleep(100000);}
        }
        /* Log languages */
        for (i=0;i<Config.audio_channels_in_dvd;i++) {
            g_string_sprintf(temp,"Audio channel %i is %s",i,Config.audio_channel_names[i]->str);
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,temp->str);
        }
        g_string_sprintf(temp,"Using audio channel %i (%s)",Config.audio_stream,Config.audio_channel_names[Config.audio_stream]->str);

        /* Log subpictures */
        for (i=0;i<Config.subpictures_in_dvd;i++) {
            g_string_sprintf(temp,"Subpicture %i is %s",i,Config.subpicture_names[i]->str);
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,temp->str);
        }
        if (Config.subpictures_in_dvd>0 && Config.subpicture_stream>-1) {
            g_string_sprintf(temp,"Using subpicture %s (%i)",Config.subpicture_names[Config.subpicture_stream]->str,Config.subpicture_stream);
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,temp->str);
        }

        /* -- Start converting -- */
        if (Config.encode && ((Config.cache==FALSE) || (Config.useDVDdevice==FALSE))) {
            /* -----------------------------------------------------------------
             * CONVERT DIRECTLY FROM SOURCE TO DIVX (source = dvdrom || cached files)
             * - if encoding is needed
             * - if direct conversion is requested
             * - if caching already has been done and we just need to encode
             */
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Started Direct Encoding");
            gdk_threads_enter();
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Encoding DVD sources to DIVX. Full movies take a long time, around 8 hours");
            gdk_threads_leave();
            g_string_sprintf(command,"dripencoder %s ",mpeg2divx_parameters());

            /* Input files */
            if (Config.full_movie_rip_mode == FALSE) {
                for (cnt = 0; cnt < selected_files; ++cnt) {
                    command = g_string_append(command,video_location->str);
                    command = g_string_append(command,files[cnt]->str);
                    command = g_string_append(command," ");
                }
            } else {
                /* Define ifo as input */
                g_string_sprintf(ifo,"%s",(gchar*)((glong)files[0]->str + 4));
                ifo = g_string_truncate(ifo,2);
                if (Config.useDVDdevice==FALSE) {
                    g_string_sprintf(temp,"ls -lsa %s* | grep -i VTS_%s_0.IFO | awk '{print($10)}'",Config.DVDsource->str,ifo->str);
                } else {
                    g_string_sprintf(temp,"ls -lsa %s* | grep -i VTS_%s_0.IFO | awk '{print($10)}'",video_location->str,ifo->str);
                }
                ifo = drip_system(temp->str,&result,FALSE);
                command = g_string_append(command,ifo->str);
                command = g_string_append(command," ");
            }
              
            command = g_string_append(command,Config.DIVXfile->str);
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Activating backend: %s",command->str);

            /* Tell encoder to start encoding */
            usleep(10000); //debugging delay...
            drip_encoder_start(encoder_client,command->str,Config.nice,&ev);
        } else {
            /* ----------------------------------------------------------------
             * CACHE DVD STREAMS TO HD 
             * and start encoder once done 
             */

            /* Display status & do the caching */
            gfloat perc = selected_files;perc = 1/perc;

            /* Display progress */
            cacheing = TRUE;
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Caching DVD streams");
            gdk_threads_enter();
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Caching DVD streams");
            gdk_threads_leave();

            /* Init dvd */
            tstdvd_device();

            /* Copy VIDEO_TS.IFO to cache location */
            g_string_sprintf(command,"mkdir -p %s",Config.CACHElocation->str);
            drip_system(command->str,&result,TRUE);
            if (strstr(video_location->str,"VIDEO_TS")!=NULL) {
                g_string_sprintf(command,"cp %sVIDEO_TS.IFO %s; chmod 600 %sVIDEO_TS.IFO",video_location->str,Config.CACHElocation->str,Config.CACHElocation->str);
            } else {
                g_string_sprintf(command,"cp %svideo_ts.ifo %s; chmod 600 %svideo_ts.ifo",video_location->str,Config.CACHElocation->str,Config.CACHElocation->str);
            }
            drip_system(command->str,&result,TRUE);

            /* Copy VTS_NN_0.IFO to cache location */
            g_string_sprintf(temp,"%s",(gchar*)((glong)files[0]->str + 4));
            temp = g_string_truncate(temp,2);
            if (strstr(video_location->str,"VIDEO_TS")!=NULL) {
                g_string_sprintf(command,"cp %sVTS_%s_0.IFO %s;chmod 600 %sVTS_%s_0.IFO",video_location->str,temp->str,Config.CACHElocation->str,Config.CACHElocation->str,temp->str);
            } else {
                g_string_sprintf(command,"cp %svts_%s_0.ifo %s;chmod 600 %svts_%s_0.ifo",video_location->str,temp->str,Config.CACHElocation->str,Config.CACHElocation->str,temp->str);
            }
            drip_system(command->str,&result,TRUE);

            /* Define cached ifo */
            g_string_sprintf(temp,"%s",(gchar*)((glong)files[0]->str + 4));
            temp = g_string_truncate(temp,2);
            g_string_sprintf(command,"ls -lsa %s* | grep -i VTS_%s_0.IFO | awk '{print($10)}'",Config.CACHElocation->str,temp->str);
            ifo = drip_system(command->str,&result,FALSE);

            /* Cache source streams */
            cache_vobs_for_ifo(); /* cache ifo related vobs */
            for (cnt = 0; cnt < selected_files; ++cnt) {
                gdk_threads_enter();
                g_string_sprintf(command,"Caching DVD stream %s",files[cnt]->str);
                gnome_appbar_set_status(GNOME_APPBAR(appbar1),command->str);
                gdk_threads_leave();
                rip_track(video_location->str,files[cnt]->str,perc,cnt,selected_files);
            }
            cacheing = FALSE;

            /* Eject DVDrom */
            if (Config.eject_dvdrom) {
                eject_dvdrom(Config.track_dir->str,Config.DVDdevice->str);
            }

            /* Encode mpeg2 cache */
            if (Config.encode) {
                g_string_sprintf(command,"dripencoder %s ",mpeg2divx_parameters());
                if (Config.full_movie_rip_mode == TRUE) {
                    command = g_string_append(command,ifo->str);
                    command = g_string_append(command," ");
                } else {
                    for (cnt = 0; cnt < selected_files; ++cnt) {
                        command = g_string_append(command,Config.CACHElocation->str);
                        command = g_string_append(command,files[cnt]->str);
                        command = g_string_append(command," ");
                    }
                }
                //g_print("Using %s for input\n",command->str);
                command = g_string_append(command,Config.DIVXfile->str);
                g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Activating backend: %s",command->str);


                /* ---------- Tell encoder to start encoding */
                usleep(10000); //debugging delay
                drip_encoder_start(encoder_client,command->str,Config.nice,&ev);


            } else {
                /* clean up temp files of HD cached encoding */
                unlink("disc-key");
                unlink("title-key");
                g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Ready.");
                converting = FALSE;
            }
        }
    } else {
        /* NO TRACKS?? This should be handled before drip ever reached this point */
        g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"NO TRACKS?? This should be handled before IP ever reached this point");
    }

    /* exit */
    gdk_threads_enter();
    gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Ready");
    gdk_threads_leave();
    g_string_free(command,TRUE);
    g_string_free(temp,TRUE);
    g_string_free(cached_files,TRUE);
    pthread_exit(0);
}

#ifdef LM_SENSORS
gpointer
heat_thread(gpointer data) {
    /* Thread that monitors lm_sensors if availeble and pauses encoding
       if temp1 > 50deg */
    gint result;
    glong temp_long;
    GString *temp;
    
    g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Hardware monitoring started");
    while (converting) {
        temp = drip_system("sensors | grep temp1 | awk '{printf($2)}'",&result,FALSE);
        temp_long = strtol(temp->str,NULL,0);
printf("temp = %li\n",temp_long);
        if (temp_long>49) {
            g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"CPU temp too high, sleeping...");
            drip_encoder_pause(encoder_client,&ev);
            while (temp_long>49) {
                sleep(60);
                temp = drip_system("sensors | grep temp1 | awk '{printf($2)}'",&result,FALSE);
                temp_long = strtol(temp->str,NULL,0);
printf("temp = %li\n",temp_long);
            }
            drip_encoder_resume(encoder_client,&ev);
        }
        sleep(60);
    }

    /* Exit */
    g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Hardware monitoring stopped");
    pthread_exit(0);
}
#endif /* LM_SENSORS */

void
on_button_main_rip_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    pthread_t ripthread;
    gint ripthread_id;

    if (converting == FALSE) {
        /* start conversion thread */
        Config.full_movie_rip_mode = TRUE; /* automatic dvd stream select */
        converting = TRUE;
        paused = FALSE;
        ripthread_id = pthread_create(&ripthread,NULL,conversion_thread,NULL);
#ifdef LM_SENSORS
        /* heat monitoring */
        if (heatthread_id!=0) pthread_cancel(heatthread);
        heatthread_id = pthread_create(&heatthread,NULL,heat_thread,NULL);
#endif /* LM_SENSORS */
    } else {
        GtkWidget* buzy;
        buzy = create_messagebox_buzy();
        gtk_widget_show(buzy);
    }
}


void
on_button_main_rescan_clicked          (GtkButton       *button,
                                        gpointer         user_data) {
    if (converting == FALSE) {
        /* start conversion thread */
        Config.full_movie_rip_mode = FALSE; /* manual dvd stream select */
        converting = TRUE;
        ripthread_id = pthread_create(&ripthread,NULL,conversion_thread,NULL);
#ifdef LM_SENSORS
        /* heat monitoring */
        if (heatthread_id!=0) pthread_cancel(heatthread);
        heatthread_id = pthread_create(&heatthread,NULL,heat_thread,NULL);
#endif /* LM_SENSORS */
    } else {
        GtkWidget* buzy;
        buzy = create_messagebox_buzy();
        gtk_widget_show(buzy);
    }
}

void
on_button_eject_clicked                (GtkButton       *button,
                                        gpointer         user_data) {
    GString *command = g_string_new("");
    GString *command2 = g_string_new("");

    if (Config.track_dir == NULL || Config.track_dir->len == 0) {
        g_string_sprintf(command,"%s",detect_source_track_dir());
    } else {
        g_string_sprintf(command,"%s",Config.track_dir->str);
    }

    if (command != NULL && command->len > 0) {
        eject_dvdrom(Config.track_dir->str,Config.DVDdevice->str);
    } else {
        g_string_sprintf(command,"Unable to eject, DVD device %s not valid? (%s)",Config.DVDdevice->str,command->str);
        drip_error(command->str);
    }

    /* Clean & Exit */
    g_string_free(command,TRUE);
    g_string_free(command2,TRUE);
    return;
}


void
on_button_main_pref_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_button_main_exit_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
    terminate(FALSE);
}


void
on_propertybox1_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
    save_config();
    propertybox1 = NULL;
}


gboolean
on_mainwindow_destroy_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    terminate(FALSE);
    return FALSE;
}

void
on_combo_entry_dvddevice_changed       (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *combo = (GtkWidget*)user_data;
    GString *command = g_string_new("");
    g_string_sprintf(Config.DVDdevice,"%s",gtk_entry_get_text(GTK_ENTRY(combo)));

    /* Clean & Exit */
    g_string_free(command,TRUE);
    return;
}

void
on_combo_entry_dvdsource_changed       (GtkEditable     *editable,
                                        gpointer         user_data)
{
    gchar *last_char;
    GtkWidget *combo = (GtkWidget*)user_data;
    g_string_sprintf(Config.DVDsource,"%s",g_strstrip(gtk_entry_get_text(GTK_ENTRY(combo))));

    /* force "/" at end */
    last_char = (gchar*)((glong)Config.DVDsource->str + Config.DVDsource->len - 1);
    if (strncmp(last_char,"/",1)) {
        Config.DVDsource = g_string_append(Config.DVDsource,"/");
    }

    return;
}


void
on_combo_entry2_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *combo = (GtkWidget*)user_data;
    g_string_sprintf(Config.DIVXfile,"%s",gtk_entry_get_text(GTK_ENTRY(combo)));
}

void
on_spinbutton_blur_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.blurradius = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
on_spinbutton_cpus_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.cpus = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}


void
on_spinbutton_video_bitrate_changed    (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.video_bitrate = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
on_spinbutton_video_stream_changed     (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.video_stream = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}


void
on_combo_audio_bitrate_changed         (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    if (strcmp(gtk_entry_get_text(GTK_ENTRY(entry)),"auto")==0) {
        Config.audio_bitrate = -1;
    } else {
        Config.audio_bitrate = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
    }
}


void
on_spinbutton_audio_stream_changed     (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.audio_stream = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}


void
on_radiobutton_divx_codecfast_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if (gtk_toggle_button_get_active(togglebutton)) {
        Config.codec = 0;
    }
}

void
on_radiobutton_divx_codeclow_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if (gtk_toggle_button_get_active(togglebutton)) {
        Config.codec = 1;
    }
}

void
on_radiobutton_divx_opendivx_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if (gtk_toggle_button_get_active(togglebutton)) {
        Config.codec = 2;
    }
}

void
on_spinbutton_clip_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
    Config.cliptop = atoi(gtk_editable_get_chars(editable,0,-1));
}

void
on_spinbutton_clipleft_changed         (GtkEditable     *editable,
                                        gpointer         user_data)
{
    Config.clipleft = atoi(gtk_editable_get_chars(editable,0,-1));
}

void
on_spinbutton_clipright_changed        (GtkEditable     *editable,
                                        gpointer         user_data)
{
    Config.clipright = atoi(gtk_editable_get_chars(editable,0,-1));
}

void
on_spinbutton_clip_offset_changed      (GtkEditable     *editable,
                                        gpointer         user_data)
{
    Config.clipbottom = atoi(gtk_editable_get_chars(editable,0,-1));
}


void
on_checkbutton_limit_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_checkbutton_eject_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.eject_dvdrom = gtk_toggle_button_get_active(togglebutton);
}

void
on_checkbutton_honor_aspect_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.honor_aspect = gtk_toggle_button_get_active(togglebutton);
}

void
on_checkbutton_hyperzoom_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.hyper_zoom = gtk_toggle_button_get_active(togglebutton);
}

void
on_checkbutton_autoclip_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.autoclip = gtk_toggle_button_get_active(togglebutton);
}


void
on_checkbutton_dvddb_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.use_dvddb = gtk_toggle_button_get_active(togglebutton);
}


void
on_entry2_changed                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.nice = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
on_spinbutton_width_changed            (GtkEditable     *editable,
                                        gpointer         user_data) {
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.width = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
on_spinbutton_heigth_changed           (GtkEditable     *editable,
                                        gpointer         user_data) {
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.height = atoi(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
on_dvdstreams_select_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
    GdkColor col_yes,col_no;
    gchar *text;
    gint result;

    /* Allocate colours */
    gdk_color_parse("tomato3",&col_no);
    gdk_color_parse("green3",&col_yes);

    /* Anything at all? */
    result = gtk_clist_get_text(clist,row,3,(gchar**)&text);
    if (result!=1) 
        return;

    if (strstr(text,"NO")!=NULL) {
        gtk_clist_set_text(clist,row,3,"   YES   ");
        gtk_clist_set_background(GTK_CLIST(clist),row,&col_yes);
    } 

    if (strstr(text,"YES")!=NULL) {
        gtk_clist_set_text(clist,row,3,"   NO   ");
        gtk_clist_set_background(GTK_CLIST(clist),row,&col_no);
    }

    return;
}

void
on_dvdstream_ok_clicked                (GtkButton       *button,
                                        gpointer         user_data) {
    gint row = 0;
    gint file = 0;
    gchar *text;

    do {
        gtk_clist_get_text(GTK_CLIST(user_data),row,3,(gchar**)&text);
        if (strstr(text,"YES") != NULL) {
            /* this file should be encoded */
            gtk_clist_get_text(GTK_CLIST(user_data),row,0,(gchar**)&text);
            files[file] = g_string_new(text);
            file++;
            //debug; g_print("%s should be encoded\n",files[row]->str);
        }
        row++;
    }
    while (gtk_clist_get_text(GTK_CLIST(user_data),row,3,(gchar**)&text) == 1);

    selected_files = file;
    gtk_widget_destroy(select_dialog);
    return;
}

void
on_dvdstream_cancel_clicked            (GtkButton       *button,
                                        gpointer         user_data) {
    selected_files = 0;
    gtk_widget_destroy(select_dialog);
    return;
}

void
on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    /* calculate bitrate */
    gint size = atoi(gtk_entry_get_text(GTK_ENTRY(spinbutton1)));
    gint duration = atoi(gtk_entry_get_text(GTK_ENTRY(spinbutton2)));
    gint audio = atoi(gtk_entry_get_text(GTK_ENTRY(spinbutton3)));
#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
    gfloat bitrate = ((size/(duration*60))-(audio*125))/(5*28);
#else
    gfloat bitrate = ((size/(duration*60))-(audio*125))/28;
#endif
    GString *format = g_string_new("");
    
    g_string_sprintf(format,"%i",(gint)bitrate);
    gtk_entry_set_text(GTK_ENTRY(entry3),format->str);

    g_string_free(format,TRUE);
    return;
}


void
on_button10_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    /* close calc window */
    gtk_widget_destroy(GTK_WIDGET(user_data));
}

void
on_checkbutton_HD_cache_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.cache = gtk_toggle_button_get_active(togglebutton);
}

void
on_checkbutton_encode_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.encode = gtk_toggle_button_get_active(togglebutton);
}

void
on_checkbutton_iso_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.create_iso = gtk_toggle_button_get_active(togglebutton);
}

void
on_spinbutton_maxsizeoutput_changed    (GtkEditable     *editable,
                                        gpointer         user_data) {
    GtkWidget *entry = (GtkWidget*)user_data;
    Config.maxsize = atol(gtk_entry_get_text(GTK_ENTRY(entry)));
}

void
on_stop_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
    if (converting) {
        stop_dialog = create_stop_dialog();
        gtk_widget_show(stop_dialog);
    }
    /* TODO : if (cacheing) { */

    return;
}

void
on_pause_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (converting) {
        if (paused==FALSE) {
            drip_encoder_pause(encoder_client,&ev);
            paused = TRUE;
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Paused encoding");
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"PAUSED, click Pause again to resume");

        } else {
            drip_encoder_resume(encoder_client,&ev);
            paused = FALSE;
            g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Resumed encoding");
            gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Resuming...");
        }
    }
    return;
}

void
on_radiobutton_useDVDdevice_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.useDVDdevice = (gtk_toggle_button_get_active(togglebutton));
    return;
}

void
on_radiobutton_useDVDsource_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.useDVDdevice = !(gtk_toggle_button_get_active(togglebutton));
    return;
}

void
on_button_ok_stop_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    if (stop_dialog) {
        drip_encoder_stop(encoder_client,&ev);
        converting = FALSE;
        pthread_cancel(ripthread);
        gtk_widget_destroy(stop_dialog);
        encoding_time = FALSE;
    }
}


void
on_button_no_stop_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    if (stop_dialog) {
        gtk_widget_destroy(stop_dialog);
    }
}


void
on_combo_entry_cache_location_changed  (GtkEditable     *editable,
                                        gpointer         user_data)
{
    gchar *last_char = NULL;
    GtkWidget *combo = (GtkWidget*)user_data;
    g_string_sprintf(Config.CACHElocation,"%s",gtk_entry_get_text(GTK_ENTRY(combo)));

    /* force "/" at end */
    last_char = (gchar*)((glong)Config.CACHElocation->str + Config.CACHElocation->len - 1);
    if (strncmp(last_char,"/",1)) {
        Config.CACHElocation = g_string_append(Config.CACHElocation,"/");
    }
    
    return;
}

void
on_checkbutton_delete_cache_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.cache_delete_on_exit = (gtk_toggle_button_get_active(togglebutton));

    return;
}

void
on_button11_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    /* OK on audio\subpicture dialog */
    gtk_widget_destroy(audio_subpicture_dialog);
    audio_subpicture_dialog = NULL;
    return;
}


void
on_button13_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    /* Cancel on audio\subpicture dialog */
    //drip_encoder_stop(encoder_client,&ev);
    converting = FALSE;
    pthread_cancel(ripthread);
    gtk_widget_destroy(audio_subpicture_dialog);
    gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Ready.");
    gdk_threads_leave();

    return;
}

void
on_radiobutton_audio_channel_0_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 0;

    return;
}


void
on_radiobutton_audio_channel_1_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 1;

    return;
}


void
on_radiobutton_audio_channel_2_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 2;

    return;
}


void
on_radiobutton_audio_channel_3_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 3;

    return;
}


void
on_radiobutton_audio_channel_4_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 4;

    return;
}


void
on_radiobutton_audio_channel_5_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 5;

    return;
}


void
on_radiobutton_audio_channel_6_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 6;

    return;
}

void
on_radiobutton_audio_channel_7_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    Config.audio_stream = 7;

    return;
}


void
on_combo_entry9_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
    gint i = 0;
    GtkWidget *combo = (GtkWidget*)user_data;
    while (i<32 && strcmp(gtk_entry_get_text(GTK_ENTRY(combo)),Config.subpicture_names[i]->str)) {
        i++;
    }
    if (i<32) { /* 33 = ( 32subpictures + 1none) -1 */
        /* Found subpicture stream */
        Config.subpicture_stream = i;
    } else {
        /* None or ?unknown? -> set to None (=-1) */
        Config.subpicture_stream = -1; 
    }
 
    //g_print("Subpicture stream = %i\n",Config.subpicture_stream);
    return;
}


