#include <gnome.h>
#include <dirent.h>
#include "../ifo/libifo/ifo.h"

/* structs */
typedef struct _drip_system_t {
    gchar *command;
    gint *exec_result;
    gboolean freebuffer;
} drip_system_t;

/* general proto's */
gboolean save_config(void);
void drip_error(gchar *text);
void drip_info(gchar *text);
GString* drip_system(gchar *command,gint *exec_result,gboolean freebuffer);
gpointer drip_system_threaded(gpointer data);
void terminate(gboolean hard);
gchar* mpeg2divx_parameters(void);
gint dirfilter_movie_vob(const struct dirent *entry);
gint dirfilter_all_vob(const struct dirent *entry);
gint dirfilter_all(const struct dirent *entry);
gint dirfilter_divx(const struct dirent *entry);
gint manual_stream_select(gchar *video_location);
gboolean detect_source_filesystem(void);
gchar* detect_source_track_dir(void);
gboolean check_for_supermount_fs(void);
gboolean user_can_mount_dvd(void);
gint detect_largest_vob_sequence(gchar *video_location);
gint dirfilter_avi(const struct dirent *entry);
void rename_divx_files(void);
gboolean eject_dvdrom(gchar *track_dir,gchar *dvddevice);
gpointer make_iso_images(gpointer data);
gchar* readlink_malloc (gchar *filename);
gpointer heat_thread(gpointer data);
gboolean cache_vobs_for_ifo(void);
void rip_track(gchar *dir,gchar *track,gfloat perc,guint nr,gint);
gboolean config_ok(void);
gpointer conversion_thread(gpointer data);
gboolean parse_clut(ifo_t *ifo);
gboolean parse_ifo(gchar *_ifo_file);
gboolean format_divxname(GString *DIVXfile);
gboolean dvdname_check(GString*);

/* interface related proto's */
void
on_ripdvd_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_rescan_for_dvd_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_save_preferences_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_stop_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_pause_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_button_main_rip_clicked             (GtkButton       *button,
                                        gpointer         user_data);

void
on_button_main_rescan_clicked          (GtkButton       *button,
                                        gpointer         user_data);

void
on_button_eject_clicked                (GtkButton       *button,
                                        gpointer         user_data);

void
on_checkbutton_honor_aspect_toggled    (GtkToggleButton *button,
                                        gpointer         user_data);

void
on_checkbutton_hyperzoom_toggled       (GtkToggleButton *button,
                                        gpointer         user_data);

void
on_checkbutton_autoclip_toggled        (GtkToggleButton *button,
                                        gpointer         user_data);

void
on_button_main_pref_clicked            (GtkButton       *button,
                                        gpointer         user_data);

void
on_button_main_exit_clicked            (GtkButton       *button,
                                        gpointer         user_data);

void
on_propertybox1_destroy                (GtkObject       *object,
                                        gpointer         user_data);

gboolean
on_mainwindow_destroy_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data);

void
on_combo_entry_dvddevice_changed       (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_combo_entry_dvdsource_changed       (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_combo_entry2_changed                (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_blur_changed             (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_cpus_changed             (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_video_bitrate_changed    (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_homepagebutton_clicked              (GtkButton       *button,
                                        gpointer         user_data);

void
on_spinbutton_video_stream_changed     (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_combo_audio_bitrate_changed         (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_audio_stream_changed     (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_radiobutton_divx_codecfast_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_divx_codeclow_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_divx_opendivx_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_spinbutton_clip_changed             (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_clipleft_changed         (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_clipright_changed        (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_clip_offset_changed      (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_entry_divx_blurradius_changed       (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_checkbutton_limit_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_checkbutton_eject_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_checkbutton_dvddb_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_entry2_changed                      (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_width_changed            (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_spinbutton_heigth_changed           (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_button_calculate_bitrate_clicked    (GtkButton       *button,
                                        gpointer         user_data);


void
on_dvdstreams_select_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data);
void
on_dvdstream_ok_clicked                (GtkButton       *button,
                                        gpointer         user_data);

void
on_dvdstream_cancel_clicked            (GtkButton       *button,
                                        gpointer         user_data);

void
on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data);

void
on_button10_clicked                    (GtkButton       *button,
                                        gpointer         user_data);

void
on_checkbutton_HD_cache_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_checkbutton_encode_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_spinbutton_maxsizeoutput_changed    (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_radiobutton_useDVDdevice_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_useDVDsource_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_button_ok_stop_clicked              (GtkButton       *button,
                                        gpointer         user_data);

void
on_button_no_stop_clicked              (GtkButton       *button,
                                        gpointer         user_data);

void
on_combo_entry_cache_location_changed  (GtkEditable     *editable,
                                        gpointer         user_data);

void
on_checkbutton_delete_cache_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_checkbutton_iso_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_button11_clicked                    (GtkButton       *button,
                                        gpointer         user_data);

void
on_button13_clicked                    (GtkButton       *button,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_0_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_1_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_2_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_3_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_4_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_5_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_6_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_radiobutton_audio_channel_7_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_combo_entry9_changed                (GtkEditable     *editable,
                                        gpointer         user_data);



extern GtkWidget *spinbutton1;
extern GtkWidget *spinbutton2;
extern GtkWidget *spinbutton3;
extern GtkWidget *entry3;
extern gint selected_files;
extern GString *files[1000];

