/*****
 *
 * css-cat.c
 * Copyright 1999 Derek Fawcus.
 *
 * decss.c
 * 2001 Jarl van Katwijk
 *
 * libdvdcss code
 * 2001 Marcel Janssen
 *
 *
 *
NOTE: This code was originally bundled with libcss-0.1.0.
 * This file is part of libcss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****/

/*
 * Decode selected sector types from a CSS encoded DVD to stdout.  Use as a
 * filter on the input to mpeg2player or ac3dec.
 */

#include <sys/ioctl.h>
#include <stdio.h>
#include <sys/wait.h>
#include <stdlib.h>
#if defined(__linux__)
# include <getopt.h>
#endif /* __linux__ */
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/cdrom.h>
#include "drip.h"
#include <gnome.h>
#include <videolan/dvdcss.h>
#include "callbacks.h"


/* stuct's */
struct {
	char *dev_name;
	int do_all;
	int do_video;
	int do_ac3;
	int do_mpg;
	int keep_pack;
	int keep_pes;
} param = {
	NULL,
    1,
    1,
    1,
    0,
    1,
    1
};

/*
 * dvdcss handle
 */
static dvdcss_handle dvdcss = NULL;
int cracked;

/* local proto's */
gboolean tstdvd_file(gchar*);
gboolean tstdvd_device(void);
gint path_to_lba(gchar*);
gint css_cat(gchar*,gchar*,gfloat,guint);

/* defines */
#define STCODE(p,a,b,c,d) ((p)[0] == a && (p)[1] == b && (p)[2] == c && (p)[3] == d)

/* globals */
u_long long offset = 0x0;
u_long long offset_end = 0x249a4;

gboolean tstdvd_close(void) {
   /* Close dvd handles */
   dvdcss_close(dvdcss);

   return TRUE;
}

gboolean tstdvd_device(void) {
   gboolean result = TRUE;

   /* Reset current disk */
   g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Opening DVD-drive using libdvdcss");
   if (dvdcss != NULL) {
       dvdcss_close(dvdcss);
   }
   dvdcss = dvdcss_open(Config.DVDdevice->str, DVDCSS_INIT_QUIET );
   cracked = 0;

   return result;
}

gboolean tstdvd_file(gchar *filename) {
   gint lba = -1;
   int titlerip, dvdcssseek;
   gboolean result = FALSE;

   lba = path_to_lba(filename);
   titlerip = (dvdcss_title( dvdcss, lba ) != -1)? 1: 0;

   //---> dvdcss_title() returns nothing of interest, we'll hope everyting goes ok all the time... 
   //if (!titlerip) {
   //    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"libdvdcss : dvdcss title succesfull");
   //} else {
   //    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"libdvdcss : dvdcss title unsuccesfull");
   //}

   dvdcssseek = (dvdcss_seek( dvdcss, lba ) != -1)? 1: 0;

   if (dvdcssseek) {
       g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"libdvdcss : dvdcss seek succesfull");
   } else {
       g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"libdvdcss : dvdcss seek unsuccesfull");
   }

   result = (lba !=-1)? TRUE:FALSE;

   return result;
}


/*
 * dvdcss end
 */

static void super_seek (int fd, u_long long offset) {
	int i;

	lseek (fd, offset%0x7FFFFFFF, SEEK_SET);
	for (i=0; i<offset/0x7FFFFFFF; i++)
		lseek (fd, 0x7FFFFFFF, SEEK_CUR);

    return;
}


/**
 *
 */

static void un_css (gchar *sourcefile,int fdi, int fdo, u_char *key_title,gfloat perc2,guint nr)
{
    /* 1- read input SHP's, 
     * 2- stream filtered by libdvdcss,
     * 3- write all mpeg2 blocks to HD
     */
    glong readbytes = 0;
    glong blocks = 0;
    glong blocks_countdown = 1000;
    gfloat perc;
    guchar sector_buf[2048];
    guchar *sp=NULL;
    struct stat file_stats;
    gint written;
    gint wr = 0;
    gint peslen = 0;

    stat(sourcefile,&file_stats);
    if (file_stats.st_size<2048) {
        g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Input stream %s too small, not caching",sourcefile);
        return;
    }
    super_seek (fdi, offset*2048);

	while ((dvdcss_read(dvdcss,sector_buf,1,DVDCSS_READ_DECRYPT)==1) && (readbytes<file_stats.st_size)) {
        readbytes += 2048;
		if (!STCODE (sector_buf,0x00,0x00,0x01,0xba)) {
			g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"Not Pack start code");
            /* exit(-1) */
		}

        /* Display progress every 1000 blocks */
        blocks++;blocks_countdown--;
        if (blocks_countdown == 0) {
            //perc = ((blocks*2048.0) / file_stats.st_size)/(gfloat)nr + perc2;
            perc = ((blocks*2048.0) / file_stats.st_size);
            gdk_threads_enter();
            gnome_appbar_set_progress(GNOME_APPBAR(appbar1),perc);
            gdk_threads_leave();
            blocks_countdown = 1000;
        }

        /* This is the modus Drip uses atm,
         * TODO: skip audio channels and subtitles not needed for encoding
         *       - less HD space
         *       - faster caching with less IO
         */

		written = 0;
        sp = sector_buf;
        peslen = 2048;
        do {
            wr = write (fdo, sp, peslen - written);
            sp += wr;
            written += wr;
        } while (wr > 0 && written < peslen);
    }
}

gint path_to_lba(gchar *path) {
    /* New path_to_lba code: 
     * uses a setuided external program to do the FIBMAP io call,
     * so Drip ran run as non root
     */
    FILE *fh;
    gint fd;
    gint result;
    gint lba = 0;
    GString *cmd = g_string_new("");
    GString *lbastr;

    g_string_sprintf(cmd, "fibmap %s", path);
    lbastr = drip_system(cmd->str,&result,FALSE);
    lba = atoi(lbastr->str);
    if (lba>0) return lba;

    /* Not succesfull, print warning */
    g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Attempt to execute fibmap failed, it should have been installed together with Drip. Drip now will try to read you DVD without decryption enabled, this only works for some DVD's");
    if ((fd = open(path, O_RDONLY)) < 0) {
        g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"DVD vob file could not be opened");
        return -1;
    }

    return lba;
}


/*
 *  Copy from css encrypted input to plain output
 */
gint css_cat(gchar *input, gchar *output,gfloat perc,guint nr) {
	guchar key_title[5];
	gint fd,fd2 = 0;
	gint ret=-1;


	if ((fd = open (input, O_RDONLY)) == -1) {   /* Open input */
		g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Can't open DVD");
	} else {
        fd2 = open(output,O_WRONLY|O_CREAT,S_IRWXU); /* Open output */
        /* Open decss filter channel */
    	un_css(input,fd, fd2, key_title,perc,nr);
    	ret = 0;
        close(fd2);
        close(fd);
    }

    /* Clean & Exit */
	return ret;
}
