
#include "../config.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <gnome.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "drip.h"

GtkWidget *appbar1;
GtkWidget *textscreen;
GtkWidget *gpixmap;
GtkWidget *hbox_main;

/* Backing pixmap for drawing area */
static GdkPixmap *pixmap = NULL;


static GnomeUIInfo file1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("_Copy DVD"),
    N_("Copy DVD"),
    (gpointer) on_ripdvd_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CONVERT,
    GDK_F1, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Compose"),
    N_("Select which DVD sources to use"),
    (gpointer) on_rescan_for_dvd_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CDROM,
    GDK_F2, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Stop encoding"),
    N_("Quit encoding"),
    (gpointer) on_stop_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_STOP,
    GDK_F2, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Pause encoding"),
    N_("Pause\resume encoding"),
    (gpointer) on_pause_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC,
    GDK_F2, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Save preferences"),
    NULL,
    (gpointer) on_save_preferences_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (on_exit1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_preferences1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
  GNOMEUIINFO_HELP ("drip"),
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_about1_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
  GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};

GtkWidget*
create_mainwindow (void) {
  GtkWidget *mainwindow;
  GtkWidget *dock1;
  GtkWidget *toolbar;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *button_main_rip;
  GtkWidget *button_eject;
  GtkWidget *button_main_rescan;
  GtkWidget *button_main_exit;
  GtkWidget *button_main_pref;
  GtkWidget *button_stop;
  GtkWidget *button_pause;
  GtkWidget *scrolledwindow3;
  GString *title = g_string_new("");

  g_string_sprintf(title,"Drip %s",VERSION);
  mainwindow = gnome_app_new ("Drip", title->str);
  g_string_free(title,TRUE);

  gtk_object_set_data (GTK_OBJECT (mainwindow), "mainwindow", mainwindow);

  dock1 = GNOME_APP (mainwindow)->dock;
  gtk_widget_ref (dock1);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "dock1", dock1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dock1);

  gnome_app_create_menus (GNOME_APP (mainwindow), menubar_uiinfo);

  gtk_widget_ref (menubar_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "file1",
                            menubar_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "ripdvd",
                            file1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "rescan_for_dvd",
                            file1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "separator1",
                            file1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "exit1",
                            file1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "settings1",
                            menubar_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (settings1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "preferences1",
                            settings1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "help1",
                            menubar_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (help1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "about1",
                            help1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_ref (toolbar);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "toolbar", toolbar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar);
  gnome_app_add_toolbar (GNOME_APP (mainwindow), GTK_TOOLBAR (toolbar), "toolbar",
                                GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
                                GNOME_DOCK_LEFT, 1, 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (toolbar), 1);
  gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 16);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_CONVERT);
  button_main_rip = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Copy DVD"),
                                _("Converts movie data from DVDrom currently loaded in DVD device"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_main_rip);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_main_rip", button_main_rip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_main_rip);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_CDROM);
  button_main_rescan = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Compose"),
                                _("Compose DVD streams for encoding"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_main_rescan);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_main_rescan", button_main_rescan,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_main_rescan);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_LAST);
  button_eject = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Eject"),
                                _("Eject the DVD from your DVDrom device"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_eject);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_eject", button_eject,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_eject);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_STOP);
  button_stop = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Stop"),
                                _("Stop current encoding process"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_stop);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_stop", button_stop,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_stop);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_EXEC);
  button_pause = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Pause"),
                                _("Pauses (and resumes) current encoding process"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_pause);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_pause", button_pause,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_pause);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_PREFERENCES);
  button_main_pref = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Preferences"),
                                _("Setup preferences"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_main_pref);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_main_pref", button_main_pref,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_main_pref);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (mainwindow, GNOME_STOCK_PIXMAP_EXIT);
  button_main_exit = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Quit"),
                                _("Exit Drip"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button_main_exit);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "button_main_exit", button_main_exit,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_main_exit);

  /* Main VBOX */
  hbox_main = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (hbox_main);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "hbox_main", hbox_main,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox_main);
  gnome_app_set_contents (GNOME_APP (mainwindow), hbox_main);
  gtk_container_set_border_width (GTK_CONTAINER (hbox_main), 0);

  /* Progress frame pixmap */
  gpixmap = gnome_pixmap_new_from_rgb_d_at_size(currentframebuffer,NULL,1,1,500,400); // TODO: ASCPECT
  //gtk_box_pack_start (GTK_BOX (hbox_main), gpixmap, FALSE, FALSE, 0);
  //gtk_widget_show(gpixmap);

  /* Logging display */
  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow3);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "scrolledwindow3", scrolledwindow3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow3);
  gtk_box_pack_end (GTK_BOX (hbox_main), scrolledwindow3, TRUE, TRUE, 1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  textscreen = gtk_text_new (NULL, NULL);
  gtk_widget_ref (textscreen);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "textscreen", textscreen,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_text_set_line_wrap(GTK_TEXT(textscreen),TRUE);
  gtk_widget_show (textscreen);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), textscreen);
  /* Logging display end */

  appbar1 = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_ref (appbar1);
  gtk_object_set_data_full (GTK_OBJECT (mainwindow), "appbar1", appbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (appbar1);
  gnome_app_set_statusbar (GNOME_APP (mainwindow), appbar1);

  gtk_signal_connect (GTK_OBJECT (mainwindow), "destroy_event",
                      GTK_SIGNAL_FUNC (on_mainwindow_destroy_event),
                      NULL);
  gnome_app_install_menu_hints (GNOME_APP (mainwindow), menubar_uiinfo);
  gtk_signal_connect (GTK_OBJECT (button_main_rip), "clicked",
                      GTK_SIGNAL_FUNC (on_button_main_rip_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_main_rescan), "clicked",
                      GTK_SIGNAL_FUNC (on_button_main_rescan_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_eject), "clicked",
                      GTK_SIGNAL_FUNC (on_button_eject_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_main_pref), "clicked",
                      GTK_SIGNAL_FUNC (on_preferences1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_main_exit), "clicked",
                      GTK_SIGNAL_FUNC (on_button_main_exit_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_stop), "clicked",
                      GTK_SIGNAL_FUNC (on_stop_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_pause), "clicked",
                      GTK_SIGNAL_FUNC (on_pause_activate),
                      NULL);

  return mainwindow;
}

GtkWidget* 
create_configdruid(void) {
  GtkWidget *window1;
  GtkWidget *drip_druid;
  GtkWidget *druidpagestart1;
  GdkColor druidpagestart1_bg_color = { 0, 3125, 1625, 37235 };
  GdkColor druidpagestart1_textbox_color = { 0, 59080, 59080, 59080 };
  GdkColor druidpagestart1_title_color = { 0, 65535, 65534, 65534 };
  GtkWidget *druidpagestandard1;
  GtkWidget *druid_vbox1;
  GtkWidget *hbox24;
  GtkWidget *label9;
  GtkWidget *vbox11;
  GtkWidget *label7;
  GtkWidget *fileentry4;
  GtkWidget *combo_entry10;
  GtkWidget *label8;
  GtkWidget *label10;
  GtkWidget *druidpagefinish1;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), _("Drip - Configure"));

  drip_druid = gnome_druid_new ();
  gtk_widget_ref (drip_druid);
  gtk_object_set_data_full (GTK_OBJECT (window1), "drip_druid", drip_druid,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (drip_druid);
  gtk_container_add (GTK_CONTAINER (window1), drip_druid);

  druidpagestart1 = gnome_druid_page_start_new ();
  gtk_widget_ref (druidpagestart1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "druidpagestart1", druidpagestart1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (druidpagestart1);
  gnome_druid_append_page (GNOME_DRUID (drip_druid), GNOME_DRUID_PAGE (druidpagestart1));
  gnome_druid_set_page (GNOME_DRUID (drip_druid), GNOME_DRUID_PAGE (druidpagestart1));
  gnome_druid_page_start_set_bg_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_bg_color);
  gnome_druid_page_start_set_textbox_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_textbox_color);
  gnome_druid_page_start_set_title_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_title_color);
  gnome_druid_page_start_set_title (GNOME_DRUID_PAGE_START (druidpagestart1), _("Drip Configuration"));
  gnome_druid_page_start_set_text (GNOME_DRUID_PAGE_START (druidpagestart1), _("A few pages will guide you trough the basic setup of Drip.\n\nOnce this is done, you'll need to configure the conversion\nparameters. You can find documentation on the Drip\nhomepage about them, I strongly suggest you read them first\nbefore backing up your DVD's because it will save you a lot\nof time."));
  gnome_druid_page_start_set_logo (GNOME_DRUID_PAGE_START (druidpagestart1),
                                   create_image ("drip/test_drip.jpg"));

  druidpagestandard1 = gnome_druid_page_standard_new_with_vals ("", NULL);
  gtk_widget_ref (druidpagestandard1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "druidpagestandard1", druidpagestandard1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show_all (druidpagestandard1);
  gnome_druid_append_page (GNOME_DRUID (drip_druid), GNOME_DRUID_PAGE (druidpagestandard1));

  druid_vbox1 = GNOME_DRUID_PAGE_STANDARD (druidpagestandard1)->vbox;
  gtk_widget_ref (druid_vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "druid_vbox1", druid_vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (druid_vbox1);

  hbox24 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox24);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbox24", hbox24,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox24);
  gtk_box_pack_start (GTK_BOX (druid_vbox1), hbox24, TRUE, TRUE, 0);

  label9 = gtk_label_new ("");
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox24), label9, FALSE, FALSE, 0);

  vbox11 = gtk_vbox_new (TRUE, 0);
  gtk_widget_ref (vbox11);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox11", vbox11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox11);
  gtk_box_pack_start (GTK_BOX (hbox24), vbox11, TRUE, TRUE, 0);

  label7 = gtk_label_new (_("\nSelect DVD player device :"));
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (vbox11), label7, FALSE, FALSE, 0);

  fileentry4 = gnome_file_entry_new (NULL, _("DVD player device"));
  gtk_widget_ref (fileentry4);
  gtk_object_set_data_full (GTK_OBJECT (window1), "fileentry4", fileentry4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry4);
  gtk_box_pack_start (GTK_BOX (vbox11), fileentry4, FALSE, FALSE, 0);

  combo_entry10 = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry4));
  gtk_widget_ref (combo_entry10);
  gtk_object_set_data_full (GTK_OBJECT (window1), "combo_entry10", combo_entry10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry10);

  label8 = gtk_label_new (_("\n\n\n\n"));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (vbox11), label8, FALSE, FALSE, 0);

  label10 = gtk_label_new ("");
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (hbox24), label10, FALSE, FALSE, 0);

  druidpagefinish1 = gnome_druid_page_finish_new ();

  gtk_widget_ref (druidpagefinish1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "druidpagefinish1", druidpagefinish1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (druidpagefinish1);
  gnome_druid_append_page (GNOME_DRUID (drip_druid), GNOME_DRUID_PAGE (druidpagefinish1));

  return window1;
}

GtkWidget*
create_propertybox1 (void)
{
  GtkWidget *propertybox1;
  GtkWidget *notebook1;
  GtkWidget *labelprop1;
  GtkWidget *vbox4;
  GtkWidget *fileentry_divx;
  GtkWidget *combo_entry2;
  GtkWidget *hbox3;
  GtkWidget *hbox3b;
  GtkWidget *hbox3c;
  GtkWidget *checkbutton_honor_aspect;
  GtkWidget *label_divx_width;
  GtkObject *spinbutton_width_adj;
  GtkWidget *spinbutton_width;
  GtkWidget *label_divx_height;
  GtkObject *spinbutton_height_adj;
  GtkWidget *spinbutton_height;
  GtkWidget *hbox4;
  GtkWidget *label_divx_video_bitrate;
  GtkObject *spinbutton_video_bitrate_adj;
  GtkWidget *spinbutton_video_bitrate;
  GtkWidget *button_calculate_bitrate;
  GtkWidget *hbox5;
  GtkWidget *label_divx_audio_bitrate;
  GtkWidget *combo_audio_bitrate;
  GList *combo_audio_bitrate_items = NULL;
  GtkWidget *combo_entry3;
  GtkWidget *checkbutton_dvddb;
  GtkWidget *hbox6;
  GtkWidget *label_divx_codec;
  GSList *codec_group = NULL;
  GtkWidget *radiobutton_divx_codecfast;
  GtkWidget *radiobutton_divx_codeclow;
  GtkWidget *radiobutton_divx_opendivx;
  GtkWidget *hbox7;
  GtkWidget *label_divx_linesclip;
  GtkObject *spinbutton_clip_adj;
  GtkWidget *spinbutton_clip;
  GtkWidget *hbox7r;
  GtkWidget *label_divx_linesclip_right;
  GtkObject *spinbutton_clip_adj_right;
  GtkWidget *spinbutton_clip_right;
  GtkWidget *hbox7l;
  GtkWidget *label_divx_linesclip_left;
  GtkObject *spinbutton_clip_adj_left;
  GtkWidget *spinbutton_clip_left;
  GtkWidget *hbox8;
  GtkWidget *label_divx_offsetclip;
  GtkObject *spinbutton_clip_offset_adj;
  GtkWidget *spinbutton_clip_offset;
  GtkWidget *hbox9;
  GtkWidget *label_divx_blurradius;
  GtkObject *spinbutton_blur_adj;
  GtkWidget *checkbutton_autoclip;
  GtkObject *spinbutton_nice_adj;
  GtkWidget *spinbutton_nice;
  GtkWidget *hbox17;
  GtkWidget *maxsizeoutput;
  GtkObject *spinbutton_maxsizeoutput_adj;
  GtkWidget *hseparator2;
  GtkWidget *spinbutton_maxsizeoutput;
  GtkWidget *spinbutton_blur;
  GtkWidget *labelprop2;
  GtkWidget *vbox3;
  GtkWidget *checkbutton_eject;
  GtkWidget *checkbutton_hyperzoom;
  GtkWidget *hbox2;
  GtkWidget *hbox15;
  GtkWidget *label_nice;
  GtkWidget *label5;
  GtkWidget *checkbutton_HD_cache;
  GtkWidget *checkbutton_encode;
  GtkWidget *vbox8;
  GtkWidget *hbox18;
  GtkWidget *radiobutton1;
  GtkWidget *combo2;
  GtkWidget *fileentry3dvddevice;
  GtkWidget *hbox19;
  GtkWidget *hbox20;
  GtkWidget *hbox21;
  GtkWidget *label_cpus;
  GtkObject *spinbutton_cpus_adj;
  GtkWidget *spinbutton_cpus;
  GtkWidget *radiobutton2;
  GtkWidget *cache_location_label;
  GtkWidget *fileentry3;
  GtkWidget *combo_entry_cache_location;
  GtkWidget *combo3;
  GtkWidget *fileentry3location;
  GtkWidget *checkbutton_delete_cache;
  GtkWidget *checkbutton_iso;
  GSList *source_group = NULL;
  GtkTooltips *tooltips;
  GString *format = g_string_new("");

  tooltips = gtk_tooltips_new ();

  propertybox1 = gnome_property_box_new ();
  gtk_object_set_data (GTK_OBJECT (propertybox1), "propertybox1", propertybox1);
  gtk_window_set_policy (GTK_WINDOW (propertybox1), FALSE, TRUE, TRUE);
  gtk_window_set_title (GTK_WINDOW (propertybox1), _("Drip - Preferences"));

  notebook1 = GNOME_PROPERTY_BOX (propertybox1)->notebook;
  gtk_object_set_data (GTK_OBJECT (propertybox1), "notebook1", notebook1);
  gtk_widget_show (notebook1);
  gtk_container_set_border_width (GTK_CONTAINER (notebook1), 9);

  vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox8);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "vbox8", vbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox8);
  gtk_container_set_border_width (GTK_CONTAINER (vbox8), 9);

  hbox18 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox18);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox18", hbox18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox18);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox18, FALSE, FALSE, 5);

  radiobutton1 = gtk_radio_button_new_with_label (source_group, _("Use DVDrom device  "));
  source_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_widget_ref (radiobutton1);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "radiobutton1", radiobutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (hbox18), radiobutton1, FALSE, FALSE, 0);
  gtk_widget_set_usize (radiobutton1, -2, 24);
  gtk_tooltips_set_tip (tooltips, radiobutton1, _("Use the DVDrom device as the source location"), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton1), Config.useDVDdevice);

  fileentry3dvddevice = gnome_file_entry_new (NULL, NULL);
  gtk_widget_ref (fileentry3dvddevice);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "fileentry3dvddevice", fileentry3dvddevice,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry3dvddevice);
  gtk_box_pack_start (GTK_BOX (hbox18), fileentry3dvddevice, TRUE, TRUE, 0);

  combo2 = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry3dvddevice));
  gtk_widget_ref (combo2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "combo_entry_cache_location", combo2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (combo2), Config.DVDdevice->str);
  gtk_tooltips_set_tip (tooltips,  combo2 , _("Use the DVDrom device as the source location"), NULL);
  gtk_widget_show (combo2);

  checkbutton_eject = gtk_check_button_new_with_label (_("Eject DVD"));
  gtk_widget_ref (checkbutton_eject);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_eject", checkbutton_eject,(GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_eject);
  gtk_box_pack_start (GTK_BOX (vbox8), checkbutton_eject, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_eject, _("Eject DVDrom when the disc is no longer needed"), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_eject), Config.eject_dvdrom);


  hseparator2 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hseparator2", hseparator2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox8), hseparator2, FALSE, FALSE, 0);
  gtk_widget_set_usize (hseparator2, -1, 30);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox19);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox19", hbox19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (vbox8), hbox19, FALSE, FALSE, 5);

  radiobutton2 = gtk_radio_button_new_with_label (source_group, _("Use source location  "));
  source_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_widget_ref (radiobutton2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "radiobutton2", radiobutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (hbox19), radiobutton2, FALSE, FALSE, 0);
  gtk_widget_set_usize (radiobutton2, -2, 24);
  gtk_tooltips_set_tip (tooltips, radiobutton2, _("Use a specific location for source streams"), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton2), !Config.useDVDdevice);
  
  fileentry3location = gnome_file_entry_new (NULL, NULL);
  gtk_widget_ref (fileentry3location);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "fileentry3location", fileentry3location,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry3location);
  gtk_box_pack_start (GTK_BOX (hbox19), fileentry3location, TRUE, TRUE, 0);

  combo3 = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry3location));
  gtk_widget_ref (combo3);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "combo3", combo3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (combo3), Config.DVDsource->str);
  gtk_tooltips_set_tip (tooltips,  combo3 , _("Use a specific location for source streams"), NULL);
  gtk_widget_show (combo3);



  labelprop1 = gtk_label_new (_("Source"));
  gtk_widget_ref (labelprop1);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "labelprop1", labelprop1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelprop1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), labelprop1);

  vbox4 = gtk_vbox_new (FALSE, 7);
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox4);
  gtk_container_set_border_width (GTK_CONTAINER (vbox4), 9);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox15);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox15", hbox15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox15);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox15, FALSE, FALSE, 0);

  label_divx_video_bitrate = gtk_label_new (_("DIVX Filename "));
  gtk_widget_ref (label_divx_video_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_video_bitrate", label_divx_video_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_video_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox15), label_divx_video_bitrate, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_video_bitrate), GTK_JUSTIFY_RIGHT);

  fileentry_divx = gnome_file_entry_new (NULL, _("Filename of output Divx"));
  gtk_widget_ref (fileentry_divx);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "fileentry_divx", fileentry_divx,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry_divx);
  gtk_box_pack_start (GTK_BOX (hbox15), fileentry_divx, FALSE, FALSE, 0);

  combo_entry2 = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry_divx));
  gtk_widget_ref (combo_entry2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "combo_entry2", combo_entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry2);
  gtk_tooltips_set_tip (tooltips, combo_entry2, _("Filename of encoded Divx"), NULL);
  gtk_entry_set_text (GTK_ENTRY (combo_entry2), Config.DIVXfile->str);

  checkbutton_dvddb = gtk_check_button_new_with_label (_("Use DVDdb"));
  gtk_widget_ref (checkbutton_dvddb);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_dvddb", checkbutton_dvddb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_dvddb);
  gtk_box_pack_start (GTK_BOX (hbox15), checkbutton_dvddb, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_dvddb, _("Use DVDdb for naming output Divx file. If the DVDdb doesn't know the dvd title, the user supplied name is ued"), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_dvddb), Config.use_dvddb);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox4, TRUE, TRUE, 0);

  label_divx_video_bitrate = gtk_label_new (_("Video bitrate  "));
  gtk_widget_ref (label_divx_video_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_video_bitrate", label_divx_video_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_video_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox4), label_divx_video_bitrate, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_video_bitrate), GTK_JUSTIFY_RIGHT);

  spinbutton_video_bitrate_adj = gtk_adjustment_new (Config.video_bitrate, 200, 20000, 1, 10, 10);
  spinbutton_video_bitrate = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_video_bitrate_adj), 50, 0);
  gtk_widget_ref (spinbutton_video_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_video_bitrate", spinbutton_video_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_video_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox4), spinbutton_video_bitrate, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbutton_video_bitrate, _("DIVX encoding bitrate"), NULL);

  button_calculate_bitrate = gtk_button_new_with_label (_("Calculate bitrate"));
  gtk_widget_ref (button_calculate_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "button_calculate_bitrate", button_calculate_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_calculate_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox4), button_calculate_bitrate, FALSE, FALSE, 0);
  GTK_WIDGET_UNSET_FLAGS (button_calculate_bitrate, GTK_CAN_FOCUS);
  gtk_tooltips_set_tip (tooltips, button_calculate_bitrate, _("Calculated divx encoding bitrate"), NULL);
  gtk_button_set_relief (GTK_BUTTON (button_calculate_bitrate), GTK_RELIEF_HALF);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox5, TRUE, TRUE, 0);

  label_divx_audio_bitrate = gtk_label_new (_("Audio bitrate  "));
  gtk_widget_ref (label_divx_audio_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_audio_bitrate", label_divx_audio_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_audio_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox5), label_divx_audio_bitrate, FALSE, FALSE, 0);

  combo_audio_bitrate = gtk_combo_new ();
  gtk_widget_ref (combo_audio_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "combo_audio_bitrate", combo_audio_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_audio_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox5), combo_audio_bitrate, TRUE, TRUE, 0);
  combo_audio_bitrate_items = g_list_append (combo_audio_bitrate_items, _("auto"));
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_audio_bitrate), combo_audio_bitrate_items);
  g_list_free (combo_audio_bitrate_items);

  combo_entry3 = GTK_COMBO (combo_audio_bitrate)->entry;
  gtk_widget_ref (combo_entry3);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "combo_entry3", combo_entry3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry3);
  if (Config.audio_bitrate>-1) {
      g_string_sprintf(format,"%i",Config.audio_bitrate);
  } else {
      g_string_sprintf(format,"auto");
  }
  gtk_entry_set_text (GTK_ENTRY (combo_entry3), format->str);
  gtk_tooltips_set_tip (tooltips, combo_entry3, _("Audio MP3 bitrate (kBit)"), NULL);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox6, TRUE, TRUE, 0);

  label_divx_codec = gtk_label_new (_("Encoding codec  "));
  gtk_widget_ref (label_divx_codec);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_codec", label_divx_codec,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_codec);
  gtk_box_pack_start (GTK_BOX (hbox6), label_divx_codec, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_codec), GTK_JUSTIFY_LEFT);

  radiobutton_divx_codecfast = gtk_radio_button_new_with_label (codec_group, _("Fast motion"));
  codec_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_divx_codecfast));
  gtk_widget_ref (radiobutton_divx_codecfast);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "radiobutton_divx_codecfast", radiobutton_divx_codecfast,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton_divx_codecfast);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobutton_divx_codecfast, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, radiobutton_divx_codecfast, _("Use MSMPEG4 fast-motion codec. Dont use this unless you know what you're doing. Fast motion looks bad on low motion scenes."), NULL);
  if (!Config.codec == 0) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_divx_codecfast),TRUE);
  }


  radiobutton_divx_codeclow = gtk_radio_button_new_with_label (codec_group, _("Low motion"));
  codec_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_divx_codeclow));
  gtk_widget_ref (radiobutton_divx_codeclow);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "radiobutton_divx_codeclow", radiobutton_divx_codeclow,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton_divx_codeclow);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobutton_divx_codeclow, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, radiobutton_divx_codeclow, _("Use MSMPEG4 low-motion codec (default)"), NULL);
  if (Config.codec == 1) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_divx_codeclow), TRUE);
  }

#if (AVIFILE_MAJOR_VERSION == 0) && (AVIFILE_MINOR_VERSION==6) 
  radiobutton_divx_opendivx = gtk_radio_button_new_with_label (codec_group, _("OpenDIVX"));
  codec_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_divx_opendivx));
  gtk_widget_ref (radiobutton_divx_opendivx);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "radiobutton_divx_opendivx", radiobutton_divx_opendivx, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton_divx_opendivx);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobutton_divx_opendivx, FALSE, FALSE,
0);
  gtk_tooltips_set_tip (tooltips, radiobutton_divx_opendivx, _("Use OpenDIVX codec. Avifile (0.6>) must have the divx4 plugin loaded"), NULL);
  if (Config.codec == 2) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_divx_opendivx), TRUE);
  }
#endif

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hseparator2", hseparator2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox4), hseparator2, TRUE, FALSE, 0);
  gtk_widget_set_usize (hseparator2, -1, 15);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox3, TRUE, TRUE, 0);

  label_divx_width = gtk_label_new (_("Width  "));
  gtk_widget_ref (label_divx_width);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_width", label_divx_width,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_width);
  gtk_box_pack_start (GTK_BOX (hbox3), label_divx_width, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_width), GTK_JUSTIFY_RIGHT);

  
  spinbutton_width_adj = gtk_adjustment_new (Config.width, -1, 10000, 1, 10, 10);
  spinbutton_width = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_width_adj), 1, 0);
  gtk_widget_ref (spinbutton_width);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_width", spinbutton_width,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_width);
  gtk_box_pack_start (GTK_BOX (hbox3), spinbutton_width, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbutton_width, _("output width (default input width = -1)"), NULL);

  label_divx_height = gtk_label_new (_("  Height  "));
  gtk_widget_ref (label_divx_height);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_height", label_divx_height,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_height);
  gtk_box_pack_start (GTK_BOX (hbox3), label_divx_height, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_height), GTK_JUSTIFY_RIGHT);

  spinbutton_height_adj = gtk_adjustment_new (Config.height, -1, 10000, 1, 10, 10);
  spinbutton_height = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_height_adj), 1, 0);
  gtk_widget_ref (spinbutton_height);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_height", spinbutton_height,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_height);
  gtk_box_pack_start (GTK_BOX (hbox3), spinbutton_height, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbutton_height, _("output height (default input height or relative to width= -1)"), NULL);

  hbox3c = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3c);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox3c", hbox3c,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3c);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox3c, TRUE, TRUE, 0);

  checkbutton_honor_aspect = gtk_check_button_new_with_label (_("Use DVD aspect"));
  gtk_widget_ref (checkbutton_honor_aspect);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_honor_aspect",
checkbutton_honor_aspect,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_honor_aspect);
  gtk_box_pack_start (GTK_BOX (hbox3c), checkbutton_honor_aspect, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_honor_aspect, _("Correct output Height based on Width and DVD aspect (4:3 or 16:9) "), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_honor_aspect), Config.honor_aspect);

  hbox3b = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3b);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox3b", hbox3b,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3b);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox3b, TRUE, TRUE, 0);

  checkbutton_hyperzoom = gtk_check_button_new_with_label (_("Hyperbolic Zoom"));
  gtk_widget_ref (checkbutton_hyperzoom);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_hyperzoom",
checkbutton_hyperzoom,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_hyperzoom);
  gtk_box_pack_start (GTK_BOX (hbox3b), checkbutton_hyperzoom, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_hyperzoom, _("Use better zooming, very slow"), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_hyperzoom), Config.hyper_zoom);


  hseparator2 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hseparator2", hseparator2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox4), hseparator2, TRUE, FALSE, 0);
  gtk_widget_set_usize (hseparator2, -1, 15);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox9, TRUE, TRUE, 0);

  label_divx_blurradius = gtk_label_new (_("Radius blur filter (pixels)  "));
  gtk_widget_ref (label_divx_blurradius);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_blurradius", label_divx_blurradius,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_blurradius);
  gtk_box_pack_start (GTK_BOX (hbox9), label_divx_blurradius, FALSE, FALSE, 0);  gtk_label_set_justify (GTK_LABEL (label_divx_blurradius), GTK_JUSTIFY_LEFT);

  spinbutton_blur_adj = gtk_adjustment_new (Config.blurradius, 0, 100, 1, 10, 10);
  spinbutton_blur = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_blur_adj),
1, 0);
  gtk_widget_ref (spinbutton_blur);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_blur", spinbutton_blur,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_blur);
  gtk_box_pack_end (GTK_BOX (hbox9), spinbutton_blur, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_blur, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_blur, _("Radius for blur filter (blur off = 0)"), NULL);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox7, TRUE, TRUE, 0);

  label_divx_linesclip = gtk_label_new (_("Clip from top (lines)  "));
  gtk_widget_ref (label_divx_linesclip);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_linesclip", label_divx_linesclip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_linesclip);
  gtk_box_pack_start (GTK_BOX (hbox7), label_divx_linesclip, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_linesclip), GTK_JUSTIFY_LEFT);

  spinbutton_clip_adj = gtk_adjustment_new (Config.cliptop, 0, 10000, 1, 10, 10);
  spinbutton_clip = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_clip_adj), 1, 0);
  gtk_widget_ref (spinbutton_clip);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_clip", spinbutton_clip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_clip);
  gtk_box_pack_end (GTK_BOX (hbox7), spinbutton_clip, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_clip, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_clip, _("Number of lines to clip from top (default is 0, no clipping)"), NULL);

  hbox7r = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7r);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox7r", hbox7r,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7r);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox7r, TRUE, TRUE, 0);

  label_divx_linesclip_right = gtk_label_new (_("Clip from right (lines)  "));
  gtk_widget_ref (label_divx_linesclip_right);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_linesclip_right", label_divx_linesclip_right,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_linesclip_right);
  gtk_box_pack_start (GTK_BOX (hbox7r), label_divx_linesclip_right, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_linesclip_right), GTK_JUSTIFY_LEFT);

  spinbutton_clip_adj_right = gtk_adjustment_new (Config.clipright, 0, 10000, 1, 10, 10);
  spinbutton_clip_right = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_clip_adj_right), 1, 0);
  gtk_widget_ref (spinbutton_clip_right);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_clip_right", spinbutton_clip_right,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_clip_right);
  gtk_box_pack_end (GTK_BOX (hbox7r), spinbutton_clip_right, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_clip_right, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_clip_right, _("Number of lines to clip from right (default is 0, no clipping)"), NULL);

  hbox7l = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7l);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox7l", hbox7l,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7l);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox7l, TRUE, TRUE, 0);

  label_divx_linesclip_left = gtk_label_new (_("Clip from left (lines)  "));
  gtk_widget_ref (label_divx_linesclip_left);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_linesclip_left", label_divx_linesclip_left,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_linesclip_left);
  gtk_box_pack_start (GTK_BOX (hbox7l), label_divx_linesclip_left, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_linesclip_left), GTK_JUSTIFY_LEFT);

  spinbutton_clip_adj_left = gtk_adjustment_new (Config.clipleft, 0, 10000, 1, 10, 10);
  spinbutton_clip_left = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_clip_adj_left), 1, 0);
  gtk_widget_ref (spinbutton_clip_left);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_clip_left", spinbutton_clip_left,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_clip_left);
  gtk_box_pack_end (GTK_BOX (hbox7l), spinbutton_clip_left, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_clip_left, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_clip_left, _("Number of lines to clip from left (default is 0, no clipping)"), NULL);


  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox8);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox8", hbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox8, TRUE, TRUE, 0);

  label_divx_offsetclip = gtk_label_new (_("Clip from bottom (lines)  "));
  gtk_widget_ref (label_divx_offsetclip);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_divx_offsetclip", label_divx_offsetclip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_offsetclip);
  gtk_box_pack_start (GTK_BOX (hbox8), label_divx_offsetclip, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_offsetclip), GTK_JUSTIFY_LEFT);

  spinbutton_clip_offset_adj = gtk_adjustment_new (Config.clipbottom, 0, 10000, 1, 10, 10);
  spinbutton_clip_offset = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_clip_offset_adj), 1, 0);
  gtk_widget_ref (spinbutton_clip_offset);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_clip_offset", spinbutton_clip_offset,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_clip_offset);
  gtk_box_pack_end (GTK_BOX (hbox8), spinbutton_clip_offset, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_clip_offset, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_clip_offset, _("Number of lines to clip from bottom (default is 0, no clipping)"), NULL);

  checkbutton_autoclip = gtk_check_button_new_with_label (_("Automatic clipping"));
  gtk_widget_ref (checkbutton_autoclip);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_autoclip",
checkbutton_autoclip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_autoclip);
  gtk_box_pack_start (GTK_BOX (vbox4), checkbutton_autoclip, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_autoclip, _("Automatic cut off black borders"), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_autoclip), Config.autoclip);

  labelprop2 = gtk_label_new (_("DIVX"));
  gtk_widget_ref (labelprop2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "labelprop2", labelprop2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelprop2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), labelprop2);

  vbox3 = gtk_vbox_new (FALSE, 8);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox3);
  gtk_container_set_border_width (GTK_CONTAINER (vbox3), 9);

  hbox17 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox17);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox17", hbox17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox17);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox17, FALSE, FALSE, 0);

  maxsizeoutput = gtk_label_new (_(" Maximum output chunk size (Mb)  "));
  gtk_widget_ref (maxsizeoutput);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "maxsizeoutput", maxsizeoutput,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (maxsizeoutput);
  gtk_box_pack_start (GTK_BOX (hbox17), maxsizeoutput, FALSE, FALSE, 0);

  spinbutton_maxsizeoutput_adj = gtk_adjustment_new (Config.maxsize, 0, 2048, 1, 10, 10);
  spinbutton_maxsizeoutput = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_maxsizeoutput_adj), 1, 0);
  gtk_widget_ref (spinbutton_maxsizeoutput);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_maxsizeoutput", spinbutton_maxsizeoutput,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_maxsizeoutput);
  gtk_box_pack_end (GTK_BOX (hbox17), spinbutton_maxsizeoutput, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_maxsizeoutput, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_maxsizeoutput, _("The maximum size of the output divx. If output grows bigger, another divx file is created. You can make CD sized chunks. Put this value to maximum will generally result into just one file"), NULL);

  hbox20 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox20);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox20", hbox20,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox20);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox20, FALSE, FALSE, 0);

  label_cpus = gtk_label_new (_(" MPEG2 decoding CPUS  "));
  gtk_widget_ref (label_cpus);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_cpus", label_cpus,                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_cpus);
  gtk_box_pack_start (GTK_BOX (hbox20), label_cpus, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_cpus), GTK_JUSTIFY_LEFT);

  spinbutton_cpus_adj = gtk_adjustment_new (Config.cpus, 1, 32, 1, 10, 10);
  spinbutton_cpus = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_cpus_adj), 1, 0);
  gtk_widget_ref (spinbutton_cpus);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_cpus", spinbutton_cpus,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_cpus);
  gtk_box_pack_end (GTK_BOX (hbox20), spinbutton_cpus, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_cpus, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_cpus, _("Number of cpu's to use for decoding. Note that the encoding takes up about 85% of the cpu load and is SINGLE cpu only because avifile does not supports SMP. So dont exspect too much speed gain out of your SMP machine for now ;("), NULL);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox2, FALSE, FALSE, 0);

  label_nice = gtk_label_new (_(" Nice value backend  "));
  gtk_widget_ref (label_nice);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label_nice", label_nice,                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_nice);
  gtk_box_pack_start (GTK_BOX (hbox2), label_nice, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_nice), GTK_JUSTIFY_LEFT);

  spinbutton_nice_adj = gtk_adjustment_new (Config.nice,-20,19, 1, 10, 10);
  spinbutton_nice = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_nice_adj), 1,
0);
  gtk_widget_ref (spinbutton_nice);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "spinbutton_nice", spinbutton_nice,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton_nice);
  gtk_box_pack_end (GTK_BOX (hbox2), spinbutton_nice, FALSE, FALSE, 0);
  gtk_widget_set_usize (spinbutton_nice, 200, -2);
  gtk_tooltips_set_tip (tooltips, spinbutton_nice, _("Divx encoding nice value, range goes from -20 (highest priority) to 19 (lowest)"), NULL);

  checkbutton_encode = gtk_check_button_new_with_label (_("Encode DVD mpeg2 streams to DIVX"));
  gtk_widget_ref (checkbutton_encode);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_encode", checkbutton_encode,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_encode);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_encode, FALSE, FALSE, 0);  gtk_tooltips_set_tip (tooltips, checkbutton_encode, _("Encode cached mpeg2 streams to divx. Disabling encoding will make Drip do just the DVD reading to HD and quit after that, usefull for when you dont know how to fill up that huge HD of yours or when you've got a DVD writer or RAM"),NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON ( checkbutton_encode), Config.encode);

  checkbutton_iso = gtk_check_button_new_with_label (_("Make ISO images out of the generated DIVX files"));
  gtk_widget_ref (checkbutton_iso);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_iso", checkbutton_iso,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_iso);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_iso, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_iso, _("Make ISO images out of the generated divx files. You needs to have mkisofs installed. (mkisofs -J -l {divx} > {divx}.iso)"),NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON ( checkbutton_iso), Config.create_iso);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator2);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hseparator2", hseparator2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox3), hseparator2, FALSE, FALSE, 0);
  gtk_widget_set_usize (hseparator2, -1, 15);

  hbox21 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox21);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "hbox21", hbox21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox21);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox21, FALSE, FALSE, 0);

  cache_location_label = gtk_label_new (_(" Cache location  "));
  gtk_widget_ref (cache_location_label);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "cache_location_label", cache_location_label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cache_location_label);
  gtk_box_pack_start (GTK_BOX (hbox21), cache_location_label, FALSE, FALSE, 0);

  fileentry3 = gnome_file_entry_new (NULL, NULL);
  gtk_widget_ref (fileentry3);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "fileentry3", fileentry3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry3);
  gtk_box_pack_start (GTK_BOX (hbox21), fileentry3, TRUE, TRUE, 0);

  combo_entry_cache_location = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry3));
  gtk_widget_ref (combo_entry_cache_location);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "combo_entry_cache_location", combo_entry_cache_location,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (combo_entry_cache_location), Config.CACHElocation->str);
  gtk_tooltips_set_tip (tooltips,  combo_entry_cache_location , _("Location when Drip caches it's data. Given directory is created if possible automatically."), NULL);
  gtk_widget_show (combo_entry_cache_location);

  checkbutton_HD_cache = gtk_check_button_new_with_label (_("Cache mpeg2 strean to HD before encoding (ENABLE IF POSSIBLE!)"));
  gtk_widget_ref (checkbutton_HD_cache);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_HD_cache", checkbutton_HD_cache,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_HD_cache);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_HD_cache, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbutton_HD_cache, _("USE THIS WHEN POSSIBLE! Cached DVD reading will consume a few gigabytes of HD space, but it will be less hard on your DVD rom device. Non-cached reading might damage your DVDrom device over time because of the high number of spinups it will endure."), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON ( checkbutton_HD_cache), Config.cache);

  checkbutton_delete_cache = gtk_check_button_new_with_label (_("Delete cache on exit"));
  gtk_widget_ref (checkbutton_delete_cache);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "checkbutton_delete_cache", checkbutton_delete_cache,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton_delete_cache);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton_delete_cache, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips,  checkbutton_delete_cache, _("Delete all cached video data when Drip exits."), NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_delete_cache), Config.cache_delete_on_exit);

  label5 = gtk_label_new (_("Encoding"));
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (propertybox1), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label5);


  /* signal handlers */
  gtk_signal_connect (GTK_OBJECT (propertybox1), "destroy",
                      GTK_SIGNAL_FUNC (on_propertybox1_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (combo2), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry_dvddevice_changed),
                      combo2);
  gtk_signal_connect (GTK_OBJECT (combo3), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry_dvdsource_changed),
                      combo3);
  gtk_signal_connect (GTK_OBJECT (combo_entry2), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry2_changed),
                      combo_entry2);
  gtk_signal_connect (GTK_OBJECT (spinbutton_video_bitrate), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_video_bitrate_changed),
                      spinbutton_video_bitrate);
  gtk_signal_connect (GTK_OBJECT (spinbutton_blur), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_blur_changed),
                      spinbutton_blur);
  gtk_signal_connect (GTK_OBJECT (spinbutton_cpus), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_cpus_changed),
                      spinbutton_cpus);
  gtk_signal_connect (GTK_OBJECT (spinbutton_width), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_width_changed),
                      spinbutton_width);
  gtk_signal_connect (GTK_OBJECT (spinbutton_height), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_heigth_changed),
                      spinbutton_height);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo_audio_bitrate)->entry), "changed",
                      GTK_SIGNAL_FUNC (on_combo_audio_bitrate_changed),
                      GTK_COMBO(combo_audio_bitrate)->entry);
  gtk_signal_connect (GTK_OBJECT (radiobutton_divx_codecfast), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton_divx_codecfast_toggled),
                      radiobutton_divx_codecfast);
  gtk_signal_connect (GTK_OBJECT (radiobutton_divx_codeclow), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton_divx_codeclow_toggled),
                      radiobutton_divx_codeclow);
  gtk_signal_connect (GTK_OBJECT (radiobutton_divx_opendivx), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton_divx_opendivx_toggled),
                      radiobutton_divx_opendivx);
  gtk_signal_connect (GTK_OBJECT (spinbutton_clip), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_clip_changed),
                      spinbutton_clip);
  gtk_signal_connect (GTK_OBJECT (spinbutton_clip_left), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_clipleft_changed),
                      spinbutton_clip);
  gtk_signal_connect (GTK_OBJECT (spinbutton_clip_right), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_clipright_changed),
                      spinbutton_clip);
  gtk_signal_connect (GTK_OBJECT (spinbutton_maxsizeoutput), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_maxsizeoutput_changed),
                      spinbutton_maxsizeoutput);
  gtk_signal_connect (GTK_OBJECT (spinbutton_clip_offset), "changed",
                      GTK_SIGNAL_FUNC (on_spinbutton_clip_offset_changed),
                      spinbutton_clip_offset);
  gtk_signal_connect (GTK_OBJECT (checkbutton_eject), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_eject_toggled),
                      checkbutton_eject);
  gtk_signal_connect (GTK_OBJECT (checkbutton_honor_aspect), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_honor_aspect_toggled),
                      checkbutton_honor_aspect);
  gtk_signal_connect (GTK_OBJECT (checkbutton_hyperzoom), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_hyperzoom_toggled),
                      checkbutton_hyperzoom);
  gtk_signal_connect (GTK_OBJECT (checkbutton_autoclip), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_autoclip_toggled),
                      checkbutton_autoclip);
  gtk_signal_connect (GTK_OBJECT (checkbutton_dvddb), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_dvddb_toggled),
                      checkbutton_dvddb);
  gtk_signal_connect (GTK_OBJECT (spinbutton_nice), "changed",
                      GTK_SIGNAL_FUNC (on_entry2_changed),
                      spinbutton_nice);
  gtk_signal_connect (GTK_OBJECT (button_calculate_bitrate), "clicked",
                      GTK_SIGNAL_FUNC (on_button_calculate_bitrate_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (checkbutton_HD_cache), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_HD_cache_toggled),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (checkbutton_encode), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_encode_toggled),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (radiobutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton_useDVDdevice_toggled),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (radiobutton2), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton_useDVDsource_toggled),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (combo_entry_cache_location), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry_cache_location_changed),
                      combo_entry_cache_location);
  gtk_signal_connect (GTK_OBJECT (checkbutton_delete_cache), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_delete_cache_toggled),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (checkbutton_iso), "toggled",
                      GTK_SIGNAL_FUNC (on_checkbutton_iso_toggled),
                      NULL);


  gtk_object_set_data (GTK_OBJECT (propertybox1), "tooltips", tooltips);

  g_string_free(format,TRUE);
  return propertybox1;
}


GtkWidget*
create_rip_dialog (void)
{
  GtkWidget *rip_dialog;
  GtkWidget *dialog_vbox5;
  GtkWidget *vbox7;
  GtkWidget *hbox15;
  GtkWidget *label_divx_video_bitrate;
  GtkWidget *fileentry_divx;
  GtkWidget *combo_entry2;
  GtkWidget *drawingarea2;
  GtkWidget *hbox21;
  GtkWidget *label_ripdvd_select_audio_channel;
  GtkWidget *vbox9;
  GSList *audio_channels_group = NULL;
  GtkWidget *radiobutton_audio_channel_0 = NULL;
  GtkWidget *radiobutton_audio_channel_1 = NULL;
  GtkWidget *radiobutton_audio_channel_2 = NULL;
  GtkWidget *radiobutton_audio_channel_3 = NULL;
  GtkWidget *radiobutton_audio_channel_4 = NULL;
  GtkWidget *radiobutton_audio_channel_5 = NULL;
  GtkWidget *radiobutton_audio_channel_6 = NULL;
  GtkWidget *radiobutton_audio_channel_7 = NULL;
  //GtkWidget *hseparator3;
  GtkWidget *hbox22;
  GtkWidget *label6;
  GtkWidget *combo_subpicture;
  GList *combo_subpicture_items = NULL;
  GtkWidget *combo_entry9;
  GtkWidget *dialog_action_area5;
  GtkWidget *button11;
  GtkWidget *button13;
  gint i = 0;

  /* Reset subpicture */
  Config.subpicture_stream = -1;

  rip_dialog = gnome_dialog_new (_("Audio & Language"), NULL);
  gtk_object_set_data (GTK_OBJECT (rip_dialog), "rip_dialog", rip_dialog);
  gtk_window_set_policy (GTK_WINDOW (rip_dialog), FALSE, TRUE, TRUE);

  dialog_vbox5 = GNOME_DIALOG (rip_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (rip_dialog), "dialog_vbox5", dialog_vbox5);
  gtk_widget_show (dialog_vbox5);

  vbox7 = gtk_vbox_new (TRUE, 0);
  gtk_widget_ref (vbox7);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "vbox7", vbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox7);
  gtk_box_pack_start (GTK_BOX (dialog_vbox5), vbox7, TRUE, TRUE, 0);

/*
  drawingarea2 = gtk_drawing_area_new ();
  gtk_widget_ref (drawingarea2);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "drawingarea2", drawingarea2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (drawingarea2);
  gtk_box_pack_start (GTK_BOX (vbox7), drawingarea2, TRUE, TRUE, 0);
*/

/* */
  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox15);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "hbox15", hbox15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox15);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox15, FALSE, FALSE, 0);

  label_divx_video_bitrate = gtk_label_new (_("DIVX Filename "));
  gtk_widget_ref (label_divx_video_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "label_divx_video_bitrate", label_divx_video_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_divx_video_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox15), label_divx_video_bitrate, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label_divx_video_bitrate), GTK_JUSTIFY_RIGHT);

  fileentry_divx = gnome_file_entry_new (NULL, _("Filename of output Divx"));
  gtk_widget_ref (fileentry_divx);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "fileentry_divx", fileentry_divx,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry_divx);
  gtk_box_pack_start (GTK_BOX (hbox15), fileentry_divx, TRUE, TRUE, 0);

  combo_entry2 = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry_divx));
  gtk_widget_ref (combo_entry2);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "combo_entry2", combo_entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry2);
  gtk_entry_set_text (GTK_ENTRY (combo_entry2), Config.DIVXfile->str);

/* */
  hbox21 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox21);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "hbox21", hbox21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox21);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox21, TRUE, TRUE, 0);

  label_ripdvd_select_audio_channel = gtk_label_new (_(" Select audio channel  "));
  gtk_widget_ref (label_ripdvd_select_audio_channel);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "label_ripdvd_select_audio_channel", label_ripdvd_select_audio_channel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_ripdvd_select_audio_channel);
  gtk_box_pack_start (GTK_BOX (hbox21), label_ripdvd_select_audio_channel, FALSE, FALSE, 0);
  //gtk_misc_set_alignment (GTK_MISC (label_ripdvd_select_audio_channel), 0.0, 0.05);

  vbox9 = gtk_vbox_new (TRUE, 0);
  gtk_widget_ref (vbox9);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "vbox9", vbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox9);
  gtk_box_pack_start (GTK_BOX (hbox21), vbox9, TRUE, TRUE, 0);

  /* Audio 0 */
  if (Config.audio_channels_in_dvd>0) {
      radiobutton_audio_channel_0 = gtk_radio_button_new_with_label (audio_channels_group, Config.audio_channel_names[0]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_0));
      gtk_widget_ref (radiobutton_audio_channel_0);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_0", radiobutton_audio_channel_0,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_0);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_0, FALSE, FALSE, 0);
  }

  /* Audio 1 */
  if (Config.audio_channels_in_dvd>1) {
      radiobutton_audio_channel_1 = gtk_radio_button_new_with_label (audio_channels_group,  Config.audio_channel_names[1]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_1));
      gtk_widget_ref (radiobutton_audio_channel_1);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_1", radiobutton_audio_channel_1,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_1);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_1, FALSE, FALSE, 0);
  }

  /* Audio 2 */
  if (Config.audio_channels_in_dvd>2) {
      radiobutton_audio_channel_2 = gtk_radio_button_new_with_label (audio_channels_group,  Config.audio_channel_names[2]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_2));
      gtk_widget_ref (radiobutton_audio_channel_2);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_2", radiobutton_audio_channel_2,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_2);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_2, FALSE, FALSE, 0);
  }

  /* Audio 3 */
  if (Config.audio_channels_in_dvd>3) {
      radiobutton_audio_channel_3 = gtk_radio_button_new_with_label (audio_channels_group, Config.audio_channel_names[3]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_3));
      gtk_widget_ref (radiobutton_audio_channel_3);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_3", radiobutton_audio_channel_3,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_3);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_3, FALSE, FALSE, 0);
  }

  /* Audio 4 */
  if (Config.audio_channels_in_dvd>4) {
      radiobutton_audio_channel_4 = gtk_radio_button_new_with_label (audio_channels_group, Config.audio_channel_names[4]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_4));
      gtk_widget_ref (radiobutton_audio_channel_4);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_4", radiobutton_audio_channel_4,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_4);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_4, FALSE, FALSE, 0);
  }

  /* Audio 5 */
  if (Config.audio_channels_in_dvd>5) {
      radiobutton_audio_channel_5 = gtk_radio_button_new_with_label (audio_channels_group, Config.audio_channel_names[5]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_5));
      gtk_widget_ref (radiobutton_audio_channel_5);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_5", radiobutton_audio_channel_5,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_5);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_5, FALSE, FALSE, 0);
  }

  /* Audio 6 */
  if (Config.audio_channels_in_dvd>6) {
      radiobutton_audio_channel_6 = gtk_radio_button_new_with_label (audio_channels_group, Config.audio_channel_names[6]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_6));
      gtk_widget_ref (radiobutton_audio_channel_6);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_6", radiobutton_audio_channel_6,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_6);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_6, FALSE, FALSE, 0);
  }

  /* Audio 7 */
  if (Config.audio_channels_in_dvd>7) {
      radiobutton_audio_channel_7 = gtk_radio_button_new_with_label (audio_channels_group, Config.audio_channel_names[7]->str);
      audio_channels_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton_audio_channel_7));
      gtk_widget_ref (radiobutton_audio_channel_7);
      gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "radiobutton_audio_channel_7", radiobutton_audio_channel_7,
                                (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (radiobutton_audio_channel_7);
      gtk_box_pack_start (GTK_BOX (vbox9), radiobutton_audio_channel_7, FALSE, FALSE, 0);
  }

  hbox22 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox22);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "hbox22", hbox22,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox22);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox22, TRUE, TRUE, 5);

  label6 = gtk_label_new (_(" Select subpicture   "));
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox22), label6, FALSE, FALSE, 0);
  //gtk_misc_set_alignment (GTK_MISC (label6), 0.5, 0.05);

  /* Subpictures */
  combo_subpicture = gtk_combo_new ();
  gtk_widget_ref (combo_subpicture);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "combo_subpicture", combo_subpicture,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_subpicture);
  gtk_box_pack_start (GTK_BOX (hbox22), combo_subpicture, TRUE, TRUE, 0);
  combo_subpicture_items = g_list_append (combo_subpicture_items, (gpointer)"None");
  for (i=0;i<Config.subpictures_in_dvd;i++) {
      combo_subpicture_items = g_list_append (combo_subpicture_items, (gpointer)Config.subpicture_names[i]->str);
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_subpicture), combo_subpicture_items);
  g_list_free (combo_subpicture_items);

  combo_entry9 = GTK_COMBO (combo_subpicture)->entry;
  gtk_widget_ref (combo_entry9);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "combo_entry9", combo_entry9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry9);
  gtk_entry_set_text (GTK_ENTRY (combo_entry9), "None");

  dialog_action_area5 = GNOME_DIALOG (rip_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (rip_dialog), "dialog_action_area5", dialog_action_area5);
  gtk_widget_show (dialog_action_area5);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area5), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area5), 8);

  gnome_dialog_append_button (GNOME_DIALOG (rip_dialog), GNOME_STOCK_BUTTON_OK);
  button11 = GTK_WIDGET (g_list_last (GNOME_DIALOG (rip_dialog)->buttons)->data);
  gtk_widget_ref (button11);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "button11", button11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button11);
  GTK_WIDGET_SET_FLAGS (button11, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (rip_dialog), GNOME_STOCK_BUTTON_CANCEL);
  button13 = GTK_WIDGET (g_list_last (GNOME_DIALOG (rip_dialog)->buttons)->data);
  gtk_widget_ref (button13);
  gtk_object_set_data_full (GTK_OBJECT (rip_dialog), "button13", button13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button13);
  GTK_WIDGET_SET_FLAGS (button13, GTK_CAN_DEFAULT);

  if (Config.audio_channels_in_dvd>0) {  
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_0), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_0_toggled),
                          radiobutton_audio_channel_0);
  }
  if (Config.audio_channels_in_dvd>1) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_1), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_1_toggled),
                          radiobutton_audio_channel_1);
  }
  if (Config.audio_channels_in_dvd>2) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_2), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_2_toggled),
                          radiobutton_audio_channel_2);
  }
  if (Config.audio_channels_in_dvd>3) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_3), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_3_toggled),
                          radiobutton_audio_channel_3);
  }
  if (Config.audio_channels_in_dvd>4) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_4), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_4_toggled),
                          radiobutton_audio_channel_4);
  }
  if (Config.audio_channels_in_dvd>5) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_5), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_5_toggled),
                          radiobutton_audio_channel_5);
  }
  if (Config.audio_channels_in_dvd>6) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_6), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_6_toggled),
                          radiobutton_audio_channel_6);
  }
  if (Config.audio_channels_in_dvd>7) {
      gtk_signal_connect (GTK_OBJECT (radiobutton_audio_channel_7), "toggled",
                          GTK_SIGNAL_FUNC (on_radiobutton_audio_channel_7_toggled),
                          radiobutton_audio_channel_7);
  }
  gtk_signal_connect (GTK_OBJECT (combo_entry9), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry9_changed),
                      combo_entry9);
  gtk_signal_connect (GTK_OBJECT (button11), "clicked",
                      GTK_SIGNAL_FUNC (on_button11_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button13), "clicked",
                      GTK_SIGNAL_FUNC (on_button13_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (combo_entry2), "changed",
                      GTK_SIGNAL_FUNC (on_combo_entry2_changed),
                      combo_entry2);

  return rip_dialog;
}

GtkWidget*
create_about_drip (void)
{
  const gchar *authors[] = {
    "Jarl van Katwijk",
    NULL
  };
  GtkWidget *about_drip;

  about_drip = gnome_about_new ("Drip", VERSION,
                        _("GPL"),
                        authors,
                        _("Application GUI and backend set for backing up DVD content in DIVX format. Using Drip might not be legal in your situation"),
                        "drip/drip.png");
  gtk_object_set_data (GTK_OBJECT (about_drip), "about_drip", about_drip);
  gtk_window_set_modal (GTK_WINDOW (about_drip), TRUE);

  return about_drip;
}

GtkWidget*
create_messagebox_betanotice (void)
{
  GtkWidget *messagebox_betanotice;
  GtkWidget *dialog_vbox1;
  GtkWidget *button3;
  GtkWidget *dialog_action_area1;
  GtkWidget *href_drip;
  GtkWidget *href_drip_howto;
  GtkTooltips *tooltips;
  GString *temp;

  tooltips = gtk_tooltips_new ();

  /* We create it with an OK button, and then remove the button, to work
     around a bug in gnome-libs. */
  temp = g_string_new("");
  g_string_sprintf(temp,"This is a BETA RELEASE of Drip (%s), it needs more development.\nCheck for updates and report bugs please.\n\n",VERSION);
  messagebox_betanotice = gnome_message_box_new (temp->str,
                              GNOME_MESSAGE_BOX_WARNING,
                              GNOME_STOCK_BUTTON_OK, NULL);
  gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (messagebox_betanotice)->action_area), GNOME_DIALOG (messagebox_betanotice)->buttons->data);
  GNOME_DIALOG (messagebox_betanotice)->buttons = NULL;
  gtk_object_set_data (GTK_OBJECT (messagebox_betanotice), "messagebox_betanotice", messagebox_betanotice);
  gtk_window_set_title (GTK_WINDOW (messagebox_betanotice), _("Drip - Beta Warning"));
  gtk_window_set_position (GTK_WINDOW (messagebox_betanotice), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (messagebox_betanotice), FALSE, FALSE, FALSE);
  gtk_window_set_modal (GTK_WINDOW (messagebox_betanotice), TRUE);

  dialog_vbox1 = GNOME_DIALOG (messagebox_betanotice)->vbox;
  gtk_object_set_data (GTK_OBJECT (messagebox_betanotice), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);



  href_drip = gnome_href_new ("http://drip.sourceforge.net/", _("Drip Homepage"));  gtk_widget_ref (href_drip);
  gtk_object_set_data_full (GTK_OBJECT (messagebox_betanotice), "href_drip", href_drip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (href_drip);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), href_drip, FALSE, FALSE, 0);

  href_drip_howto = gnome_href_new ("http://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/~checkout~/drip/drip/doc/C/drip/index.html?rev=1.18&content-type=text/html", _("Encoding HOWTO"));  gtk_widget_ref (href_drip_howto);
  gtk_object_set_data_full (GTK_OBJECT (messagebox_betanotice), "href_drip", href_drip_howto,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (href_drip_howto);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), href_drip_howto, FALSE, FALSE, 0);


  gnome_dialog_append_button (GNOME_DIALOG (messagebox_betanotice), GNOME_STOCK_BUTTON_OK);
  button3 = GTK_WIDGET (g_list_last (GNOME_DIALOG (messagebox_betanotice)->buttons)->data);
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (messagebox_betanotice), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  dialog_action_area1 = GNOME_DIALOG (messagebox_betanotice)->action_area;
  gtk_widget_ref (dialog_action_area1);
  gtk_object_set_data_full (GTK_OBJECT (messagebox_betanotice), "dialog_action_area1", dialog_action_area1,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_object_set_data (GTK_OBJECT (messagebox_betanotice), "tooltips", tooltips);

  return messagebox_betanotice;
}


GtkWidget*
create_messagebox_buzy (void)
{
  GtkWidget *messagebox_buzy;
  GtkWidget *dialog_vbox2;
  GtkWidget *button4;
  GtkWidget *dialog_action_area2;

  /* We create it with an OK button, and then remove the button, to work
     around a bug in gnome-libs. */
  messagebox_buzy = gnome_message_box_new (_("Drip is buzy converting,\nplease wait or Stop the process."),
                              GNOME_MESSAGE_BOX_INFO,
                              GNOME_STOCK_BUTTON_OK, NULL);
  gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (messagebox_buzy)->action_area), GNOME_DIALOG (messagebox_buzy)->buttons->data);
  GNOME_DIALOG (messagebox_buzy)->buttons = NULL;
  gtk_object_set_data (GTK_OBJECT (messagebox_buzy), "messagebox_buzy", messagebox_buzy);
  gtk_window_set_title (GTK_WINDOW (messagebox_buzy), _("Drip - Buzy"));
  gtk_window_set_position (GTK_WINDOW (messagebox_buzy), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (messagebox_buzy), TRUE);
  gtk_window_set_policy (GTK_WINDOW (messagebox_buzy), FALSE, FALSE, FALSE);

  dialog_vbox2 = GNOME_DIALOG (messagebox_buzy)->vbox;
  gtk_object_set_data (GTK_OBJECT (messagebox_buzy), "dialog_vbox2", dialog_vbox2);
  gtk_widget_show (dialog_vbox2);

  gnome_dialog_append_button (GNOME_DIALOG (messagebox_buzy), GNOME_STOCK_BUTTON_OK);
  button4 = GTK_WIDGET (g_list_last (GNOME_DIALOG (messagebox_buzy)->buttons)->data);
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (messagebox_buzy), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

  dialog_action_area2 = GNOME_DIALOG (messagebox_buzy)->action_area;
  gtk_widget_ref (dialog_action_area2);
  gtk_object_set_data_full (GTK_OBJECT (messagebox_buzy), "dialog_action_area2", dialog_action_area2,
                            (GtkDestroyNotify) gtk_widget_unref);

  return messagebox_buzy;
}


GtkWidget*
create_infobox(gchar *text)
{
  GtkWidget *infobox;
  GtkWidget *dialog_vbox;
  GtkWidget *button;
  GtkWidget *dialog_action_area;

  /* We create it with an OK button, and then remove the button, to work
     around a bug in gnome-libs. */
  infobox = gnome_message_box_new (_(text),
                 GNOME_MESSAGE_BOX_INFO,
                 GNOME_STOCK_BUTTON_OK, NULL);
  gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (infobox)->action_area), GNOME_DIALOG (infobox)->buttons->data);
  GNOME_DIALOG (infobox)->buttons = NULL;
  gtk_object_set_data (GTK_OBJECT (infobox), "infobox",infobox);
  gtk_window_set_title (GTK_WINDOW (infobox), _("Drip - Information"));
  gtk_window_set_position (GTK_WINDOW (infobox), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (infobox), TRUE);
  gtk_window_set_policy (GTK_WINDOW (infobox), FALSE, FALSE, FALSE);

  dialog_vbox = GNOME_DIALOG (infobox)->vbox;
  gtk_object_set_data (GTK_OBJECT (infobox), "dialog_vbox", dialog_vbox);
  gtk_widget_show (dialog_vbox);

  gnome_dialog_append_button (GNOME_DIALOG (infobox), GNOME_STOCK_BUTTON_OK);
  button = GTK_WIDGET (g_list_last (GNOME_DIALOG (infobox)->buttons)->data);
  gtk_widget_ref (button);
  gtk_object_set_data_full (GTK_OBJECT (infobox), "button", button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  dialog_action_area = GNOME_DIALOG (infobox)->action_area;
  gtk_widget_ref (dialog_action_area);
  gtk_object_set_data_full (GTK_OBJECT (infobox), "dialog_action_area", dialog_action_area,
                            (GtkDestroyNotify) gtk_widget_unref);

  return infobox;
}


GtkWidget*
create_manual_selection_dialog (gint files,struct dirent **vobsdir,gchar *track_dir) {
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox3;
  GtkWidget *vbox5;
  GtkWidget *label_dvd_sources;
  GtkWidget *scrolledwindow2;
  GtkWidget *clist1;
  GtkWidget *label_select_file;
  GtkWidget *label_select_size;
  GtkWidget *label_select_rip;
  GtkWidget *dialog_action_area3;
  GtkWidget *button5;
  GtkWidget *button7;
  gint cnt;
  gchar *row[4];
  GdkColor col_yes,col_no;
  struct stat file_stats;
  GString *col0 = g_string_new("");
  GString *col1 = g_string_new("");
  GString *col2 = g_string_new("");
  GString *col3 = g_string_new("");

  dialog1 = gnome_dialog_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_policy (GTK_WINDOW (dialog1), TRUE, TRUE, TRUE);
  gtk_widget_set_usize (dialog1, 600, 600);
  gtk_window_set_title (GTK_WINDOW (dialog1), _("Drip - Source selection"));

  dialog_vbox3 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox3", dialog_vbox3);
  gtk_widget_show (dialog_vbox3);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox5", vbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), vbox5, TRUE, TRUE, 0);

  label_dvd_sources = gtk_label_new (_("Select DVD source files to be converted\nSelect IFO files if you want to backup a sequence\n"));  gtk_widget_ref (label_dvd_sources);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label_dvd_sources", label_dvd_sources,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_dvd_sources);
  gtk_box_pack_start (GTK_BOX (vbox5), label_dvd_sources, FALSE, FALSE, 0);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "scrolledwindow2", scrolledwindow2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow2);
  gtk_box_pack_start (GTK_BOX (vbox5), scrolledwindow2, TRUE, TRUE, 0);

  clist1 = gtk_clist_new (4);
  gtk_widget_ref (clist1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "clist1", clist1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (clist1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), clist1);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 0, 160);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 1, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 2, 150);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 3, 80);
  gtk_clist_set_column_auto_resize (GTK_CLIST (clist1), 0,TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (clist1), 1,TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (clist1), 2,TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (clist1), 3,TRUE);
  gtk_clist_column_titles_show (GTK_CLIST (clist1));

  label_select_file = gtk_label_new (_("DVD file"));
  gtk_widget_ref (label_select_file);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label_select_file", label_select_file,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_select_file);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 0, label_select_file);

  label_select_size = gtk_label_new (_("Size (kB)"));
  gtk_widget_ref (label_select_size);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label_select_size", label_select_size,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_select_size);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 1, label_select_size);

  label_select_size = gtk_label_new (_("Description"));
  gtk_widget_ref (label_select_size);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label_select_size", label_select_size,                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_select_size);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 2, label_select_size);

  label_select_rip = gtk_label_new (_("Use this file?"));
  gtk_widget_ref (label_select_rip);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label_select_rip", label_select_rip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_select_rip);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 3, label_select_rip);

  /* Allocate colours */
  gdk_color_parse("tomato3",&col_no);
  gdk_color_parse("green3",&col_yes);

  for (cnt = 0; cnt < files; ++cnt) {
      
      g_string_sprintf(col0,"%s",vobsdir[cnt]->d_name);
      row[0] = col0->str;

      g_string_sprintf(col1,"%s%s",track_dir,vobsdir[cnt]->d_name);
      stat(col1->str, &file_stats);
      g_string_sprintf(col1,"   %li   ",(long)file_stats.st_size);
      row[1] = col1->str;

      /* lowercase filename, so both UDF and ISO9660 can be handled */
      g_strdown(vobsdir[cnt]->d_name);
      /* Build description and convertable data */
      if (strstr(vobsdir[cnt]->d_name,"subpicture")!=NULL) {
          row[2] = "   Subpicture stream   ";
          row[3] = " ";
          gtk_clist_append(GTK_CLIST(clist1),row);
          } else {
          if (strstr(vobsdir[cnt]->d_name,"video_ts.ifo")!=NULL) {
              row[2] = "   Secondary navigational information   ";
              row[3] = " ";
              gtk_clist_append(GTK_CLIST(clist1),row);
          } else {
              if (strstr(vobsdir[cnt]->d_name,"video_ts.bup")!=NULL) {
                  row[2] = "   Secondary Navigational information backup   ";
                  row[3] = " ";
                  gtk_clist_append(GTK_CLIST(clist1),row);
              } else {
                  if (strstr(vobsdir[cnt]->d_name,".vob")!=NULL) {
                      row[2] = "   Video Transport Stream   ";
                      if ( (strstr(vobsdir[cnt]->d_name,"_0.vob")==NULL) && (strstr(vobsdir[cnt]->d_name,"video")==NULL) ) {
                          row[3] = "   NO   ";
                          gtk_clist_append(GTK_CLIST(clist1),row);
                          gtk_clist_set_background(GTK_CLIST(clist1),cnt,&col_no);
                      } else {
                          row[3] = " ";
                          gtk_clist_append(GTK_CLIST(clist1),row);
                      }
                  } else {
                      if ( (strstr(vobsdir[cnt]->d_name,".ifo")!=NULL) && (strstr(vobsdir[cnt]->d_name,"vts")!=NULL) ) {
                          row[2] = "   Navigational information   ";
                          row[3] = "   NO   ";
                          gtk_clist_append(GTK_CLIST(clist1),row);
                          gtk_clist_set_background(GTK_CLIST(clist1),cnt,&col_no);
                      } else {
                          if (strstr(vobsdir[cnt]->d_name,".bup")!=NULL) {
                              row[2] = "   Navigational info backup   ";
                              row[3] = " ";
                              gtk_clist_append(GTK_CLIST(clist1),row);
                          } else {
                              if (strstr(vobsdir[cnt]->d_name,".aob")!=NULL) {
                                  row[2] = "   Audio Transport Stream   ";
                                  row[3] = " ";
                                  gtk_clist_append(GTK_CLIST(clist1),row);
                              }
                          }
                      }
                  }
              }
          }
      }
  }

  /* -------- */

  dialog_action_area3 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area3", dialog_action_area3);
  gtk_widget_show (dialog_action_area3);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area3), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area3), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button5 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button5", button5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button5);
  GTK_WIDGET_SET_FLAGS (button5, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button7 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button7", button7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button7);
  GTK_WIDGET_SET_FLAGS (button7, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (clist1), "select_row",
                      GTK_SIGNAL_FUNC (on_dvdstreams_select_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (on_dvdstream_ok_clicked),
                      clist1);
  gtk_signal_connect (GTK_OBJECT (button7), "clicked",
                      GTK_SIGNAL_FUNC (on_dvdstream_cancel_clicked),
                      NULL);

  g_string_free(col0,TRUE);
  g_string_free(col1,TRUE);
  g_string_free(col2,TRUE);
  g_string_free(col3,TRUE);
  return dialog1;
}

GtkWidget*
create_calculate_bitrate (void)
{
  GtkWidget *dialog2;
  GtkWidget *dialog_vbox4;
  GtkWidget *vbox6;
  GtkWidget *hbox10;
  GtkWidget *label_calc_size;
  GtkObject *spinbutton1_adj;
  GtkWidget *hbox12;
  GtkWidget *label_calc_seconds;
  GtkObject *spinbutton2_adj;
  GtkWidget *hbox11;
  GtkWidget *label_calc_audio;
  GtkObject *spinbutton3_adj;
  GtkWidget *hseparator1;
  GtkWidget *hbox14;
  GtkWidget *dialog_action_area4;
  GtkWidget *label_calc_bitrate;
  GtkWidget *button8;
  GtkWidget *button10;

  dialog2 = gnome_dialog_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (dialog2), "dialog2", dialog2);
  gtk_widget_set_usize (dialog2, 500, -2);
  gtk_window_set_policy (GTK_WINDOW (dialog2), FALSE, FALSE, FALSE);

  dialog_vbox4 = GNOME_DIALOG (dialog2)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog2), "dialog_vbox4", dialog_vbox4);
  gtk_widget_show (dialog_vbox4);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox6);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "vbox6", vbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox4), vbox6, TRUE, TRUE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (vbox6), hbox10, FALSE, FALSE, 0);

  label_calc_size = gtk_label_new (_(" Requested size (bytes) "));
  gtk_widget_ref (label_calc_size);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "label_calc_size", label_calc_size,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_calc_size);
  gtk_box_pack_start (GTK_BOX (hbox10), label_calc_size, FALSE, FALSE, 0);

  spinbutton1_adj = gtk_adjustment_new (737280000, 0, 9999999999, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton1);
  gtk_box_pack_start (GTK_BOX (hbox10), spinbutton1, TRUE, TRUE, 0);

  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox12);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "hbox12", hbox12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox12);
  gtk_box_pack_start (GTK_BOX (vbox6), hbox12, TRUE, TRUE, 0);

  label_calc_seconds = gtk_label_new (_(" Movie duration (minutes) "));
  gtk_widget_ref (label_calc_seconds);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "label_calc_seconds", label_calc_seconds,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_calc_seconds);
  gtk_box_pack_start (GTK_BOX (hbox12), label_calc_seconds, FALSE, FALSE, 0);

  spinbutton2_adj = gtk_adjustment_new (90, 0, 999999999, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_ref (spinbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "spinbutton2", spinbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton2);
  gtk_box_pack_start (GTK_BOX (hbox12), spinbutton2, TRUE, TRUE, 0);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox11);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "hbox11", hbox11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox11);
  gtk_box_pack_start (GTK_BOX (vbox6), hbox11, TRUE, TRUE, 0);

  label_calc_audio = gtk_label_new (_(" Audio (kBit) "));
  gtk_widget_ref (label_calc_audio);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "label_calc_audio", label_calc_audio,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_calc_audio);
  gtk_box_pack_start (GTK_BOX (hbox11), label_calc_audio, FALSE, FALSE, 0);

  spinbutton3_adj = gtk_adjustment_new (128, 0, 300, 1, 10, 10);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_widget_ref (spinbutton3);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "spinbutton3", spinbutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton3);
  gtk_box_pack_start (GTK_BOX (hbox11), spinbutton3, TRUE, TRUE, 0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "hseparator1", hseparator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox6), hseparator1, TRUE, TRUE, 10);

  hbox14 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox14);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "hbox14", hbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox14);
  gtk_box_pack_start (GTK_BOX (vbox6), hbox14, TRUE, TRUE, 0);

  label_calc_bitrate = gtk_label_new (_(" Encoding bitrate "));
  gtk_widget_ref (label_calc_bitrate);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "label_calc_bitrate", label_calc_bitrate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_calc_bitrate);
  gtk_box_pack_start (GTK_BOX (hbox14), label_calc_bitrate, FALSE, FALSE, 0);

  entry3 = gtk_entry_new ();
  gtk_widget_ref (entry3);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "entry3", entry3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry3);
  gtk_box_pack_start (GTK_BOX (hbox14), entry3, TRUE, TRUE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry3), FALSE);

  dialog_action_area4 = GNOME_DIALOG (dialog2)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog2), "dialog_action_area4", dialog_action_area4);
  gtk_widget_show (dialog_action_area4);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area4), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area4), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog2), GNOME_STOCK_BUTTON_OK);
  button8 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog2)->buttons)->data);
  gtk_widget_ref (button8);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "button8", button8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button8);
  GTK_WIDGET_SET_FLAGS (button8, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog2), GNOME_STOCK_BUTTON_CLOSE);  button10 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog2)->buttons)->data);
  gtk_widget_ref (button10);
  gtk_object_set_data_full (GTK_OBJECT (dialog2), "button10", button10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button10);
  GTK_WIDGET_SET_FLAGS (button10, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button8), "clicked",
                      GTK_SIGNAL_FUNC (on_button8_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button10), "clicked",
                      GTK_SIGNAL_FUNC (on_button10_clicked),
                      dialog2);

  return dialog2;
}

/* Create a new backing pixmap of the appropriate size */
static gint configure_event( GtkWidget         *widget,
                             GdkEventConfigure *event )
{
  if (pixmap)
    gdk_pixmap_unref(pixmap);

  pixmap = gdk_pixmap_new(widget->window,
              widget->allocation.width,
              widget->allocation.height,
              -1);
  gdk_draw_rectangle (pixmap,
              widget->style->white_gc,
              TRUE,
              0, 0,
              widget->allocation.width,
              widget->allocation.height);

  return TRUE;
}

/* Redraw the screen from the backing pixmap */
static gint expose_event( GtkWidget      *widget,
                          GdkEventExpose *event )
{
  gdk_draw_pixmap(widget->window,
          widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
          pixmap,
          event->area.x, event->area.y,
          event->area.x, event->area.y,
          event->area.width, event->area.height);

  return FALSE;
}

/* Draw a rectangle on the screen */
static void draw_brush( GtkWidget *widget,
                        gdouble    x,
                        gdouble    y)
{
  GdkRectangle update_rect;

  update_rect.x = x;
  update_rect.y = y;
  update_rect.width = 1;
  update_rect.height = 1;
  gdk_draw_rectangle (pixmap,
              widget->style->black_gc,
              TRUE,
              update_rect.x, update_rect.y,
              update_rect.width, update_rect.height);
  gtk_widget_draw (widget, &update_rect);
}

static gint button_press_event( GtkWidget      *widget,
                                GdkEventButton *event )
{
  if (event->button == 1 && pixmap != NULL)
    draw_brush (widget, event->x, event->y);

  return TRUE;
}

static gint motion_notify_event( GtkWidget *widget,
                                 GdkEventMotion *event )
{
  int x, y;
  GdkModifierType state;

  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }

  if (state & GDK_BUTTON1_MASK && pixmap != NULL)
    draw_brush (widget, x, y);

  return TRUE;
}


GtkWidget*
create_clipping_dialog (void) {
  GtkWidget *clipping_dialog;
  GtkWidget *dialog_vbox6;
  GtkWidget *drawing_area;
  GtkWidget *dialog_action_area6;
  GtkWidget *button14;
  GtkWidget *button16;

  clipping_dialog = gnome_dialog_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (clipping_dialog), "clipping_dialog", clipping_dialog);

  dialog_vbox6 = GNOME_DIALOG (clipping_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (clipping_dialog), "dialog_vbox6", dialog_vbox6);
  gtk_widget_show (dialog_vbox6);

  drawing_area = gtk_drawing_area_new ();
  gtk_widget_ref (drawing_area);
  gtk_object_set_data_full (GTK_OBJECT (clipping_dialog), "drawingarea1", drawing_area,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (drawing_area);
  gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), 200, 200);
  gtk_box_pack_start (GTK_BOX (dialog_vbox6), drawing_area, TRUE, TRUE, 0);

  /* Signals used to handle backing pixmap */

  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
              (GtkSignalFunc) expose_event, NULL);
  gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",
              (GtkSignalFunc) configure_event, NULL);

  /* Event signals */

  gtk_signal_connect (GTK_OBJECT (drawing_area), "motion_notify_event",
              (GtkSignalFunc) motion_notify_event, NULL);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
              (GtkSignalFunc) button_press_event, NULL);

  gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
             | GDK_LEAVE_NOTIFY_MASK
             | GDK_BUTTON_PRESS_MASK
             | GDK_POINTER_MOTION_MASK
             | GDK_POINTER_MOTION_HINT_MASK);

  dialog_action_area6 = GNOME_DIALOG (clipping_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (clipping_dialog), "dialog_action_area6", dialog_action_area6);
  gtk_widget_show (dialog_action_area6);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area6), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area6), 8);

  gnome_dialog_append_button (GNOME_DIALOG (clipping_dialog), GNOME_STOCK_BUTTON_OK);
  button14 = GTK_WIDGET (g_list_last (GNOME_DIALOG (clipping_dialog)->buttons)->data);
  gtk_widget_ref (button14);
  gtk_object_set_data_full (GTK_OBJECT (clipping_dialog), "button14", button14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button14);
  GTK_WIDGET_SET_FLAGS (button14, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (clipping_dialog), GNOME_STOCK_BUTTON_CANCEL);
  button16 = GTK_WIDGET (g_list_last (GNOME_DIALOG (clipping_dialog)->buttons)->data);
  gtk_widget_ref (button16);
  gtk_object_set_data_full (GTK_OBJECT (clipping_dialog), "button16", button16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button16);
  GTK_WIDGET_SET_FLAGS (button16, GTK_CAN_DEFAULT);

  return clipping_dialog;
}

GtkWidget*
create_stop_dialog(void)
{
  GtkWidget *dialog3;
  GtkWidget *dialog_vbox7;
  GtkWidget *stop_text;
  GtkWidget *dialog_action_area7;
  GtkWidget *button_ok_stop;
  GtkWidget *button_no_stop;

  dialog3 = gnome_dialog_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (dialog3), "dialog3", dialog3);
  gtk_window_set_policy (GTK_WINDOW (dialog3), FALSE, FALSE, TRUE);
  gtk_window_set_title (GTK_WINDOW (dialog3), _("Drip - Stop"));

  dialog_vbox7 = GNOME_DIALOG (dialog3)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog3), "dialog_vbox7", dialog_vbox7);
  gtk_widget_show (dialog_vbox7);

  stop_text = gtk_label_new (_("\nStop encoding process?\n"));
  gtk_widget_ref (stop_text);
  gtk_object_set_data_full (GTK_OBJECT (dialog3), "stop_text", stop_text,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (stop_text);
  gtk_box_pack_start (GTK_BOX (dialog_vbox7), stop_text, FALSE, FALSE, 0);

  dialog_action_area7 = GNOME_DIALOG (dialog3)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog3), "dialog_action_area7", dialog_action_area7);
  gtk_widget_show (dialog_action_area7);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area7), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area7), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog3), GNOME_STOCK_BUTTON_YES);
  button_ok_stop = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog3)->buttons)->data);
  gtk_widget_ref (button_ok_stop);
  gtk_object_set_data_full (GTK_OBJECT (dialog3), "button_ok_stop", button_ok_stop,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_ok_stop);
  GTK_WIDGET_SET_FLAGS (button_ok_stop, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog3), GNOME_STOCK_BUTTON_NO);
  button_no_stop = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog3)->buttons)->data);
  gtk_widget_ref (button_no_stop);
  gtk_object_set_data_full (GTK_OBJECT (dialog3), "button_no_stop", button_no_stop,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button_no_stop);
  GTK_WIDGET_SET_FLAGS (button_no_stop, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button_ok_stop), "clicked",
                      GTK_SIGNAL_FUNC (on_button_ok_stop_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button_no_stop), "clicked",
                      GTK_SIGNAL_FUNC (on_button_no_stop_clicked),
                      NULL);

  return dialog3;
}

