/*
 * Drip - Jarl van Katwijk
 * 
 * Drip is GPL licenced
 */

#include "../config.h"
#include <gnome.h>
#include <libgnorba/gnorba.h>
#include <glib.h>
#include "interface.h"
#include "support.h"
#include "drip.h"
#include "corbashared.h"
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <fcntl.h>
#include "callbacks.h"
#include <orb/orbit.h>
#include "encoder.h"
#include "gui.h"
#include <sys/time.h>



/* local proto's */
gboolean corba_init(void);
gboolean corba_close(void);
gpointer start_gui_server_orb(gpointer gev);
void ping(PortableServer_Servant servant,CORBA_long value,CORBA_Environment *ev);
void ready(PortableServer_Servant servant,CORBA_Environment *ev);
void log(PortableServer_Servant servant,CORBA_long level,const char* message,CORBA_Environment *ev);
void progress(PortableServer_Servant servant,CORBA_long frame,CORBA_long total,CORBA_long currentvob,CORBA_long totalvobs,CORBA_Environment *ev);
void frame(PortableServer_Servant servant,CORBA_long frame,CORBA_long x,CORBA_long y,CORBA_Environment *ev);
void errorsignal(PortableServer_Servant servant,CORBA_long signal,CORBA_Environment *ev);
ORBit_MessageValidationResult drip_ORBit_request_validate(CORBA_unsigned_long request_id,CORBA_Principal *principal,CORBA_char *operation);
gpointer corba_hook(gpointer data);
gboolean logging_locked = FALSE;
GdkFont *logfont;
GdkColor col_debug,col_warning,col_error,col_critical,col_info,col_message;
struct timeval encoding_tv;
struct timezone encoding_tz={0,0};
gboolean encoding_time = FALSE;



/* Structs */
PortableServer_ServantBase__epv base_epv = {
  NULL,
  NULL,
  NULL
};


/* Globals */
static CORBA_Principal _gnorba_request_cookie = { 0, 0, NULL, CORBA_TRUE };
POA_drip_gui__epv drip_gui_epv = { NULL, ping, ready, log,progress,frame,errorsignal};
POA_drip_gui__vepv poa_drip_gui_vepv = { &base_epv, &drip_gui_epv };
POA_drip_gui poa_drip_gui_servant = { NULL, &poa_drip_gui_vepv };

gint log_file;
drip_config Config;
GString *status_action;
guint status_perc;
gint linenr = 0;
guint log_handler_drip_id;
drip_encoder encoder_client;
drip_gui drip_gui_client;
CORBA_Environment ev;
CORBA_ORB orb;
pthread_t serverorbthread;
gint serverorbthread_id;
gboolean server_orb = FALSE;
gboolean corba = FALSE;
GtkWidget *mainwindow;
GtkWidget *framewindow = NULL;
gpointer currentframebuffer;
gint xd = 0;
gboolean making_iso;


/* CORBA IMPLEMENTATIONS */
void ping(PortableServer_Servant servant,CORBA_long value,CORBA_Environment *ev) {
    gdk_threads_enter();
    gtk_widget_set_sensitive(mainwindow,TRUE);
    gdk_threads_leave();

    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"CORBA Ping %i",value);
    if (value==1234) {
        g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"GUI and backend ready.");
    } else {
        g_log(DRIP_LD,G_LOG_LEVEL_WARNING,"Backend not compatible, exiting in 10 seconds...");
        sleep(10);
        terminate(FALSE);
    }
    return;
}

void ready(PortableServer_Servant servant,CORBA_Environment *ev) {
    /* Done encoding, reset and notice user */
    GString *command = g_string_new("");
    gint i;

    /* Rename generated files */
    rename_divx_files();

    /* Notice user */
    g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Ready, encoding done.");
    gtk_widget_destroy(gpixmap);

    /* Reset */
    if (!Config.cache) {
        /* eject DVDrom */
        if (Config.eject_dvdrom) {
            eject_dvdrom(Config.track_dir->str,Config.DVDdevice->str);
        }
    } else {
        /* clean up temp files of HD cached encoding */
        unlink("disc-key");
        unlink("title-key");
    }

    if (Config.create_iso == TRUE) {
        /* Make ISO images out of generated DIVX's */
        pthread_t isothread;
        gint isothread_id;
        making_iso = TRUE;
        isothread_id = pthread_create(&isothread,NULL,make_iso_images,NULL);
        while (making_iso==TRUE) usleep(100000);
    }

    /* Mark conversion status  & clean file selection */
    converting = FALSE;
    for (i=0;i<selected_files;i++) {
        g_string_free(files[i],TRUE);
    }
    selected_files = -1;

    /* Set GUI statusbar */
    gdk_threads_enter();
    gnome_appbar_set_status(GNOME_APPBAR(appbar1),"Ready.");
    gnome_appbar_set_progress(GNOME_APPBAR(appbar1),0);
    gdk_threads_leave();

    /* Clean & exit */
    g_string_free(command,TRUE);
    return;
}

void log(PortableServer_Servant servant,CORBA_long level,const char* message,CORBA_Environment *ev) {
    g_log(DRIP_LD,level,"BACKEND: %s",message);
    return;
}

void progress(PortableServer_Servant servant,CORBA_long frame,CORBA_long total,CORBA_long currentvob,CORBA_long totalvobs,CORBA_Environment *ev) {
    /* Display progress to user */
    GString *frames = g_string_new("");
    gfloat base = totalvobs;
    gfloat perc = totalvobs;
    gfloat part = 0;
    GString *eta;
    struct timeval current_tv;
    struct timezone current_tz={0,0};
    gulong seconds_left = 59;
    gulong minutes_left = 59;
    gulong hours_left = 99;



    if (total>0) {
        /* Calc perc */
        if (perc>0) {
            perc = 1/perc*(currentvob+1);
            if (total>0) {
                part = perc*((gfloat)frame/(gfloat)total);
            }
        }
        if (base>0) {
            base = 1/base*currentvob;
        }

        /* Check if progress makes any sense, correct if not */
        if (perc>1)    { perc = 1;}
        if (part>perc) { part = perc;}
        if (part<base) { part = base;}
        if (part==0)   { part = 0.0000001;};

        /* Build ETA time string */
        if (encoding_time == FALSE) {
            /* get start time */
            gettimeofday(&encoding_tv,&encoding_tz);
            encoding_time = TRUE;
        }
        gettimeofday(&current_tv,&current_tz);
        seconds_left = (gulong)((current_tv.tv_sec - encoding_tv.tv_sec) * (1-part) / part);

        hours_left = (guint)(seconds_left/3600);
        seconds_left -= 3600*hours_left;

        minutes_left = (guint)(seconds_left/60);
        seconds_left -= 60*minutes_left;

        eta = g_string_new("");
        if (hours_left>0) {
            g_string_sprintf(eta,"%li hours %li minutes",hours_left,minutes_left);
        } else {
            g_string_sprintf(eta,"%li minutes %li seconds",minutes_left,seconds_left);
        }

        /* Show */
        g_string_sprintf(frames,"Frame %i/%i (%.2f fps),  %s remaining",frame,total,(gfloat)frame/(gfloat)(current_tv.tv_sec - encoding_tv.tv_sec),eta->str);
        gdk_threads_enter();
        gnome_appbar_set_status(GNOME_APPBAR(appbar1),frames->str);
        gnome_appbar_set_progress(GNOME_APPBAR(appbar1),part);
        gdk_threads_leave();
        g_string_free(eta,TRUE);
    } else {
        g_string_sprintf(frames,"Encoding frame %i",frame);
        gdk_threads_enter();
        gnome_appbar_set_status(GNOME_APPBAR(appbar1),frames->str);
        gnome_appbar_set_progress(GNOME_APPBAR(appbar1),part);
        gdk_threads_leave();
    }

    /* Clean & exit */
    g_string_free(frames,TRUE);
    return;
}

/* Show progress frame that's been generate by the backend every N frames */
void frame(PortableServer_Servant servant,CORBA_long frame,CORBA_long x,CORBA_long y,CORBA_Environment *ev) {
    gint ffd;
    gint yd;

    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"New progress frame %i is available (%i,%i)",frame,x,y);
    gdk_threads_enter();

    /* Clean previous frame */
    if (gpixmap!=NULL) {
        gtk_widget_destroy(gpixmap);
        g_free(currentframebuffer);
    }
    /* Allocate frame window */
    currentframebuffer = g_malloc(x*y*3);
    /* Build frame window */
    ffd = open("/tmp/dripframe.raw",O_RDONLY);
    read(ffd,currentframebuffer,x*y*3);
    close(ffd);
    /* Build new pixmap widget */
    //if (xd==0) { xd = hbox_main->allocation.width;}  // FIXED SIZE
    xd = hbox_main->allocation.width;                  // ADAPTIVE SIZE

    yd = y*((gfloat)xd/x);
    gpixmap = gnome_pixmap_new_from_rgb_d_at_size(currentframebuffer,NULL,x,y,xd*3/4,yd*3/4); // TODO: ASCPECT
    gtk_box_pack_start (GTK_BOX (hbox_main), gpixmap, FALSE, FALSE, 0);
    gtk_widget_show(gpixmap);

    gdk_threads_leave();
    return;
}

void errorsignal(PortableServer_Servant servant,CORBA_long signal,CORBA_Environment *ev) {
    g_log(DRIP_LD,G_LOG_LEVEL_CRITICAL,"BACKEND: Signal %i has occured! Please send a detailed bug report to the Drip Project",signal);
    return;
}



/* Regulair functions */

static gboolean
orb_handle_connection(GIOChannel *source, GIOCondition cond,GIOPConnection *cnx) {
    /* The best way to know about an fd exception is if select()/poll()
     * tells you about it, so we just relay that information on to ORBit
     * if possible
     */
    if(cond & (G_IO_HUP|G_IO_NVAL|G_IO_ERR))
        giop_main_handle_connection_exception(cnx);
    else
        giop_main_handle_connection(cnx);
    return TRUE;
}

static void orb_add_connection(GIOPConnection *cnx) {
    int tag;
    GIOChannel *channel;

    channel = g_io_channel_unix_new(GIOP_CONNECTION_GET_FD(cnx));
    tag = g_io_add_watch_full   (channel, G_PRIORITY_DEFAULT,
                                (GIOCondition)(G_IO_IN|G_IO_ERR|G_IO_HUP|G_IO_NVAL),
                                (GIOFunc)orb_handle_connection,
                                cnx, NULL);
    g_io_channel_unref (channel);
    cnx->user_data = GUINT_TO_POINTER (tag);
    return;
}

static void orb_remove_connection(GIOPConnection *cnx) {
    gtk_input_remove((guint)cnx->user_data);
    cnx->user_data = (gpointer)-1;
    return;
}

ORBit_MessageValidationResult drip_ORBit_request_validate(CORBA_unsigned_long request_id,CORBA_Principal *principal,CORBA_char *operation) {
    /* Disable gnorba authentication */
    return ORBIT_MESSAGE_ALLOW_ALL;
}

/* Starts the server orb */
gpointer
start_gui_server_orb(gpointer gev) {
    CORBA_Environment ev;
    PortableServer_ObjectId objid = {0, sizeof("gnomedrip"), "gnomedrip"};
    PortableServer_POA poa;
    int argc = 1;
    char *argv[2];
    char *retval;
    CORBA_ORB orb;
    gint fd;
    GIOChannel *file;
    gint bytessaved;

    argv[0] = (gchar*)"gnomedripserver";
    argv[1] = 0;

    /* Debug */
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"start_gui_server_orb, pid = %i",getpid());

    /* init orb */
    CORBA_exception_init(&ev);

    gdk_threads_enter();
    //DISABLED; orb = gnome_CORBA_init("gnomedrip", VERSION, &argc, argv, 0, &ev);
    orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    gdk_threads_leave();
    //orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", ev);
    IIOPAddConnectionHandler = orb_add_connection;
    IIOPRemoveConnectionHandler = orb_remove_connection;
    /* Disable gnorba authentication meganism */
    ORBit_set_request_validation_handler(drip_ORBit_request_validate);
    ORBit_set_default_principal(&_gnorba_request_cookie);


    /* Setup POA */
    POA_drip_gui__init(&poa_drip_gui_servant, &ev);
    poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
    PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(poa, &ev), &ev);
    PortableServer_POA_activate_object_with_id(poa,&objid, &poa_drip_gui_servant, &ev);

    drip_gui_client = PortableServer_POA_servant_to_reference(poa,&poa_drip_gui_servant,&ev);
    if (!drip_gui_client) {
        g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"Cannot get objref");
    }

    /* Save ior to dripgui.ior */
    retval = CORBA_ORB_object_to_string(orb, drip_gui_client, &ev);
    fd = open("dripgui.ior",O_WRONLY|O_CREAT,S_IREAD|S_IWRITE);
    file = g_io_channel_unix_new(fd);
    g_io_channel_write(file,retval,strlen(retval),&bytessaved);
    g_io_channel_close(file);
    close(fd);

    /* run orb */
    CORBA_free(retval);
    /* connect ORBit ORB to glib event handler : */
    server_orb = TRUE;
    //ORBit_custom_run_setup(orb, ev);
    CORBA_ORB_run(orb, &ev);

    /* Clean & exit */
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Server ORB exited");
    return NULL;
}

/* Starts the dripencoder, and hooks to it's Corba orb */
gboolean
corba_init() {
    gint argc = 1;
    gchar *argv[2];
    //GString *iorstring = g_string_new("");
    gchar *buf;
    struct stat file_stats;
    GString *iorstring = g_string_new("");
    gint fd;
    gint tries = 0;

    argv[0] = (gchar*)"gnomedrip";
    argv[1] = 0;
    unlink("dripgui.ior");

    /* Debug */
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Corba init, pid = %i",getpid());

    /* init orb */
    CORBA_exception_init(&ev);

    gdk_threads_enter();
    //DISABLED; orb = gnome_CORBA_init("gnomedrip",VERSION,&argc, argv, 0, &ev);
    orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    gdk_threads_leave();
    /* Disable gnorba authentication meganism */
    ORBit_set_request_validation_handler(drip_ORBit_request_validate);
    ORBit_set_default_principal(&_gnorba_request_cookie);

    /* start server orb */
    serverorbthread_id = pthread_create(&serverorbthread,NULL,start_gui_server_orb,(gpointer)&ev);
    while (!server_orb) {
        /* Wait for server orb to be reeady */
        usleep(10000);
    }
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Server orb ready");

    /* connect to drip_encoder */
    fd = open("dripencoder.ior",O_RDONLY);
    g_print("Connecting to dripencoder...");
    while (tries<15 && fd<0) {
        g_print(".");
        usleep(1000000);
        close(fd);
        fd = open("dripencoder.ior",O_RDONLY);
        tries++;
    }
    /* Still nothing here? */
    if (fd<0) {
        g_print(" nothing there!\n");
        gdk_threads_enter();
        drip_info("Cannot contact the backend\n\nMake sure you're running the right script to start Drip\n(the 'drip' script) and the dripencoder is running.");
        gdk_threads_leave();
        g_print("Cannot open dripencoder.ior. Make sure you're running the right script to start Drip (the 'drip' script) and the dripencoder is running.\n");
        close(fd);
        exit(1);
    } else {
        g_print("\n");
    }

    /* Setup client orb */
    stat("dripencoder.ior", &file_stats);
    buf = malloc(1000);
    memset(buf,0,1000);
    read(fd,buf,file_stats.st_size);
    close(fd);
    g_string_sprintf(iorstring,"%s",buf);
    iorstring = g_string_truncate(iorstring,file_stats.st_size);
    encoder_client = CORBA_ORB_string_to_object(orb, (CORBA_char*)iorstring->str, &ev);
    if (!encoder_client) {
        g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"Cannot bind to dripencoder");
        return FALSE;
    }

    /* free & exit */
    g_string_free(iorstring,TRUE);
    free(buf);
    return TRUE;
}


/* releases dripencoder and cleans up Corba env */
gboolean
corba_close() {
    CORBA_Object_release(encoder_client, &ev);
    CORBA_Object_release((CORBA_Object)orb, &ev);
    return TRUE;
}


/* Searches the /etc/fstab for either a cdrom or dvd entry 
 * If it finds one it returns the device for it. If not, 
 * default to /dev/cdrom ( a safe bet )
 */
gchar*
detect_dvdrom() {
	GString * device;
        gint result;
	device = g_string_new("");
	
	/* Look for a device in mtab  */
	device = drip_system("grep dvd /etc/fstab | awk '{print $1}'",&result,FALSE);
	if (device->len==0) {
            /* Couldn't find anything. Safe default */
            return "/dev/cdrom";
        } else {
            return device->str;
        }
}

void log_handler(const gchar *log_domain,GLogLevelFlags log_level,const gchar *message,gpointer user_data) {
    /* Handles logging. See glib logging system for more info */
    time_t Time = time(NULL);
    GString *date;
    GString *domain;
    GString *level;
    GString *logstring;
    GString *displaystring;
    gboolean quit;
    gint bytes = 0;
    gchar *last_char;
    GdkColor *colour = NULL;

    /* Check log domain */
    if (strcmp(log_domain,DRIP_LD)!=0) {
        g_error("Got a log for domain %s, which does not belong to the drip log domain.",log_domain);
    }
    /* Format domain */
    domain = g_string_new(log_domain);
    while (domain->len < 8) {
        domain = g_string_append(domain," ");
    }

    /* Handle log_level, strip flags */
    log_level = (GLogLevelFlags)(log_level & 252);
    switch (log_level) {
        case G_LOG_LEVEL_ERROR:      level = g_string_new("Error");
                                     colour = &col_error;
                                     quit = TRUE;
                                     break;
        case G_LOG_LEVEL_CRITICAL:   level = g_string_new("Critical");
                                     colour = &col_critical;
                                     quit = TRUE;
                                     break;
        case G_LOG_LEVEL_WARNING:    level = g_string_new("Warning");
                                     colour = &col_warning;
                                     quit = FALSE;
                                     break;
        case G_LOG_LEVEL_MESSAGE:    level = g_string_new("Message");
                                     colour = &col_message;
                                     quit = FALSE;
                                     break;
        case G_LOG_LEVEL_INFO:       level = g_string_new("Info");
                                     colour = &col_info;
                                     quit = FALSE;
                                     break;
#ifdef DEBUG
        case G_LOG_LEVEL_DEBUG:      level = g_string_new("Debug");
                                     colour = &col_debug;
                                     quit = FALSE;
                                     break;
#else
        case G_LOG_LEVEL_DEBUG:      g_string_free(domain,TRUE);
                                     colour = &col_debug;
                                     return;
                                     break;
#endif
        default:                     level = g_string_new("Unknown");
                                     //colour = &col_unknown;
                                     quit = FALSE;
                                     break;
    }


    while (level->len < 8) {
        level =  g_string_append(level," ");
    }

    /* Format date string */
    date = g_string_new(ctime(&Time));
    date = g_string_truncate(date,(date->len)-1);

    /* Store log entry */
    logstring = g_string_new("");
    g_string_sprintf(logstring,"%s\t| %i\t| %s | %s\n",date->str,linenr,level->str,message);
    displaystring = g_string_new("");
    g_string_sprintf(displaystring,"%s | %s\n",date->str,message);
    last_char = (gchar*)((glong)logstring->str + logstring->len - 2); /* we got 2x \n? */
    if (*last_char == 10) {
        logstring = g_string_truncate(logstring,logstring->len -1);
    }
    bytes = write(log_file,logstring->str,logstring->len); /* write log entry to log file */

    /* Write criticals to tty and new dialog window */
    if (log_level == G_LOG_LEVEL_CRITICAL) {
        gdk_threads_enter();
        drip_info(logstring->str);
        g_print(logstring->str);
        gdk_threads_leave();
    }

    /* Log to GUI */
    if ( (textscreen!=NULL) && (corba==corba) && (log_level!=G_LOG_LEVEL_DEBUG)) {
        gdk_threads_enter();
        gtk_text_insert (GTK_TEXT(textscreen), logfont, colour, NULL, displaystring->str, -1);
        gdk_threads_leave();
    }
    linenr++;

    /* Should we exit Drip? */
    if (quit) {
g_print("sleeping...");
       sleep(10);
g_print(" done\n");
       terminate(TRUE);
    }

    /* Free strings */
    g_string_free(date,TRUE);
    g_string_free(domain,TRUE);
    g_string_free(level,TRUE);
    g_string_free(logstring,TRUE);
    g_string_free(displaystring,TRUE);
    logging_locked = FALSE;
    fflush(NULL);
    return;
}

void
drip_init() {
    GString *configfile;
    GString *HOME;
    GIOChannel *configfile_channel = NULL;
    glong configfile_size;
    guint bytesloaded;
    gint loaderror;
    gchar *buf;
    gint fd;
    gint i;
    glong bytes_to_read;
    struct stat file_stats;
    gboolean true_udf;
    gchar *symlink;
    GString *command = g_string_new("");
    GtkWidget *welcome;
    GtkWidget *druid;

    HOME = g_string_new("");
    /* Initialize all drip defaults */
    status_action = g_string_new("");
    configfile = g_string_new("");
    g_string_sprintf(configfile,"%s/.drip.cfg",getenv("HOME"));
    for (i=0;i<8;i++) { Config.audio_channel_names[i] = g_string_new("");}
    for (i=0;i<32;i++) { Config.subpicture_names[i] = g_string_new("");}

    /* Logging font and colours */
    //logfont = gdk_font_load("-B&H-LucidaTypewriter-Medium-R-Normal-Sans-12-120-75-75-M-70-ISO8859-1");
    logfont = gdk_font_load("-Schumacher-Clean-Medium-R-Normal--14-140-75-75-C-70-ISO646.1991-IRV");
    gdk_color_parse("midnight blue",&col_debug);
    gdk_color_parse("DarkRed",&col_warning);
    gdk_color_parse("tomato3",&col_error);
    gdk_color_parse("Red",&col_critical);
    gdk_color_parse("DarkSlateGrey",&col_info);
    gdk_color_parse("black",&col_message);

    /* Default dvd related settings */
    Config.DVDdevice = g_string_new("");
    Config.DVDsource = g_string_new("");
    Config.DIVXfile = g_string_new("");
    Config.track_dir = g_string_new("");
    Config.CACHElocation = g_string_new("");
    Config.audio_channels_in_dvd = 0;
    Config.subpictures_in_dvd = -1;
    Config.subpicture_stream = -1;
    Config.clutfile = g_string_new("");

    /* DVD volume settings */
    Config.DVDvolumeid = g_string_new("");
    Config.DVDpublisher = g_string_new("");
    Config.DVDpreparer = g_string_new("");
    Config.DVDcreationdate = g_string_new("");
    Config.DVDvolumesize_MB = 0;

    /* Drip logging domain */
    g_string_sprintf(HOME,"%s/.drip.log",getenv("HOME"));
    unlink(HOME->str); /* delete old log file, TODO: log file rotation */
    log_file = open(HOME->str,O_WRONLY|O_CREAT,S_IREAD|S_IWRITE); /* create log file */
    log_handler_drip_id = g_log_set_handler (DRIP_LD, (GLogLevelFlags)(G_LOG_LEVEL_WARNING |G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),log_handler, NULL);
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Activated log handler and created log file %s",HOME->str);
    g_print("Activated log handler and created log file %s\n------------------------------------------\n",HOME->str);

    /* Any saved configuration? */
    fd = open(configfile->str,O_RDONLY);
    if (fd>-1) {
        configfile_channel = g_io_channel_unix_new(fd);
        stat(configfile->str, &file_stats);
        configfile_size = file_stats.st_size;
        if (configfile_size>0) {
            g_log(DRIP_LD,G_LOG_LEVEL_INFO,"Loading saved configuration...");
            /* Read configuration from disk */
            buf = g_malloc0(configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&bytes_to_read,sizeof(glong),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,buf,bytes_to_read,&bytesloaded);
                g_string_sprintf(Config.DVDdevice,"%s",(gchar*)buf);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&bytes_to_read,sizeof(glong),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,buf,bytes_to_read,&bytesloaded);
                g_string_sprintf(Config.DIVXfile,"%s",(gchar*)buf);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.width,sizeof(glong),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.height,sizeof(glong),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.video_bitrate,sizeof(guint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.video_stream,sizeof(guint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.audio_bitrate,sizeof(gint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.audio_stream,sizeof(guint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.codec,sizeof(gint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.cliptop,sizeof(guint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.clipbottom,sizeof(guint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.blurradius,sizeof(guint),&bytesloaded);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.cache,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.eject_dvdrom,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.use_dvddb,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.nice,sizeof(gint),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.full_movie_rip_mode,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.maxsize,sizeof(glong),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&true_udf,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.encode,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&bytes_to_read,sizeof(glong),&bytesloaded);
                loaderror = g_io_channel_read(configfile_channel,buf,bytes_to_read,&bytesloaded);
                g_string_sprintf(Config.DVDsource,"%s",(gchar*)buf);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.useDVDdevice,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.cpus,sizeof(gint),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.done_warning,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&bytes_to_read,sizeof(glong),&bytesloaded);
                loaderror = g_io_channel_read(configfile_channel,buf,bytes_to_read,&bytesloaded);
                g_string_sprintf(Config.CACHElocation,"%s",(gchar*)buf);
                memset(buf,0,configfile_size);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.cache_delete_on_exit,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.create_iso,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.hyper_zoom,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.autoclip,sizeof(gboolean),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.clipright,sizeof(guint),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.clipleft,sizeof(guint),&bytesloaded);
            loaderror = g_io_channel_read(configfile_channel,(gchar*)&Config.honor_aspect,sizeof(gboolean),&bytesloaded);

            g_free(buf);
            g_io_channel_close(configfile_channel);
            close(fd);
        }
    } else {
        /* Default divx related settings */
        g_log(DRIP_LD,G_LOG_LEVEL_INFO,"Setting up default configuration...");
        //g_string_sprintf(Config.DVDdevice,"%s",detect_dvdrom());
        /* Traceback if Config.DVDdevice is a symbolic link */
//        symlink = readlink_malloc(Config.DVDdevice->str);
//       if (symlink!=NULL) { /* DVDdevice is a symlink, use real device instead */
//            g_string_sprintf(command,"Using DVD device %s because %s is a symlink",symlink,Config.DVDdevice->str);
//            g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,command->str);
//            g_string_sprintf(Config.DVDdevice,"%s",symlink);
//            g_free(symlink);
//        }
        g_string_sprintf(Config.DVDsource,"/tmp/");
        g_string_sprintf(Config.DIVXfile,"Drip.avi");
        g_string_sprintf(Config.CACHElocation,"/tmp/");
        Config.width = -1;
        Config.height = -1;
        Config.honor_aspect = TRUE;
        Config.hyper_zoom = TRUE;
        Config.video_bitrate = 1000;
        Config.video_stream = 0;
        Config.audio_bitrate = -1;
        Config.audio_stream = 0;
        Config.codec = 1;
        Config.cliptop = 0;
        Config.clipbottom = 0;
        Config.autoclip = TRUE;
        Config.clipright = 0;
        Config.clipleft = 0;
        Config.blurradius = 0;
        /* Default various other settings */
        Config.cache = TRUE;
        Config.cache_delete_on_exit = TRUE;
        Config.eject_dvdrom = TRUE;
        Config.nice = 1;
        Config.full_movie_rip_mode = TRUE;
        Config.use_dvddb = FALSE;
        Config.maxsize = 697;
        Config.encode = TRUE;
        Config.useDVDdevice = TRUE;
        Config.cpus = 1;
        Config.done_warning = TRUE;
        Config.create_iso = FALSE;

        /* No save config around? -> Welcome user */
        //druid = create_configdruid();
        //gtk_widget_show(druid);
        welcome = create_messagebox_betanotice();
        gtk_widget_show(welcome);
    }

    /* Clean & Exit */
    if (Config.CACHElocation->len==0) {
        /* check if cache location is empty */
        g_string_sprintf(Config.CACHElocation,"./");
    }
    g_string_free(HOME,TRUE);
    g_string_free(configfile,TRUE);
    g_string_free(command,TRUE);
    return;
}


gpointer
corba_hook(gpointer data) {
    /* corba hook */
    usleep(100000);

    if (corba_init()==FALSE) {
        g_log(DRIP_LD,G_LOG_LEVEL_ERROR,"Corba could not be initialized, check ORBit and the C++ bindings");
    };

    /* Disable gnorba authentication meganism */
    ORBit_set_request_validation_handler(drip_ORBit_request_validate);
    ORBit_set_default_principal(&_gnorba_request_cookie);

    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Testing Corba, pid = %i",getpid());
    drip_encoder_ping(encoder_client,&ev);

    /* Disable gnorba authentication meganism */
    ORBit_set_request_validation_handler(drip_ORBit_request_validate);
    ORBit_set_default_principal(&_gnorba_request_cookie);

    corba = TRUE;
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Exiting corba init thread");
    //while (1) usleep(1000000); /* DEBUGGIN, GDB EXITS WITHOUT THIS LINE */
    pthread_exit(0);
    return NULL;
}


int
main (int argc, char *argv[]) {
    pthread_t corbahook;
    gint corbahook_id;

#ifdef ENABLE_NLS
    bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
    textdomain (PACKAGE);
#endif

    /* Init's */
    g_thread_init(NULL);
    gtk_init(&argc,&argv);
    gnome_init ("drip", VERSION, argc, argv);
    drip_init();
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Drip %s, pid = %i",VERSION,getpid());

    /* Hook ORBit handlers to glib loop */
    IIOPAddConnectionHandler = orb_add_connection;
    IIOPRemoveConnectionHandler = orb_remove_connection;

    /* Allocate frame window */
    currentframebuffer = g_malloc(1024*1024*3);

    /* Build main windows */
    mainwindow = create_mainwindow();
    gtk_widget_show (mainwindow);
    gtk_widget_set_sensitive(mainwindow,FALSE);

    /* Start (delayed) corba setup */
    corbahook_id = pthread_create(&corbahook,NULL,corba_hook,NULL);
    
    /* We can log to the GUI now, show some info */
    g_log(DRIP_LD,G_LOG_LEVEL_MESSAGE,"Welcome to Drip %s",VERSION);

    /* Disable gnorba authentication meganism */
    ORBit_set_request_validation_handler(drip_ORBit_request_validate);
    ORBit_set_default_principal(&_gnorba_request_cookie);

    /* Start glib */
    g_log(DRIP_LD,G_LOG_LEVEL_DEBUG,"Frontend running inner loop");
    gtk_main();
    return 0;
}

