
/*
 * subpicture.c - DVD subpicture handling - Jarl van Katwijk
 *
 * HOWTO use this code:
 * - call subpicture_init 
 * - call subpicture_render with every mpeg2 package containing subpicture data
 * - call subpicture_overlay with every video frame that needs subpictures
 * this should apply the right subpicture frames for the video ones. 
 * Only 24 bits are supported now, edit overlay.c for other.
 *
 * TODO: add 8/16/32 bits rendering.
 *
 *
 *
 *
 *
 * Mostly based on hard work by:
 *
 * Copyright (C) 2000   Samuel Hocevar <sam@via.ecp.fr>
 *                       and Michel Lespinasse <walken@via.ecp.fr>
 *
 * Lots of rearranging by:
 *	Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *	Thomas Mirlacher <dent@cosy.sbg.ac.at>
 *		implemented reassembling
 *		cleaner implementation of SPU are saving
 *		overlaying (proof of concept for now)
 *		... and yes, it works now with oms
 *		added tranparency (provided by the SPU hdr) 
 *		changed structures for easy porting to MGAs DVD mode
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *                                                     
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <glib.h>
#include <time.h>
#include <signal.h>
#include <setjmp.h>
#include "overlay.h"
#include "spu.h"

#define DISPLAY_INIT
#define REASSEMBLY_START        0
#define REASSEMBLY_MID          1
#define REASSEMBLY_UNNEEDED     2

#define CMD_SPU_MENU            0x00
#define CMD_SPU_SHOW            0x01
#define CMD_SPU_HIDE            0x02
#define CMD_SPU_SET_PALETTE     0x03
#define CMD_SPU_SET_ALPHA       0x04
#define CMD_SPU_SET_SIZE        0x05
#define CMD_SPU_SET_PXD_OFFSET  0x06
#define CMD_SPU_EOF             0xff

// #define DEBUG

typedef guint uint_32;
typedef guchar uint_8;

static struct spu_img_struct spu;
static gint id; /* 0: up  1: down */
static gint aligned;
static gint offset[2];
static guchar *buffer = NULL;
static guint reassembly_buffer_len = 0;
static guint reassembly_buffer_full_len = 0;
static guint reassembly_buffer_data_len = 0;
static guint data_size;
static guint32 x;
static guint32 y;
static guint field;             // which field we are currently decoding
static guint reassembly_flag = REASSEMBLY_START;
struct reassembly_s {
        guint8 *buf;
        guint8 *buf_ptr;        // actual pointer to still empty buffer
        guint buf_len;
        guint cmd_offset;
} reassembly;
gint fd; /* input subpicture filedescriptor */
guint spu_clut[16];
guchar *data_buffer = NULL;
gulong data_buffer_size = 0;
gulong CRClist[65535];
guint CRClist_counter = 0;
glong frames_to_show_current_overlay = 0;
gboolean show_subpicture = FALSE;
gint subpicture_stream = -1;
guint* subpicture_background = NULL;
gint subpicture_x = -1;
gint subpicture_y = -1;
sigjmp_buf environment;
gint iteration_spuDrawPixel = 0;
gfloat framerate;

static u_int _get_bits (u_int bits, struct spu_img_struct *spu)
{
        static u_int data;
	static u_int bits_left;
	u_int ret = 0;

	if (!bits) {	// for realignment to next byte
		bits_left = 0;
	}

	while (bits) {
		if (bits > bits_left) {
			ret |= data << (bits - bits_left);
			bits -= bits_left;

			data = reassembly.buf[spu->offset[field]++];
			bits_left = 8;
		} else {
			bits_left -= bits;
			ret |= data >> (bits_left);
			data &= (1 << bits_left) - 1;
			bits = 0;
		}
	}

	return ret;	
}


void spuInit (void)
{
}	


/* Expand\Draw a colourId+len to SPU render buffer */
static inline void _spu_put_pixel (struct spu_img_struct *spu, u_int len, guint8 colorid,gboolean back)
{
	guint8 *spu_data_ptr = &spu->data[spu->_x + spu->_y * spu->width];
        guint pixel;
	
	spu->_x += len;

        if (back==TRUE) {
            /* Use colour from CLUT and transparency to build a SPU pixel */
            pixel = (spu->trans[colorid])*16 + (spu->clut_index[colorid]);
        } else {
            /* Draw a 100% transparent pixel so nothing is drawn */
            pixel = 0x0000;
        }

        //printf("Setting spu buffer pixel %x len %i (spuclut=%x, trans=%x, colourid=%i)\n",pixel,len,spu->clut_index[colorid],spu->trans[colorid],colorid);
        /* Set pixels */
	memset (spu_data_ptr, pixel, len);

	return;
}


static int _spu_next_line (struct spu_img_struct *spu)
{
	_get_bits (0, spu); // byte align rle data
	
	spu->_x = 0;
	spu->_y++;
	field = (field+1) & 0x01; // Toggle fields
	
	if (spu->_y >= spu->height) {
		return -1;
	}

	return 0;
}


static struct reassembly_s *_reassembly (guint8 *pkt_data, u_int pkt_len) {
	if (reassembly_flag == REASSEMBLY_UNNEEDED)
		reassembly_flag = REASSEMBLY_START;

	if (reassembly_flag == REASSEMBLY_START) {
		reassembly.buf_len = (((u_int)pkt_data[0])<<8) | pkt_data[1];
		reassembly.cmd_offset = (((u_int)pkt_data[2])<<8) | pkt_data[3];

		if (pkt_len >= reassembly.buf_len) {
			reassembly.buf = pkt_data;
			reassembly_flag = REASSEMBLY_UNNEEDED;
			return &reassembly;
		} else {
			reassembly.buf_ptr = reassembly.buf;

			memcpy (reassembly.buf_ptr, pkt_data, pkt_len);
			reassembly.buf_ptr += pkt_len;
			reassembly_flag = REASSEMBLY_MID;
		}
	} else {
		if ((reassembly.buf_ptr+pkt_len) > (reassembly.buf+reassembly.buf_len))
			pkt_len = reassembly.buf_len - (reassembly.buf_ptr - reassembly.buf);


		memcpy (reassembly.buf_ptr, pkt_data, pkt_len);
		reassembly.buf_ptr += pkt_len;

		if (reassembly.buf_ptr >= (reassembly.buf+reassembly.buf_len)) {
			reassembly_flag = REASSEMBLY_START;
			return &reassembly;
		}
	}
	
	return NULL;	
}


/* The time is given as an offset from the presentation time stamp
   and it is measured in number of fields. If we play a NTSC movie
   the time for each field is 1/(2*29.97) seconds. */

int spuParseHdr (struct spu_img_struct *spu, guint8 *pkt_data, u_int pkt_len)
{
	struct reassembly_s *reassembly;
	guint8 *buf; 
	u_int DCSQ_offset, prev_DCSQ_offset = -1;
    u_int i = 0;

	if (!(reassembly = _reassembly (pkt_data, pkt_len)))
		return -1;

	buf = reassembly->buf;
	DCSQ_offset = reassembly->cmd_offset;
	
    while (DCSQ_offset != prev_DCSQ_offset) { /* Display Control Sequences */

		i = DCSQ_offset;

		spu->time_execute = /* PTS + */ ((buf[i] << 8) + buf[i+1]) * 1000 * 1.0/(2*framerate);
		i += 2;
		
		prev_DCSQ_offset = DCSQ_offset;
		DCSQ_offset = (buf[i] << 8) + buf[i+1];
		i += 2;
		
		while (buf[i] != CMD_SPU_EOF) {		// Command Sequence
			switch (buf[i]) {
			case CMD_SPU_SHOW:		// show subpicture
                                //DONT HONOR; show_subpicture = TRUE;
				i++;
				break;
				
			case CMD_SPU_HIDE:		// hide subpicture
                                //DONT HONOR THIS; show_subpicture = FALSE;
				i++;
				break;
				
			case CMD_SPU_SET_PALETTE: {	// CLUT
				spu_clut_t *clut =  (spu_clut_t *) &buf[i+1];
				spu->clut_index[0] = clut->entry0;
				spu->clut_index[1] = clut->entry1;
				spu->clut_index[2] = clut->entry2;
				spu->clut_index[3] = clut->entry3;
				i += 3;
#ifdef DEBUG
printf("SPU SET PALETTE : %i, %i, %i, %i\n",spu->clut_index[0],spu->clut_index[1],spu->clut_index[2],spu->clut_index[3]);
#endif
				break;
			}	
			case CMD_SPU_SET_ALPHA:	{	// transparency palette
				spu_clut_t *trans = (spu_clut_t *) &buf[i+1];
                                if (trans->entry0!=0 || trans->entry1!=0 || trans->entry2!=0 || trans->entry3!=0) {
				spu->trans[0] = trans->entry0;
				spu->trans[1] = trans->entry1;
				spu->trans[2] = trans->entry2;
				spu->trans[3] = trans->entry3;
#ifdef DEBUG
printf("SPU SET TRANSPARENCY : %i, %i, %i, %i\n",spu->trans[0],spu->trans[1],spu->trans[2],spu->trans[3]);
#endif
                                }
				i += 3;
				break;
			}
	
			case CMD_SPU_SET_SIZE:		// image coordinates
				spu->x = (buf[i+1] << 4) |
					(buf[i+2] >> 4);
				spu->width = (((buf[i+2] & 0x0f) << 8) |
					buf[i+3]) - spu->x + 1; // 1-720

				spu->y = (buf[i+4]  << 4) |
					(buf[i+5] >> 4);
				spu->height = (((buf[i+5] & 0x0f) << 8)
					| buf[i+6]) - spu->y + 1; // 1-576


				/* Private stuff */
				spu->_x = spu->_y = 0;
				i += 7;
				break;
				
			case CMD_SPU_SET_PXD_OFFSET:	// image 1 / image 2 offsets
				spu->offset[0] = (((u_int)buf[i+1]) << 8) | buf[i+2];
				spu->offset[1] = (((u_int)buf[i+3]) << 8) | buf[i+4];
				i += 5;
				break;
				
/*
			case CMD_SPU_MENU:
				spu->clut_index[0] = 0;
				spu->clut_index[1] = 9;
				spu->clut_index[2] = 8;
				spu->clut_index[3] = 12;
				spu->trans[0] = 0;
				spu->trans[1] = spu->trans[2] = spu->trans[3] = 15;
				i++;
				break;
*/
			default:
				i++;
				break;
			}
		}
		i++; // lose the CMD_SPU_EOF code (no need to, really)
		
		/* Until we change the interface we parse all 'Command Sequence's 
		   but just overwrite the data in spu. Should be a list instead. */
	}
	
	/* Here we should have a linked list of display commands ready to 
	   be decoded/executed by later calling some spu???() */

	return 0;
}


void spuParseData (struct spu_img_struct *spu)
{
        u_int color;
        u_int len;
        u_int vlc;
	field = 0;
	_get_bits (0, spu);	// Reset/init bit code
	
	while ((spu->offset[1] < reassembly.cmd_offset)) {

		vlc = _get_bits (4, spu);
		if (vlc < 0x0004) {
			vlc = (vlc << 4) | _get_bits (4, spu);
			if (vlc < 0x0010) {
				vlc = (vlc << 4) | _get_bits (4, spu);
				if (vlc < 0x0040) {
					vlc = (vlc << 4) | _get_bits (4, spu);
				}
			}
		}

		color = vlc & 0x03;
		len = vlc>>2;

		// if len == 0 -> end sequence - fill to end of line
		len = len ? : spu->width - spu->_x;
		
		_spu_put_pixel (spu, len, color,TRUE);

		if (spu->_x >= spu->width)
			if (_spu_next_line (spu) < 0)
				goto clean_up;
	}

	/* Like the eof-line escape, fill the rest of the sp. with background */
	_spu_put_pixel (spu, spu->width - spu->_x, color,FALSE); //0
	while (!_spu_next_line (spu)) {
		_spu_put_pixel (spu, spu->width - spu->_x, color,FALSE); //0
	}

clean_up:
	reassembly_flag = REASSEMBLY_START;
}

/*
 * This function will be called during the processing of the input mpeg2 stream. 
 * Every call the background frame will be overlayed by the subpicture, and the
 * counter 'frames_to_show_current_overlay' will be lowered. When this counter
 * reaches 0, overlayingi the pubpicture to the nackground will NOT be done anymore.
 * The subpicture frames that are used are rendered by 'subpicture_render(guchar*)'
 */
void subpicture_overlay(u_int* bg_frame,uint x,uint y,uint o) {
    if (show_subpicture==TRUE) {
        bg_frame = overlay (bg_frame, x, y, o, &spu, 0.85);
        frames_to_show_current_overlay--;
        if (frames_to_show_current_overlay<0) {
            show_subpicture=FALSE;
        }
    }

    return;
}


/*
 * This function is called by the mpeg2 demuxer: every subpicture package is passed on here.
 * Only the prefered subpicture stream is used (language) to fill the local buffer. Once the
 * buffer is filled with enough data to render the next subpicture frame, this is done and 
 * the function subpicture_overlay(...) will automatically use it to apply to the background
 * frames.
 */
void subpicture_render(u_char* data) {
    u_int pkt_len;
    u_char* data_current;
    int i;
    glong CRC = 0;
    u_int data_size_total;
    u_char *data_buffer_temp = NULL;
        
    /* Check for language, based on values from subpicture_init */
    if (subpicture_stream != (data[23 + data[22]])) {
        /* not the prefered language, leave */
        return;
    }

    /* Calculate a CRC value for this package. CRC's are used to determine whether or not a 
     * package has already been processed. Somehow packages arrive multiple times here... */
    //TODO: CRC value too simple?? 
    for (i=0;i<(2048);i++) {
        CRC+=data[i];
    }

    /* CRC check current package */
    for (i=0;i<CRClist_counter;i++) {
        if (CRC==CRClist[i]) {
            //printf("CRC hit, not using this package\n");
            return;
        }
    }

    /* CRC passed, put current CRC value in history */
    CRClist[CRClist_counter] = CRC;
    CRClist_counter++;
    /* Handle overflow: move last 1000 CRC value to begin and reset counter */
    if (CRClist_counter==65535) {
        //printf("Resetting crc counter\n");
        CRClist_counter = 1000;
        for (i=0;i<1000;i++) {
            CRClist[i]=CRClist[i+64535];
        }
        /* NOTE: not cleaning of remaining entries, not needed due to counter */
    } 


    /* Add new data to data_buffer & recalc data_buffer_size */
    memcpy(data_buffer+data_buffer_size, data+24+data[22], 2048-(24+data[22]) );
    data_buffer_size += 2048-(24+data[22]);

    /* parse off the padding bytes */
    data_current = data_buffer;
    pkt_len = data_current[0];
    while((pkt_len == 0xff) && (data_current<(data_buffer+data_buffer_size))) {
        data_current++;
        pkt_len = data_current[0];
    }
    if (pkt_len == 0xff) {
        printf("SPU buffer overflow, resetting buffer and discarding package\n");
        memset(data_buffer,0,65535);
        data_buffer_size = 0;
        return;
    }

    /* Packet length */
    pkt_len = (pkt_len << 8) + data_current[1];
    if (pkt_len == 0x0) {
        printf("SPU pkt_len = 0, resetting buffer and discarding package\n");
        memset(data_buffer,0,2000000);
        data_buffer_size = 0;
        return;
    } else if (pkt_len > 30000) {
        printf("SPU pkt_len > 30000, resetting buffer and discarding package\n");
        memset(data_buffer,0,2000000);
        data_buffer_size = 0;
        return;
    }

    /* do we have enough data buffered? */
    if ((pkt_len+6)>data_buffer_size) {
        /* No, return. Package reassembling will continue with next spu */
        return;
    }

    /* Render new subpicture frame */
    show_subpicture = TRUE; /* default to SHOW when a new subpicture is ready.. TODO:???? */
    data_size = (((unsigned int)data_current[2]) << 8) + data_current[3];
    data_size_total = (data_current[data_size+2] << 8) + data_current[data_size+3];
    if (data_size+24 != data_size_total) {
        printf("SPU datasize's mismatch (data_size=%i, data_size_total=%i), resetting buffer\n",data_size,data_size_total);
        /* reset buffer */
        memset(data_buffer,0,2000000);
        data_buffer_size = 0;
        return;
    } else {
        /* Process package */
        memset(spu.data,0,2000000);
        if (!spuParseHdr (&spu, data_current, pkt_len)) {
            spuParseData (&spu);
        }
        frames_to_show_current_overlay = spu.time_execute * (1/(framerate*3)); /* displayed subpicture frames */
    }

    /* Update data_buffer & size */
    i = pkt_len+6;
    while (data_buffer[i] == 0xff) {
        i++;
    }

    /* temp: clean buffer */
    if (data_buffer_size<i) {
        /* more to clear as we got? -> reset buffer */
        printf("SPU buffer too small to clear, more to clear as is actually there? Resetting buffer\n");
        memset(data_buffer,0,2000000);
        data_buffer_size = 0;
        return;
    }
    /* Rotate buffer */
    g_memmove(data_buffer, data_buffer+i, data_buffer_size-i );
    data_buffer_size -= i;

    return;
}

/* 
 * Initializes the subpicture handling to:
 * - language, valued 0-31 ( the += 0x20 is done here as you can see
 * - width of background frame
 * - height of background frame
 * - pointer to background frame buffer
 * - Framerate
 */
void subpicture_init (gint language, gint sx,gint sy,u_int* background,gfloat FR,gchar* clutfile) {
    gint i;
 
    /* Clear CRC table */
    for (i=0;i<65535;i++) {
        CRClist[i] = 0;
    }

    /* Read CLUT table */
    printf("Opening clut file %s\n",clutfile);
    if (clutfile!=NULL) {
        fd = open(clutfile,O_RDONLY);
        for (i=0;i<16;i++) {
            read(fd,&spu_clut[i],sizeof(u_int));
            #ifdef DEBUG
            printf("Colour %i : %x\n",i,spu_clut[i]);
            #endif
        }
        close(fd);
    } else {
        /* Use static table.. */
        spu_clut[0] = 0xffffff;
        spu_clut[1] = 0xE0E0E0;
        spu_clut[2] = 0xffffff;
        spu_clut[3] = 0x000000;
        spu_clut[4] = 0x005555;
        spu_clut[5] = 0x00ff00;
        spu_clut[6] = 0xcc2255;
        spu_clut[7] = 0xcc0055;
        spu_clut[8] = 0x404040;
        spu_clut[9] = 0x202020;
        spu_clut[10] = 0xb0b0b0;
        spu_clut[11] = 0xd0d0d0;
        spu_clut[12] = 0x606000;
        spu_clut[13] = 0x707000;
        spu_clut[14] = 0x808000;
        spu_clut[15] = 0x606000;
    }

    /* Reset variables */
    CRClist_counter = 0;
    subpicture_stream = language + 0x20;  /* language code + 0x20 */
    subpicture_background = background;   /* pointer to background frame */
    subpicture_x = sx;
    subpicture_y = sy;
    framerate = FR;
    show_subpicture = FALSE;

    /* Allocate buffers */
    if (buffer==NULL) buffer = (u_char*)malloc(2000000); /* holds rendered subpicture frame */
    if (data_buffer==NULL) data_buffer = (u_char*)malloc(2000000);
    if (spu.data==NULL) spu.data = malloc(2000000);
    if (reassembly.buf==NULL) reassembly.buf = malloc(2000000);

    return;
}


